<?php
/**
 * Created by PhpStorm.
 * User: francismararaujo
 * Date: 23/10/14
 * Time: 17:07
 */

namespace TecDiet\Services;

use Doctrine\ORM\EntityManager;
use TecDiet\Util\Validation;

class StructuresService implements Queries {

    protected $entityManager;
    protected $connection;
    protected $structures = array();
    protected $maxLevelPrat;
    protected $cid;
    public function __construct(EntityManager $entityManager) {

        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();

    }

    private function initDataEvaluation() {

        $sth = $this->connection->prepare(self::DATA_EVALUATION);
        $sth->execute();

        $dataEvaluations = $sth->fetchAll(\PDO::FETCH_ASSOC);

        foreach ($dataEvaluations as $dataEvaluation) {

            $this->structures['dataEvaluations'][] = array(
                'dataEvaluationId' => $dataEvaluation['CDTIPODADO'],
                'methodEvaluationId' => $dataEvaluation['CDTBMETOAVAL']
            );

        }

    }
    private function initAntroMethods($unitsID){
        $sth = $this->connection->prepare(self::ANTHROPOMETRIC_METHODS);
        $sth->execute(array(
            'CDFILIAL' => $unitsID
            )
        );

        $AntroMethods = $sth->fetchAll(\PDO::FETCH_ASSOC);

        foreach ($AntroMethods as $AntroMethod) {

            $this->structures['AntroMethods'][] = array(
                'unit' => $unitsID,
                'reference' => $AntroMethod['CDTBREFMETOA'],
                'methodEvaluationId' => $AntroMethod['CDTBMETOAVAL'],
                'name' => $AntroMethod['NMTBMETOAVAL'],
                'dataIni' => intval($AntroMethod['VRIDADEINITB']),
                'dataFim' => intval($AntroMethod['VRIDADEFINTB'])
            );
        }
    }

    private function initDataTypeEvaluation() {

        $sth = $this->connection->prepare(self::DATA_TYPE_EVALUATION);
        $sth->execute();

        $dataTypeEvaluations = $sth->fetchAll(\PDO::FETCH_ASSOC);

        foreach ($dataTypeEvaluations as $dataTypeEvaluation) {
            
            if(isset($dataTypeEvaluation['DSUNIDDADO'])){
                $nameAndUnit = $dataTypeEvaluation['NMTIPODADO'].' (' .$dataTypeEvaluation['DSUNIDDADO'].')';
            }else{
                $nameAndUnit = $dataTypeEvaluation['NMTIPODADO'];
            }
            $this->structures['dataTypeEvaluations'][] = array(
                'dataTypeEvaluationId'   => $dataTypeEvaluation['CDTIPODADO'],
                'dataTypeEvaluationName' => $nameAndUnit
            );

        }

    }

    private function initNormalFeedings($unitsID) {

        $sth = $this->connection->prepare(self::NORMAL_FEEDING_NOW);
        $sth->execute(array('CDFILIAL' => $unitsID));

        $normalFeedings = $sth->fetchAll(\PDO::FETCH_ASSOC);

        foreach ($normalFeedings as $normalFeeding) {

            $this->structures['normalFeedingsNow'][] = array(
                'key' => $normalFeeding['ROWNUM'],
                'unit' => $normalFeeding['CDFILIAL'],
                'menuDate' => $normalFeeding['DTCARDAPIO'],
                'plateName' => $normalFeeding['NMPRATO'],
                'plateAuxId' => $normalFeeding['CDFILAUXPRAT'],
                'formulationId' => $normalFeeding['CDPRATO'],
                'plateType' => $normalFeeding['CDTIPOPRAT'],
                'plateTypeName' => $normalFeeding['NMTIPOPRAT'],
                'optional' => $normalFeeding['IDOPCCNFMES'],
                'idopccnfcard' => $normalFeeding['IDOPCCNFCARD'],
            );             

        }

    }
    
    private function initNutricionist($nutritionistId, $unitsID) {

        $sth = $this->connection->prepare(self::GET_NUTRICIO);
        $sth->execute(array('CDFILIAL' => $unitsID, 'CDOPERADOR' => $nutritionistId));

        $nutritionists = $sth->fetchAll(\PDO::FETCH_ASSOC);
        
        foreach ($nutritionists as $nutritionist) {

            $this->structures['nutritionist'][] = array(
                'id' => $nutritionist['CDNUTRICIO'],
                'unit' => $nutritionist['CDFILIAL'],
                'nrcrnnutrici' => $nutritionist['NRCRNNUTRICI'],
                'active' => $nutritionist['IDATIVO'],
            );             

        }

    }
    
    private function initFoods($unitsID) {
        $cdtabecdipa = $this->cid;
        $sth = $this->connection->prepare(self::MAP_KITCHEN_MAID);
        $sth->execute(array('CDFILIAL' => $unitsID));

        $maps = $sth->fetchAll(\PDO::FETCH_ASSOC);
        
        foreach ($maps as $map) {

            $this->structures['Foods'][] = array(
                'unit' => $map['CDFILIAL'],
                'serviceDate' => $map['DTSERVINTE'],
                'id' => $map['CDPACIENTE'],
                'patientName' => $map['NMPACIENTE'],
                'patientFullName' => $map['NMINTEIRO'],
                'patientBirth' => $map['DTNASCPACI'],
                'patientMotherName' => $map['NMMAEPACIUPP'],
                'serviceId' => $map['CDSERVFILI'],
                'serviceName' => $map['NMSERVICO'],
                'sequential' => $map['NRSEQUSERV'],
                'dietId' => $map['CDDIETPAD'],
                'dietName' => $map['DSDIETPAD'],
                'idformdiet' => $map['IDFORMDIET'],
                'storey' => $map['CDANDAR'],
                'hall' => $map['CDALA'],
                'bed' => $map['CDLEITO'],
                'bedName' => $map['NMLEITO'],
                'situation' => $map['DTHRENTREGA'] !== null && $map['DTHRRECOLHE'] === null ? 'Recolher' : 'Entregar',
                'deliveryDate' => $map['DTHRENTREGA'],
                'deliveryOperator' => $map['CDRESPENTREGA'],
                'collectDate' => $map['DTHRRECOLHE'],
                'collectOperator' => $map['CDRESPRECOLHE'],
                'idtpservhosp' => $map['IDTPSERVHOSP'],
                'solicitante' => Validation::getControlFeedingRequester($map['STPACIACOM']),
                'dietTypeName' => Validation::getControlFeedingServiceName($map['IDFORMDIET']),
                'serviceHourInitial' => $map['HRINICSERV'] ,
                'serviceHourFinal' => $map['HRFINASERV'],
                'cid' => $cdtabecdipa,
            );

        }

    }
    
    private function initDietPlateServ($unitsID) {
        $currentDate = date('d/m/Y');
        $cdtabecdipa = $this->cid;
        $sth = $this->connection->prepare(self::MP_TOT_PRATOSERV);
        $sth->execute(array('P_CDFILIAL' => $unitsID, 'P_CDPACIENTE' => NULL, 'P_CDTABECID' => $cdtabecdipa, 'P_DATA' => $currentDate, 'P_CDSERVFILI' => NULL, 'P_CDALA' => NULL, 'P_CDANDAR' => NULL));
        $maps = $sth->fetchAll(\PDO::FETCH_ASSOC);

        foreach ($maps as $map) {
            $this->structures['DietPlateServ'][] = array(
                'deliveryPoint' => $map['PONTOENTREGA'],
                'unit'          => $map['CDFILIAL'],
                'serviceDate'   => $map['DTSERVINTE'],
                'storey'        => $map['CDANDAR'],
                'hall'          => $map['CDALA'],
                'bed'           => $map['CDLEITO'],
                'cdpaciente'    => $map['CDPACIENTE'],
                'serviceId'     => $map['CDSERVFILI'],
                'codplate'      => $map['CDDIETPAD'],
            );
        }
    }
    
    private function initParamGeral() {
        $sth = $this->connection->prepare(self::GET_PARAMGERAL);
        $sth->execute(array());
        $paramgeral = $sth->fetchAll(\PDO::FETCH_ASSOC);

        foreach ($paramgeral as $param) {
            $this->structures['ParamGeral'][] = array(
                'cdParGeral' => $param['CDPARGERAL'],
            );
        }
    }
    
    private function initDietPlate($unitsID) {
        $currentDate = date('d/m/Y');
        $sth = $this->connection->prepare(self::MP_TOT_DIETPLATE);
        $sth->execute(array('P_CDFILIAL' => $unitsID, 'P_CDSERVFILI' => 'T', 'P_DTSERVINTE' => $currentDate, 'P_CDALA' => 'T', 'P_CDANDAR' => 'T'));
        $maps = $sth->fetchAll(\PDO::FETCH_ASSOC);

        foreach ($maps as $map) {
            $this->structures['DietPlate'][] = array(
                'currentAmout'  => $map['QUANTIDADE'],
                'formulationId' => $map['CDPRATO'],
                'plateName'     => $map['NMPRATO'],
                'plateAuxId'    => $map['CDFILAUXPRAT'],
                'storey'        => $map['CDANDAR'],
                'hall'          => $map['CDALA'],
                'serviceId'     => $map['CDSERVFILI'],
            );
        }
    }
    
    private function initDieta($unitsID) {
        $currentDate = date('d/m/Y');
        $sth = $this->connection->prepare(self::MP_DIET_TOT);
        $sth->execute(array('P_DATAATUAL' => $currentDate, 'P_CDFILIAL' => $unitsID, 'P_CDALA' => 'T', 'P_CDANDAR' => 'T'));

        $maps = $sth->fetchAll(\PDO::FETCH_ASSOC);

        foreach ($maps as $map) {
            $this->structures['Dieta'][] = array(
                'hall'          => $map['CDALA'],
                'hallName'      => $map['NMALA'],
                'storey'        => $map['CDANDAR'],
                'storeyName'    => $map['NMANDAR'],
                'unit'          => $map['CDFILIAL'],
                'serviceId'     => $map['CDSERVFILI'],
                'serviceName'   => $map['NMSERVICO'],
                'serviceDate'   => $map['DTSERVINTE']
            );
        }
    }

    private function initExaminations() {

        $sth = $this->connection->prepare(self::EXAMINATIONS);
        $sth->execute();

        $examinations = $sth->fetchAll(\PDO::FETCH_ASSOC);

        foreach ($examinations as $examination) {

            $this->structures['examinations'][] = array(
                'examinationId' => $examination['CDEXAME'],
                'examinationName' => $examination['NMEXAME'],
                'examinationUnit' => $examination['CDUNIDEXE'],
                'examinationFormula' => $examination['TXFORMEXAM']
            );

        }

    }

    private function initAccessRoads() {

        $sth = $this->connection->prepare(self::ACCESS_ROADS);
        $sth->execute();

        $roads = $sth->fetchAll(\PDO::FETCH_ASSOC);

        foreach ($roads as $road) {

            $this->structures['accessRoads'][] = array(
                'accessRoadId' => $road['CDVIACESDIET'],
                'accessRoadName' => $road['NMVIACESDIET']
            );

        }

    }

    private function initHealthInsurance() {

        $sth = $this->connection->prepare(self::HEALTH_INSURANCE);
        $sth->execute();

        $companies = $sth->fetchAll(\PDO::FETCH_ASSOC);

        foreach ($companies as $company) {

            $this->structures['healthInsurance'][] = array(
                'healthInsuranceId' => $company['CDSERPADDIE'],
                'healthInsuranceName' => $company['NMSERPADDIE']
            );

        }

    }

    private function initNutritionalStatus() {

        $sth = $this->connection->prepare(self::NUTRITIONAL_STATUS);
        $sth->execute();

        $nutritionalStatus = $sth->fetchAll(\PDO::FETCH_ASSOC);

        foreach ($nutritionalStatus as $status) {

            $this->structures['nutritionalStatus'][] = array(
                'nutritionalStatusId' => $status['CDNIASNUTR'],
                'nutritionalStatusName' => $status['NMNIASNUTR']
            );

        }

    }

    private function initServices($unitsID) {

        $sth = $this->connection->prepare(self::SERVICES);
        $sth->execute(
            array(
                'CDFILIAL' => $unitsID
            )
        );

        $services = $sth->fetchAll(\PDO::FETCH_ASSOC);

        foreach ($services as $service) {

            $this->structures['services'][] = array(
                'unit' => $service['CDFILIAL'],
                'service' => $service['IDTPSERVHOSP'],
                'serviceType' => $service['IDTIPOSERV'],
                'serviceId' => $service['CDSERVFILI'],
                'serviceName' => $service['NMSERVICO']
            );

        }

    }
    
    private function initOralServices($unitsID) {

        $sth = $this->connection->prepare(self::ORAL_SERVICES);
        $sth->execute(
            array(
                'CDFILIAL' => $unitsID,
                'TCDSERVFILI' => 'T'
            )
        );

        $oralServices = $sth->fetchAll(\PDO::FETCH_ASSOC);

        foreach ($oralServices as $service) {

            $this->structures['oralServices'][] = array(
                'unit' => $service['CDFILIAL'],
                'service' => $service['IDTPSERVHOSP'],
                'serviceType' => $service['IDTIPOSERV'],
                'serviceId' => $service['CDSERVFILI'],
                'serviceName' => $service['NMSERVICO']
            );

        }

    }

    private function initLactaryServices($unitsID) {
        
        $sth = $this->connection->prepare(self::LACTARY_SERVICES);
        $sth->execute(
            array(
                'CDFILIAL' => $unitsID
            )
        );
        
        $lactaryServices = $sth->fetchAll(\PDO::FETCH_ASSOC);
        
        foreach ($lactaryServices as $service) {

            $this->structures['lactaryServices'][] = array(
                'unit' => $service['CDFILIAL'],
                'service' => $service['IDTPSERVHOSP'],
                'serviceType' => $service['IDTIPOSERV'],
                'serviceId' => $service['CDSERVFILI'],
                'serviceName' => $service['NMSERVICO']
            );

        }

    }
    
    private function initEnteralServices($unitsID) {

        $sth = $this->connection->prepare(self::ENTERAL_SERVICES);
        $sth->execute(
            array(
                'CDFILIAL' => $unitsID
            )
        );

        $enteralServices = $sth->fetchAll(\PDO::FETCH_ASSOC);
        
        foreach ($enteralServices as $service) {

            $this->structures['enteralServices'][] = array(
                'unit' => $service['CDFILIAL'],
                'service' => $service['IDTPSERVHOSP'],
                'serviceType' => $service['IDTIPOSERV'],
                'serviceId' => $service['CDSERVFILI'],
                'serviceName' => $service['NMSERVICO']
            );

        }

    }

    private function getFormulations($unitsID) {

        $sth = $this->connection->prepare(self::FORMULATION);
        
        $sth->execute(
            array(
                'CDFILIAL' => $unitsID
            )
        );

        return $sth->fetchAll(\PDO::FETCH_ASSOC);

    }

    private function getMedicalTeams($unitsID) {

        $sth = $this->connection->prepare(self::MEDICAL_TEAMS);
        $sth->execute(
            array(
                'CDFILIAL' => $unitsID
            )
        );

        return $sth->fetchAll(\PDO::FETCH_ASSOC);

    }

    private function getDietsServices($unitsID){

        $sth = $this->connection->prepare(self::DIETS_SERVICES);
        $sth->execute(
            array(
                'CDFILIAL' => $unitsID
            )
        );

        return $sth->fetchAll(\PDO::FETCH_ASSOC);

    }

    private function getDiets($unitsID) {

        $sth = $this->connection->prepare(self::DIETS);
        $sth->execute();

        return $sth->fetchAll(\PDO::FETCH_ASSOC);

    }

    private function getDietsFields($unitsID) {

        $sth = $this->connection->prepare(self::DIETS_FIELDS);
        $sth->execute(
            array(
                'CDFILIAL' => $unitsID
            )
        );

        return $sth->fetchAll(\PDO::FETCH_ASSOC);

    }

    private function getBeds($unitsID, $nutritionistId, $has) {
        if($has){
            $sth = $this->connection->prepare(self::NUTRITIONIST_BEDS);
            $sth->execute(
                array(
                    'CDFILIAL'  => $unitsID,
                    'CDNUTRICIO'=> $nutritionistId
                )
            );

        }else{
            $sth = $this->connection->prepare(self::BEDS);
            $sth->execute(
                array(
                    'CDFILIAL' => $unitsID
                )
            );
    
        }
        
        $beds = $sth->fetchAll(\PDO::FETCH_ASSOC);
        return $beds;
    }

    private function getBedsPrescriptions($unitsID, $nutritionistId, $has) {
        if($has){
            $sth = $this->connection->prepare(self::NUTRITIONIST_BEDS_PRESCRIPTIONS);
            $sth->execute(
                array(
                    'CDFILIAL' => $unitsID,
                    'CDNUTRICIO'=> $nutritionistId
                )
            );
            
        }else{
            $sth = $this->connection->prepare(self::BEDS_PRESCRIPTIONS);
            $sth->execute(
                array(
                    'CDFILIAL' => $unitsID
                )
            );
        }
        $beds = $sth->fetchAll(\PDO::FETCH_ASSOC);
        return $beds;
    }
    
    private function hasParams($nutritionistId, $unitsID){
        $sth = $this->connection->prepare(self::HAS_NUTRITIONIST_PARAMS);
        $sth->execute(
            array(
                'CDFILIAL'  => $unitsID,
                'CDNUTRICIO'=> $nutritionistId
            )
        );
        $has = $sth->fetchAll(\PDO::FETCH_ASSOC);
        return !!$has;
    }

    private function getHalls($unitsID, $nutritionistId, $has) {
        if($has){
            $sth = $this->connection->prepare(self::NUTRITIONIST_HALLS);
            $sth->execute(
                array(
                    'CDFILIAL'  => $unitsID,
                    'CDNUTRICIO'=> $nutritionistId
                )
            );
    
        }else{
            $sth = $this->connection->prepare(self::HALLS);
            $sth->execute(
                array(
                    'CDFILIAL' => $unitsID
                )
            );
        }

        $halls = $sth->fetchAll(\PDO::FETCH_ASSOC);
        return $halls;

    }
    
    private function getHallsDistincts($unitsID, $nutritionistId, $has) {
        if($has){
            $sth = $this->connection->prepare(self::NUTRITIONIST_HALLS_DISTINCT);
            $sth->execute(
                array(
                    'CDFILIAL' => $unitsID,
                    'CDNUTRICIO'=> $nutritionistId
                )
            );

        }else{
            $sth = $this->connection->prepare(self::HALLS_DISTINCT);
            $sth->execute(
                array(
                    'CDFILIAL' => $unitsID
                )
            );
        }
        
        $halls = $sth->fetchAll(\PDO::FETCH_ASSOC);
        return $halls;

    }

    private function getStoreys($unitsID, $nutritionistId, $has) {
        if($has){
            $sth = $this->connection->prepare(self::NUTRITIONIST_STOREYS);
            $sth->execute(
                array(
                    'CDFILIAL'  => $unitsID,
                    'CDNUTRICIO'=> $nutritionistId
                )
            );

        }else{
            $sth = $this->connection->prepare(self::STOREYS);
            $sth->execute(
                array(
                    'CDFILIAL' => $unitsID
                )
            );
        }
        
        $storeys = $sth->fetchAll(\PDO::FETCH_ASSOC);    
        return $storeys;
    }
    
    private function getPlates($unitsID) {

        $sth = $this->connection->prepare(self::ALL_PLATES);
        $sth->execute(
            array(
                'CDFILIAL' => $unitsID
            )
        );

        return $sth->fetchAll(\PDO::FETCH_ASSOC);

    }
    
    private function getPlatesByUnit($unitsID){
        $sth = $this->connection->prepare(self::ALL_PLATES_BY_UNIT);
        $sth->execute(
            array(
                'CDFILIAL' => $unitsID
            )
        );
        
        return $sth->fetchAll(\PDO::FETCH_ASSOC);
    }
    
    private function getPlatesPla($unitsID){
        $sth = $this->connection->prepare(self::ALL_PLATES_PLA);
        $sth->execute(
            array(
                'CDFILIAL' => $unitsID
            )
        );
        
        return $sth->fetchAll(\PDO::FETCH_ASSOC);
    }

    private function getLactaryPlates($unitsID) {
        
        $sth = $this->connection->prepare(self::LACTARY_PLATES);
        $sth->execute(
            array(
                'CDFILIAL' => $unitsID,
                'CDNVPRATO' => $this->maxLevelPrat
            )
        );

        return $sth->fetchAll(\PDO::FETCH_ASSOC);

    }
    
    private function getFoodIntoleranceGroup($unitsID) {
        
        $sth = $this->connection->prepare(self::FOOD_INTOLERANCE_GROUP);
        $sth->execute(
            array(
                'P_CDFILIAL' => $unitsID
            )
        );
        
        return $sth->fetchAll(\PDO::FETCH_ASSOC);
    }
    
    private function getAllGroupSubgroup() {
        
        $sth = $this->connection->prepare(self::ALL_GROUP_SUBGROUP);
        $sth->execute();
        
        return $sth->fetchAll(\PDO::FETCH_ASSOC);
    }
    
    private function initStructure($nutritionistId, $unitsID) {
        $hasNutritionistParams = $this->hasParams($nutritionistId, $unitsID);
        
        $plates = $this->getPlates($unitsID);
        
        foreach ($plates as $plate) {
            $isRequiredOnMenu = $plate['IDOPCCNFMES'] === 'N' ? true : false;
            $this->structures['plates'][] = array(
                'id' => $plate['ROWNUM'],
                'unit' => $plate['CDFILIAL'],
                'codplate' => $plate['CDDIETPAD'],
                'dietid' => $plate['DIETID'],
                'dietName' => $plate['DSDIETPAD'],
                'dtCard' => $plate['DTCARDAPIO'],
                'formulationId' => $plate['CDPRATO'],
                'plateName' => $plate['NMPRATO'],
                'plateType' => $plate['NMTIPOPRAT'],
                'cdtipoprat' => $plate['CDTIPOPRAT'],
                'isRequiredOnMenu' => $isRequiredOnMenu,
                'cdservfili' => $plate['CDSERVFILI'],
                'NRORDPRATCAR' => $plate['NRORDPRATCAR']
            );

        }
        
        $foodIntoleranceGroup = $this->getFoodIntoleranceGroup($unitsID);
        foreach ($foodIntoleranceGroup as $group){
            $this->structures['foodGroup'][] = array(
                'cdfilial'=> $group['CDFILIAL'],
                'cdpaciente'=> $group['CDPACIENTE'],
                'cdgrupprod'=> $group['CDGRUPPROD'],
                'cdsubgrprod'=> $group['CDSUBGRPROD'],
                'nmgrupprod'=> $group['NMGRUPPROD'],
                'nmsubgrprod'=> $group['NMSUBGRPROD'],
                'nmgroupsub' =>$group['NMGROUPSUB']
            );
        }
        
        $allGroupSubgroup = $this->getAllGroupSubgroup();
        foreach ($allGroupSubgroup as $currentGroupSubgroup){
            $this->structures['allGroupSubgroup'][] = array(
                'nmgrupprod' =>$currentGroupSubgroup['NMGRUPPROD'],
                'nmsubgrprod'=>$currentGroupSubgroup['NMSUBGRPROD'],
                'nmgroupsub' =>$currentGroupSubgroup['NMGROUPSUB'],
                'cdgrupprod'=> $currentGroupSubgroup['CDGRUPPROD'],
                'cdsubgrprod'=> $currentGroupSubgroup['CDSUBGRPROD']
                
            );
        }
        
        $platesByUnit = $this->getPlatesByUnit($unitsID);
        
        foreach ($platesByUnit as $plate){
            $this->structures['platesByUnit'][] = array(
                'formulationId' => $plate['CDPRATO'],
                'plateName' => $plate['NMPRATO'],
                'plateType'=>$plate['CDTIPOPRAT'],
                'plateTypeName'=>$plate['NMTIPOPRAT'],
                'nmpratipo' => $plate['NMPRATIPO'],
                'cdservfili'=>$plate['CDSERVFILI']
            );
        }
        
        $platesPla = $this->getPlatesPla($unitsID);
        
        foreach ($platesPla as $plate){
            $isRequiredOnMenu = $plate['IDOPCCNFMES'] === 'N' ? true : false;
            $this->structures['platesPla'][] = array(
                'id' => $plate['ROWNUM'],
                'unit' => $plate['CDFILIAL'],
                'dtCard' => $plate['DTCARDAPIO'],
                'formulationId' => $plate['CDPRATO'],
                'plateName' => $plate['NMPRATO'],
                'plateType' => $plate['NMTIPOPRAT'],
                'cdtipoprat' => $plate['CDTIPOPRAT'],
                'isRequiredOnMenu' => $isRequiredOnMenu,
                'cdservfili' => $plate['CDSERVFILI'],
                'NRORDPRATCAR' => $plate['NRORDPRATCAR']
            );
        }

        $lactary_Plates = $this->getLactaryPlates($unitsID);
        
        foreach ($lactary_Plates as $lacPlate) {
            $this->structures['lactaryPlates'][] = array(
                'cdformulacao' => $lacPlate['CDFORMULACAO'],
                'nmformulacao' => $lacPlate['NMFORMULACAO'],
                'normalizedName' => $lacPlate['NORMALIZEDNAME'],
                'un' => $lacPlate['SGUNIDADE']
            );

        }


        $storeys = $this->getStoreys($unitsID, $nutritionistId, $hasNutritionistParams);

        foreach ($storeys as $storey) {

            $this->structures['storeys'][] = array(
                'unit' => $storey['CDFILIAL'],
                'storey' => $storey['CDANDAR'],
                'storeyName' => $storey['NMANDAR']
            );

        }

        $halls = $this->getHalls($unitsID, $nutritionistId, $hasNutritionistParams);

        foreach ($halls as $hall) {

            $this->structures['halls'][] = array(
                'unit' => $hall['CDFILIAL'],
                'storey' => $hall['CDANDAR'],
                'hall' => $hall['CDALA'],
                'hallName' => $hall['NMALA']
            );

        }
        
        $hallsDistincts = $this->getHallsDistincts($unitsID, $nutritionistId, $hasNutritionistParams);

        foreach ($hallsDistincts as $hall) {

            $this->structures['hallsDistincts'][] = array(
                'unit' => $hall['CDFILIAL'],
                'storey' => $hall['CDANDAR'],
                'hall' => $hall['CDALA'],
                'hallName' => $hall['NMALA']
            );

        }

        $beds = $this->getBeds($unitsID, $nutritionistId, $hasNutritionistParams);

        foreach ($beds as $bed) {

            $this->structures['beds'][] = array(
                'unit'          => $bed['CDFILIAL'],
                'storey'        => $bed['CDANDAR'],
                'hall'          => $bed['CDALA'],
                'bed'           => $bed['CDLEITO'],
                'bedName'       => $bed['NMLEITO'],
                'storeyName'    => $bed['NMANDAR'],
                'hallName'      => $bed['NMALA']
            );

        }

        $bedsPrescriptions = $this->getBedsPrescriptions($unitsID, $nutritionistId, $hasNutritionistParams);

        foreach ($bedsPrescriptions as $bedsPrescription) {

            $this->structures['bedsPrescriptions'][] = array(
                'unit' => $bedsPrescription['CDFILIAL'],
                'storeyName' => $bedsPrescription['NMANDAR'],
                'hallName' => $bedsPrescription['NMALA'],
                'bedName' => $bedsPrescription['NMLEITO'],
                'bed' => $bedsPrescription['CDLEITO'],
                'storey' => $bedsPrescription['CDANDAR'],
                'hall' => $bedsPrescription['CDALA']
            );
        }
        
        $dietsServices = $this->getDietsServices($unitsID);

        foreach ($dietsServices as $diet) {

            $this->structures['dietsServices'][] = array(
                'unit' => $diet['CDFILIAL'],
                'service' => $diet['IDFORMDIET'],
                'serviceId' => $diet['CDSERVFILI'],
                'dietId' => $diet['CDDIETPAD'],
                'dietName' => $diet['DSDIETPAD'],
                'dietGroup' => $diet['CDDIETGRUP']
            );

        }

        $diets = $this->getDiets($unitsID);

        foreach ($diets as $diet) {

            $this->structures['diets'][] = array(
                'service' => $diet['IDFORMDIET'],
                'dietId' => $diet['CDDIETPAD'],
                'dietName' => $diet['DSDIETPAD'],
                'dietGroup' => $diet['CDDIETGRUP']
            );

        }

        $medicalTeams = $this->getMedicalTeams($unitsID);

        foreach ($medicalTeams as $medicalTeam) {

            $this->structures['medicalTeams'][] = array(
                'unit' => $medicalTeam['CDFILIAL'],
                'medicalTeamId' => $medicalTeam['CDEQUIMED'],
                'medicalTeamName' => $medicalTeam['NMEQUIMED']
            );

        }

        $formulations = $this->getFormulations($unitsID);

        foreach ($formulations as $formulation) {

            $this->structures['formulations'][] = array(
                'unit' => $formulation['CDFILIAL'],
                'formulationId' => $formulation['CDFORMULACAO'],
                'formulationName' => $formulation['NMFORMULACAO'],
                'normalizedName' => $formulation['NORMALIZEDNAME'],
                'productId' => $formulation['CDPRODUTO'],
                'plateAuxId' => $formulation['CDFILAUXPRAT'],
                'preparationId' => $formulation['IDTIPO']
            );

        }
    }
    public function getPlatesByQuery ($query, $param){
        $sth = $this->connection->prepare($query);
        $sth->execute($param);
        $dataSet = $sth->fetchAll(\PDO::FETCH_ASSOC);
        return $dataSet;
    }
    public function findPlates($params){
        $callback = array();
        $paramUnit = array(
            'UNIT' => $params['CDFILIAL']
        );
        $normalFeedings = !isset($params['type']) || $params['type'] === 'normalFeedings' ? $this->getPlatesByQuery(self::FIND_NORMAL_FEEDINGS, $paramUnit) : array();
        foreach ($normalFeedings as $plate){
            $callback['normalFeedings'][] = array(
                'id' => $plate['CDPACIENTE'],
                'unit' => $plate['CDFILIAL'],
                'serviceId' => '0',
                'key' => $plate['ROWNUM'].$plate['CDPACIENTE'],
                'unique' => $plate['CDSEQUCOPR'],
                'plateName' => $plate['NMPRATO'],
                'plateType' => $plate['CDTIPOPRAT'],
                'plateTypeName' => $plate['NMTIPOPRAT'],
                'serviceDate' => $plate['DTSERVINTE'],
                'plateNumbers' => $plate['QTPRATCONF'],
                'plateAuxId' => $plate['CDFILAUXPRAT'],
                'formulationId' => $plate['CDPRATO'],
                'observation' => $plate['DSOBSCONF'],
                'sequential' => $plate['NRSEQUSERV'],
                'dietId' => $plate['CDDIETPAD'],
                'isRequiredOnMenu' => $plate['IDOPCAOCONF'] === 'N' ? true : false,
                'selectedByUser' => $plate['IDESCOCONF'] == 'S' ? true : false,
                'idrestconf' => $plate['IDRESTCONF']
            );
        }
        $lactaryFeedings = !isset($params['type']) || $params['type'] === 'lactaryFeedings' ? $this->getPlatesByQuery(self::FIND_LACTARY_FEEDING, $paramUnit) : array();
        foreach ($lactaryFeedings as $lactaryFeeding) {
            // $qtpratconf = $lactaryFeeding['QTPRATCONF'][0] === '.' || $lactaryFeeding['QTPRATCONF'][0] === ',' ? '0'.$lactaryFeeding['QTPRATCONF'] : $lactaryFeeding['QTPRATCONF'];
            $callback['lactaryFeedings'][] = array(
                'id' => $lactaryFeeding['CDPACIENTE'],
                'unit' => $lactaryFeeding['CDFILIAL'],
                'serviceId' => '1',
                'key' => $lactaryFeeding['ROWNUM'].$lactaryFeeding['CDPACIENTE'],
                'unique' => $lactaryFeeding['CDSEQUCOPR'],
                'formulationName' => $lactaryFeeding['NMPRATO'],
                'serviceDate' => $lactaryFeeding['DTSERVINTE'],
                'lactaryDate' => $lactaryFeeding['DTCONSCONF'],
                'hourLactaryPlate' => Validation::floatToTime($lactaryFeeding['HRCONSCONF']),
                'plateNumbers' => $lactaryFeeding['QTPRATCONF'],
                'plateAuxId' => $lactaryFeeding['CDFILAUXPRAT'],
                'formulationId' => $lactaryFeeding['CDPRATO'],
                'observation' => $lactaryFeeding['DSOBSCONF'],
                'sequential' => $lactaryFeeding['NRSEQUSERV']
            );
        }
        $enteralFeedings = !isset($params['type']) || $params['type'] === 'enteralFeedings' ? $this->getPlatesByQuery(self::FIND_ENTERAL_FEEDING, $paramUnit) : array();
        foreach ($enteralFeedings as $enteralFeeding) {
            if($enteralFeeding['IDPREPCONF'] == 'A') {
                $formulationName = $enteralFeeding['NMPRATO'];
                $formulationId = $enteralFeeding['CDPRATO'];
            }else {
                $formulationName = $enteralFeeding['NMPRODUTO'];
                $formulationId = $enteralFeeding['CDPRODUTO'];
            }
            // $qtpratconf = $enteralFeeding['QTPRATCONF'][0] === '.' || $enteralFeeding['QTPRATCONF'][0] === ',' ? '0'.$enteralFeeding['QTPRATCONF'] : $enteralFeeding['QTPRATCONF'];
            // $qtinfuconf = $enteralFeeding['QTINFUCONF'][0] === '.' || $enteralFeeding['QTINFUCONF'][0] === ',' ? '0'.$enteralFeeding['QTINFUCONF'] : $enteralFeeding['QTINFUCONF'];
            // $qtservconf = $enteralFeeding['QTSERVCONF'][0] === '.' || $enteralFeeding['QTSERVCONF'][0] === ',' ? '0'.$enteralFeeding['QTSERVCONF'] : $enteralFeeding['QTSERVCONF'];
            $callback['enteralFeedings'][] = array(
                'id' => $enteralFeeding['CDPACIENTE'],
                'unit' => $enteralFeeding['CDFILIAL'],
                'serviceId' => '2',
                'key' => $enteralFeeding['ROWNUM'].$enteralFeeding['CDPACIENTE'],
                'unique' => $enteralFeeding['CDSEQUCOPR'],
                'formulationName' => $formulationName,
                'accessRoadId' => $enteralFeeding['CDVIACESDIET'],
                'accessRoadName' => $enteralFeeding['NMVIACESDIET'],
                'serviceDate' => $enteralFeeding['DTSERVINTE'],
                'preparationId' => $enteralFeeding['IDPREPCONF'],
                'preparationName' => Validation::getPreparationName($enteralFeeding['IDPREPCONF']),
                'unidadeVolume' => $enteralFeeding['UNIDPRA'],
                'enteralDate' => $enteralFeeding['DTCONSCONF'],
                'time' => Validation::floatToTime($enteralFeeding['HRCONSCONF']),
                'volPrescription' => $enteralFeeding['QTPRATCONF'],
                'plateAuxId' => $enteralFeeding['CDFILAUXPRAT'],
                'volInfudido' => $enteralFeeding['QTINFUCONF'],
                'formulationId' => $formulationId,
                'serviceNumbers' => $enteralFeeding['QTSERVCONF'],
                'observation' => $enteralFeeding['DSOBSCONF'],
                'sequential' => $enteralFeeding['NRSEQUSERV']
            );
        }
        return $callback;
    }
    
    private function setMaxLevelPratAndCid(){
        $query = 'SELECT CDPICTPRAT, CDTABECDIPA FROM PARAMGERAL';
        $std = $this->connection->prepare($query);
        $std->execute();
        
        $result = $std->fetch();
        
        if ($result && is_array($result)) {
            $this->maxLevelPrat = sizeof(explode('.', $result['CDPICTPRAT']));
            $this->cid = $result['CDTABECDIPA'];
        } else {
            $this->maxLevelPrat = 0;
            $this->cid = null;
        }
    }
    
    public function get($nutritionistId, $unitsID) {
        $this->setMaxLevelPratAndCid();
        $this->initStructure($nutritionistId, $unitsID);
        
        $this->initNutritionalStatus();
        $this->initHealthInsurance();
        $this->initAccessRoads();
        $this->initAntroMethods($unitsID);
        $this->initExaminations();
        $this->initDataTypeEvaluation();
        $this->initDataEvaluation();
        $this->initServices($unitsID);
        $this->initOralServices($unitsID);
        $this->initLactaryServices($unitsID);
        $this->initEnteralServices($unitsID);
        $this->initNormalFeedings($unitsID);
        $this->initNutricionist($nutritionistId, $unitsID);
        $this->initFoods($unitsID);
        $this->initDieta($unitsID);
        $this->initDietPlate($unitsID);
        $this->initDietPlateServ($unitsID);
        $this->initParamGeral($unitsID);
        return $this->structures;

    }

}