describe("Project", function () {
	var localStorageEngine, requestDataSourceEngine, SQLStorageStrategy, restEngine = {
		requestMetaData: function () {
			return Q.when({});
		}
	};
	
	/*jslint evil: true */
	var eventEngine = new EventEngine(new EventFactory(new MethodBuilder({}, {
		$eval: function (a, b) {
			return eval(a, b);
		}
	}), {}), new EventHandler({}, {}, console), {
		updateTemplate: function () {}
	}, {
		i18n: function (text) {
			return text;
		}
	});
	
	var aclEngine = new AclEngine();
	
	var screenService = {
		isPopupOpen: function () {
			return false;
		}
	};
	
	var baseMenu = function () {
		return 	{
			name: "applications",
			label: "Aplicações",
			icon: "application icon-white",
			hasChild: true,
			isVisible: true
		};
	};
	
	var metaDataFactory = new MetaDataFactory(localStorageEngine, requestDataSourceEngine,
		SQLStorageStrategy, restEngine, eventEngine, screenService, aclEngine, {
			project: {}, updateTemplate: function() {}
		}, new ZhUtil({}), Q, {
			getDataSourceLazy: function () {}
		});
	
	it("Method addMenu should add a groupMenu when there is no parent defined", function() {
		var baseProject = ZhHelper.projectTest();
		
		var project = metaDataFactory.projectFactory(baseProject.zeedhi_project);
		var previousProjectGroupMenus = project.groupMenu.length;
		
		var groupMenu = baseMenu();
		project.addMenu(groupMenu);
		
		expect(project.groupMenu.length).toBe(previousProjectGroupMenus + 1);
	});
	
	it("Method addMenu should add a menu to the specified parent", function() {
		var baseProject = ZhHelper.projectTest();
		
		var project = metaDataFactory.projectFactory(baseProject.zeedhi_project);
		
		var menu = baseMenu();
		
		var previousProjectGroupMenuMenusLength = project.groupMenu[0].menus.length;
		project.addMenu(menu, project.groupMenu[0].name);
		
		expect(project.groupMenu[0].menus.length).toBe(previousProjectGroupMenuMenusLength + 1);
	});
	
	it("Method addMenus should add some menus", function() {
		var baseProject = ZhHelper.projectTest();
		
		var project = metaDataFactory.projectFactory(baseProject.zeedhi_project);
		project.groupMenu[0].menus = null;
		var menus = [];
		
		var menu = baseMenu();
		menu.parent = project.groupMenu[0].name;
		menus.push(menu);
		menu = baseMenu();
		menu.parent = project.groupMenu[0].name;
		menu.name = "secondMenu";
		menus.push(menu);
		
		project.addMenus(menus);
		
		expect(project.groupMenu[0].menus.length).toBe(2);
		expect(project.groupMenu[0].menus[0].name).toBe("applications");
		expect(project.groupMenu[0].menus[1].name).toBe("secondMenu");
	});
	
	it("Method removeMenu should remove a menu or a groupMenu", function() {
		var baseProject = ZhHelper.projectTest();
		
		var project = metaDataFactory.projectFactory(baseProject.zeedhi_project);
		var previousGroupMenuLength = project.groupMenu.length;
		var applicationsMenu = baseMenu();
		applicationsMenu.parent = project.groupMenu[0];
		project.groupMenu[0].menus = [applicationsMenu];
		
		project.removeMenu('applications');
		expect(project.groupMenu[0].menus.length).toBe(0);
		
		project.removeMenu(project.groupMenu[0].name);
		expect(project.groupMenu.length).toBe(previousGroupMenuLength - 1);
	});

	describe("Default timeout", function() {
		var requestFactory;
		var httpCalledCount;
		var httpParameters = [];
		var requestEngine;
		var _templateManager;

		beforeEach(function() {
			module(function($provide) {
				$provide.constant("$q", Q);
			});
			window.templateUrl = '../../dyman/templates/dyman/';
			window.serviceUrl  = '../backend/service/index.php';
	        module('ZeedhiApp');
	        module('ZeedhiServices');

			inject(function(ZhLogger, templateManager, ScreenService, ZhFeatures, ZHPromise, eventAggregator, requestInterceptingEngine) {
				httpCalledCount = 0;
				httpParameters = [];

				var httpMock = function(params) {
					httpCalledCount++;
					httpParameters.push(params);

					var obj = {
						success: function() {return obj;},
						then: function() {return obj;}
					};
					return obj;
				};

				_templateManager = templateManager;
				var _restEngine = new RestEngine(ZhLogger, httpMock, templateManager, ScreenService, ZhFeatures, ZHPromise, eventAggregator, requestInterceptingEngine);
				requestFactory = new RequestFactory(templateManager);
				requestEngine = new RequestEngine(_restEngine);
			});
		});

		it("Timeout must be the framework default timeout(15000)", function() {
			var baseProject = ZhHelper.projectTest();
			_templateManager.project = metaDataFactory.projectFactory(baseProject.zeedhi_project);

			requestEngine.doRequest(requestFactory.factory({
				'serviceName': '/service',
				'requestType': 'row',
				'row': {
					'name': 'Name',
					'value': 1
				}
			}));
			
				expect(httpCalledCount).toBe(1);
				expect(httpParameters[0].timeout).toBe(15000);
		});

		it("Timeout must be the project default timeout(10000)", function() {
			var baseProject = ZhHelper.projectTest();
			baseProject.zeedhi_project.timeout = 10000;
			_templateManager.project = metaDataFactory.projectFactory(baseProject.zeedhi_project);

			requestEngine.doRequest(requestFactory.factory({
				'serviceName': '/service',
				'requestType': 'row',
				'row': {
					'name': 'Name',
					'value': 1
				}
			}));
			
			expect(httpCalledCount).toBe(1);
			expect(httpParameters[0].timeout).toBe(10000);
		});
	});
});