var UnitController = function(ScreenService, UserService, SynchronizeService, localStorageEngine, templateManager, Environment, eventAggregator, restEngine, ZHPromise) {

    this.onEnter = function(widget) {

        widget.reload().then(function() {
            widget.moveToFirst();
        });

    };

    this.openUnitPopup = function() {
        var widgetUnit = templateManager.container.getWidget('WidgetUnit');
        widgetUnit.isVisible = true;
        widgetUnit.edit();
        widgetUnit.search = '';
        widgetUnit.dataSourceFilter = [{
            "name": "CDFILIAL",
            "operator": "<>",
            "value": "@getLocalVar('__CDFILIAL')@"
        }];
        widgetUnit.reload();
        ScreenService.openPopup(widgetUnit);
    };

    this.closeUnitPopup = function(widget) {
        widget.isVisible = false;
        ScreenService.closePopup();
    };

    eventAggregator.subscribe('afterMoveRowUnit', function(container) {
        afterMoveRow(container);
    });

    eventAggregator.subscribe('afterMoveRowUnitSync', function(container) {
        afterMoveRowSync(container);
    });

    eventAggregator.subscribe('openUnitPopup', function(widgetUnit) {
        if (widgetUnit !== undefined) {
            // var widgetUnit = container.getWidget('WidgetUnit');
            widgetUnit.isVisible = true;
            widgetUnit.edit();
            widgetUnit.search = '';
            widgetUnit.dataSourceFilter = [{
                "name": "CDFILIAL",
                "operator": "<>",
                "value": "@getLocalVar('__CDFILIAL')@"
            }];
            widgetUnit.reload();
            ScreenService.openPopup(widgetUnit);
        }
    });

    function afterMoveRow(container) {

        var message = ScreenService.i18n('Deseja confirmar a troca de unidade?');
        ScreenService.confirmMessage(message, 'question', function() {

            var widgetUnit = container.getWidget('WidgetUnit');
            widgetUnit.currentRow.changed = true;
            UserService.save(widgetUnit.currentRow).then(function() {
                SynchronizeService.sync(true).then(function() {

                    ScreenService.showMessage(ScreenService.i18n('Unidade alterada com sucesso!'), 'success');
                    widgetUnit.isVisible = false;
                    ScreenService.closePopup();
                    restEngine.post('/Patients/patientSequentialAjusta', {
                        "requestType": "Row",
                        "row": {
                            'unit': widgetUnit.currentRow.CDFILIAL
                        }
                    }).then(function(callback) {
                        if (callback && callback.dataset && callback.dataset.patientSequentialAjusta) {
                            if (callback.dataset.patientSequentialAjusta.IDCTRLCDPACI) {
                                localStorageEngine.setLocalVar('__ISSEQUENTIAL', callback.dataset.patientSequentialAjusta.IDCTRLCDPACI);
                            }
                            if (callback.dataset.patientSequentialAjusta.IDAJUSTAPACI) {
                                localStorageEngine.setLocalVar('__ISAJUSTA', callback.dataset.patientSequentialAjusta.IDAJUSTAPACI);
                            } else {
                                localStorageEngine.setLocalVar('__ISSEQUENTIAL', null);
                                localStorageEngine.setLocalVar('__ISAJUSTA', null);
                            }
                        }
                    });
                    setTimeout(function() {
                        templateManager.updateTemplate();
                        eventAggregator.publish("reloadScreenAndClearFilters", container.getWidget('WidgetMain'));
                        Environment.setLocalVar('__CDFILIAL', widgetUnit.currentRow.CDFILIAL);
                        Environment.setLocalVar('__NRORGTRAB', widgetUnit.currentRow.NRORG);
                        Environment.setLocalVar('__NMFILIAL', widgetUnit.currentRow.NMFILIAL);

                        var descriptions = widgetUnit.currentRow.CDFILIAL + ' - ' + widgetUnit.currentRow.NMFILIAL;
                        templateManager.userData.descriptions[0] = descriptions;
                        menu.closeMenu(); // jshint ignore:line
                    }, 600);
                });
            }, function() {
                ScreenService.showMessage(ScreenService.i18n("Erro ao alterar filial!"));
            });
        });
    }

    function afterMoveRowSync(container) {

        var message = ScreenService.i18n('Deseja confirmar a unidade selecionada?');
        ScreenService.confirmMessage(message, 'question', function() {
            var widgetUnit = container.getWidget('WidgetUnitSync');
            // widgetUnit.currentRow.changed = true;
            ScreenService.showLoader();
            localStorageEngine.getDataSource('/User').then(function(user) {
                user[0].CDFILIALSELECIONADA = widgetUnit.currentRow.CDFILIAL;
                user[0].CDFILIAL = widgetUnit.currentRow.CDFILIAL;
                restEngine.post('/User/save', {
                    "requestType": "DataSet",
                    "dataset": user
                }, function(success) {
                    var deferred = ZHPromise.defer();
                    if (success.length !== 0) {
                        ScreenService.showMessage(ScreenService.i18n("Erro ao alterar filial!"));
                        ScreenService.openWindow('Dashboard');
                        deferred.reject();
                        return deferred;
                    } else {
                        UserService.save(user).then(function() {
                            widgetUnit.isVisible = false;
                            ScreenService.closePopup();
                            eventAggregator.publish("wscCallUpdate", null);
                        }, function() {
                            ScreenService.showMessage(ScreenService.i18n("Erro ao alterar filial!"));
                            ScreenService.openWindow('Dashboard');
                        });
                    }
                }, undefined, function(error) {
                    console.log(error);
                    ScreenService.showMessage(ScreenService.i18n("Erro ao alterar filial!"));
                    ScreenService.openWindow('Dashboard');
                });
            }).finally(function() {
                ScreenService.hideLoader();
            });
        });
    }

};
Configuration(function(ContextRegister) {
    ContextRegister.register('UnitController', UnitController);
});
