// FILE: js/libraries/excelFormulaUtilities.js
/*
 * excelFormulaUtilitiesJS
 * https://github.com/joshatjben/excelFormulaUtilitiesJS/
 *
 * Copyright 2011, Josh Bennett
 * licensed under the MIT license.
 * https://github.com/joshatjben/excelFormulaUtilitiesJS/blob/master/LICENSE.txt
 *
 * Some functionality based off of the jquery core lib
 * Copyright 2011, John Resig
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * Based on Ewbi's Go Calc Prototype Excel Formula Parser. [http://ewbi.blogs.com/develops/2004/12/excel_formula_p.html]
 */

(function() {

    window.excelFormulaUtilities = window.excelFormulaUtilities || {};
    var core = window.excelFormulaUtilities.core = {};
    window.excelFormulaUtilities.string = window.excelFormulaUtilities.string || {};

    /**
     * Simple/quick string formater. This will take an input string and apply n number of arguments to it.
     *
     * <b>example:</b><br />
     * <code>
     * <pre>
     *	var foo = excelFormulaUtilities.core.formatStr("{0}", "foo"); // foo will be set to "foo"
     *	var fooBar = excelFormulaUtilities.core.formatStr("{0} {1}", "foo", "bar"); // fooBar will be set to "fooBar"
     *	var error = excelFormulaUtilities.core.formatStr("{1}", "error"); // will throw an index out of range error since only 1 extra argument was passed, which would be index 0.
     * </pre>
     * </code>
     *
     * @memberOf window.excelFormulaUtilities.core
     * @function
     * @param {String} inStr 
     **/
    window.excelFormulaUtilities.string.formatStr = function(inStr) {
        var formattedStr = inStr;
        var argIndex = 1;
        for (; argIndex < arguments.length; argIndex++) {
            var replaceIndex = (argIndex - 1);
            var replaceRegex = new RegExp("\\{{1}" + replaceIndex.toString() + "{1}\\}{1}", "g");
            formattedStr = formattedStr.replace(replaceRegex, arguments[argIndex]);
        }
        return formattedStr;
    };

    window.excelFormulaUtilities.string.trim = function(inStr) {
        return inStr.replace(/^\s|\s$/, "");
    };

    window.excelFormulaUtilities.string.trim = function(inStr) {
        return inStr.replace(/^(?:\s|&nbsp;|<\s*br\s*\/*\s*>)*|(?:\s|&nbsp;|<\s*br\s*\/*\s*>)*$/, "");
    };

    //Quick and dirty type checks
    /**
     * @param {object} obj
     * @returns {boolean}
     * @memberOf window.excelFormulaUtilities.core
     */
    var isFunction = core.isFunction = function(obj) {
        return (typeof obj) === "function";
    };

    /**
     * @param {object} obj
     * @returns {boolean}
     * @memberOf window.excelFormulaUtilities.core
     */
    var isArray = core.isArray = function(obj) {
        return (typeof obj) === "object" && obj.length;
    };

    /**
     * @param {object} obj
     * @returns {boolean}
     * @memberOf window.excelFormulaUtilities.core
     */
    var isWindow = core.isWindow = function() {
        var obj;
        return obj && typeof obj === "object" && "setInterval" in obj;
    }; /*----The functionality below has based off of the jQuery core library----*/

    /**
     * Check if the object is a plain object or not. This has been pulled from the jQuery core and modified slightly.
     * @param {object} obj
     * @returns {boolean} returns weather the object is a plain object or not.
     * @memberOf window.excelFormulaUtilities.core
     */
    var isPlainObject = core.isPlainObject = function(obj) {
        // Must be an Object.
        // Because of IE, we also have to check the presence of the constructor property.
        // Make sure that DOM nodes and window objects don't pass through, as well
        if (!obj || typeof obj !== "object" || obj.nodeType || isWindow(obj)) {
            return false;
        }
        // Not own constructor property must be Object
        if (obj.constructor && !hasOwnProperty.call(obj, "constructor") && !hasOwnProperty.call(obj.constructor.prototype, "isPrototypeOf")) {
            return false;
        }
        // Own properties are enumerated firstly, so to speed up,
        // if last one is own, then all properties are own.
        var lastKey;
        for (var key in obj) {
            lastKey = key;
        }
        return lastKey === undefined || hasOwnProperty.call(obj, lastKey);
    };

    /**
     * This has been pulled from the jQuery core and modified slightly. see http://api.jquery.com/jQuery.extend/
     * @param {object} target
     * @param {object} object add one or more object to extend the target.
     * @returns {object} returns the extended object.
     * @memberOf window.excelFormulaUtilities.core
     */
    core.extend = function() {
        var options, name, src, copy, copyIsArray, clone, target = arguments[0] || {},
            i = 1,
            length = arguments.length,
            deep = false;
        // Handle a deep copy situation
        if (typeof target === "boolean") {
            deep = target;
            target = arguments[1] || {};
            // skip the boolean and the target
            i = 2;
        }
        // Handle case when target is a string or something (possible in deep copy)
        if (typeof target !== "object" && !isFunction(target)) {
            target = {};
        }
        // extend jQuery itself if only one argument is passed
        if (length === i) {
            target = this;
            --i;
        }
        for (; i < length; i++) {
            // Only deal with non-null/undefined values
            if ((options = arguments[i]) !== null) {
                // Extend the base object
                for (name in options) {
                    src = target[name];
                    copy = options[name];
                    // Prevent never-ending loop
                    if (target === copy) {
                        continue;
                    }
                    // Recurse if we're merging plain objects or arrays
                    if (deep && copy && (isPlainObject(copy) || (copyIsArray = isArray(copy)))) {
                        if (copyIsArray) {
                            copyIsArray = false;
                            clone = src && isArray(src) ? src : [];
                        } else {
                            clone = src && isPlainObject(src) ? src : {};
                        }
                        // Never move original objects, clone them
                        target[name] = core.extend(deep, clone, copy);
                        // Don't bring in undefined values
                    } else if (copy !== undefined) {
                        target[name] = copy;
                    }
                }
            }
        }
        // Return the modified object
        return target;
    }; /*----end of jquery functionality----*/

}());

/*
 * excelFormulaUtilitiesJS
 * https://github.com/joshatjben/excelFormulaUtilitiesJS/
 *
 * Copyright 2011, Josh Bennett
 * licensed under the MIT license.
 * https://github.com/joshatjben/excelFormulaUtilitiesJS/blob/master/LICENSE.txt
 *
 * Some functionality based off of the jquery core lib
 * Copyright 2011, John Resig
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * Based on Ewbi's Go Calc Prototype Excel Formula Parser. [http://ewbi.blogs.com/develops/2004/12/excel_formula_p.html]
 */
(function(root) {
    var excelFormulaUtilities = root.excelFormulaUtilities = root.excelFormulaUtilities || {},
        core = root.excelFormulaUtilities.core,
        formatStr = root.excelFormulaUtilities.string.formatStr,
        trim = root.excelFormulaUtilities.string.trim,

        types = {},
        TOK_TYPE_NOOP = types.TOK_TYPE_NOOP = "noop",
        TOK_TYPE_OPERAND = types.TOK_TYPE_OPERAND = "operand",
        TOK_TYPE_FUNCTION = types.TOK_TYPE_FUNCTION = "function",
        TOK_TYPE_SUBEXPR = types.TOK_TYPE_SUBEXPR = "subexpression",
        TOK_TYPE_ARGUMENT = types.TOK_TYPE_ARGUMENT = "argument",
        TOK_TYPE_OP_PRE = types.TOK_TYPE_OP_PRE = "operator-prefix",
        TOK_TYPE_OP_IN = types.TOK_TYPE_OP_IN = "operator-infix",
        TOK_TYPE_OP_POST = types.TOK_TYPE_OP_POST = "operator-postfix",
        TOK_TYPE_WSPACE = types.TOK_TYPE_WSPACE = "white-space",
        TOK_TYPE_UNKNOWN = types.TOK_TYPE_UNKNOWN = "unknown",

        TOK_SUBTYPE_START = types.TOK_SUBTYPE_START = "start",
        TOK_SUBTYPE_STOP = types.TOK_SUBTYPE_STOP = "stop",

        TOK_SUBTYPE_TEXT = types.TOK_SUBTYPE_TEXT = "text",
        TOK_SUBTYPE_NUMBER = types.TOK_SUBTYPE_NUMBER = "number",
        TOK_SUBTYPE_LOGICAL = types.TOK_SUBTYPE_LOGICAL = "logical",
        TOK_SUBTYPE_ERROR = types.TOK_SUBTYPE_ERROR = "error",
        TOK_SUBTYPE_RANGE = types.TOK_SUBTYPE_RANGE = "range",

        TOK_SUBTYPE_MATH = types.TOK_SUBTYPE_MATH = "math",
        TOK_SUBTYPE_CONCAT = types.TOK_SUBTYPE_CONCAT = "concatenate",
        TOK_SUBTYPE_INTERSECT = types.TOK_SUBTYPE_INTERSECT = "intersect",
        TOK_SUBTYPE_UNION = types.TOK_SUBTYPE_UNION = "union";

    root.excelFormulaUtilities.isEu = typeof root.excelFormulaUtilities.isEu === 'boolean' ? root.excelFormulaUtilities.isEu : false;

    /**
     * @class
     */

    function F_token(value, type, subtype) {
        this.value = value;
        this.type = type;
        this.subtype = subtype;
    }

    /**
     * @class
     */

    function F_tokens() {

        this.items = [];

        this.add = function(value, type, subtype) {
            if (!subtype) {
                subtype = "";
            }
            var token = new F_token(value, type, subtype);
            this.addRef(token);
            return token;
        };
        this.addRef = function(token) {
            this.items.push(token);
        };

        this.index = -1;
        this.reset = function() {
            this.index = -1;
        };
        this.BOF = function() {
            return (this.index <= 0);
        };
        this.EOF = function() {
            return (this.index >= (this.items.length - 1));
        };
        this.moveNext = function() {
            if (this.EOF()) {
                return false;
            }
            this.index += 1;
            return true;
        };
        this.current = function() {
            if (this.index === -1) {
                return null;
            }
            return (this.items[this.index]);
        };
        this.next = function() {
            if (this.EOF()) {
                return null;
            }
            return (this.items[this.index + 1]);
        };
        this.previous = function() {
            if (this.index < 1) {
                return null;
            }
            return (this.items[this.index - 1]);
        };

    }

    function F_tokenStack() {

        this.items = [];

        this.push = function(token) {
            this.items.push(token);
        };
        this.pop = function(name) {
            var token = this.items.pop();
            return (new F_token(name || "", token.type, TOK_SUBTYPE_STOP));
        };

        this.token = function() {
            return ((this.items.length > 0) ? this.items[this.items.length - 1] : null);
        };
        this.value = function() {
            return ((this.token()) ? this.token().value.toString() : "");
        };
        this.type = function() {
            return ((this.token()) ? this.token().type.toString() : "");
        };
        this.subtype = function() {
            return ((this.token()) ? this.token().subtype.toString() : "");
        };

    }

    function getTokens(formula) {

        var tokens = new F_tokens(),
            tokenStack = new F_tokenStack(),

            offset = 0,

            currentChar = function() {
                return formula.substr(offset, 1);
            },
            doubleChar = function() {
                return formula.substr(offset, 2);
            },
            nextChar = function() {
                return formula.substr(offset + 1, 1);
            },
            EOF = function() {
                return (offset >= formula.length);
            },

            token = "",

            inString = false,
            inPath = false,
            inRange = false,
            inError = false,
            regexSN = /^[1-9]{1}(\.[0-9]+)?E{1}$/;

        while (formula.length > 0) {
            if (formula.substr(0, 1) === " ") {
                formula = formula.substr(1);
            } else {
                if (formula.substr(0, 1) === "=") {
                    formula = formula.substr(1);
                }
                break;
            }
        }

        while (!EOF()) {

            // state-dependent character evaluation (order is important)
            // double-quoted strings
            // embeds are doubled
            // end marks token
            if (inString) {
                if (currentChar() === "\"") {
                    if (nextChar() === "\"") {
                        token += "\"";
                        offset += 1;
                    } else {
                        inString = false;
                        tokens.add(token, TOK_TYPE_OPERAND, TOK_SUBTYPE_TEXT);
                        token = "";
                    }
                } else {
                    token += currentChar();
                }
                offset += 1;
                continue;
            }

            // single-quoted strings (links)
            // embeds are double
            // end does not mark a token
            if (inPath) {
                if (currentChar() === "'") {

                    if (nextChar() === "'") {
                        token += "'";
                        offset += 1;
                    } else {
                        inPath = false;
                        token += "'";
                    }
                } else {
                    token += currentChar();
                }

                offset += 1;
                continue;
            }

            // bracked strings (range offset or linked workbook name)
            // no embeds (changed to "()" by Excel)
            // end does not mark a token
            if (inRange) {
                if (currentChar() === "]") {
                    inRange = false;
                }
                token += currentChar();
                offset += 1;
                continue;
            }

            // error values
            // end marks a token, determined from absolute list of values
            if (inError) {
                token += currentChar();
                offset += 1;
                if ((",#NULL!,#DIV/0!,#VALUE!,#REF!,#NAME?,#NUM!,#N/A,").indexOf("," + token + ",") !== -1) {
                    inError = false;
                    tokens.add(token, TOK_TYPE_OPERAND, TOK_SUBTYPE_ERROR);
                    token = "";
                }
                continue;
            }

            // scientific notation check
            if (("+-").indexOf(currentChar()) !== -1) {
                if (token.length > 1) {
                    if (token.match(regexSN)) {
                        token += currentChar();
                        offset += 1;
                        continue;
                    }
                }
            }

            // independent character evaulation (order not important)
            // establish state-dependent character evaluations
            if (currentChar() === "\"") {
                if (token.length > 0) {
                    // not expected
                    tokens.add(token, TOK_TYPE_UNKNOWN);
                    token = "";
                }
                inString = true;
                offset += 1;
                continue;
            }

            if (currentChar() === "'") {
                if (token.length > 0) {
                    // not expected
                    tokens.add(token, TOK_TYPE_UNKNOWN);
                    token = "";
                }
                token = "'";
                inPath = true;
                offset += 1;
                continue;
            }

            if (currentChar() === "[") {
                inRange = true;
                token += currentChar();
                offset += 1;
                continue;
            }

            if (currentChar() === "#") {
                if (token.length > 0) {
                    // not expected
                    tokens.add(token, TOK_TYPE_UNKNOWN);
                    token = "";
                }
                inError = true;
                token += currentChar();
                offset += 1;
                continue;
            }

            // mark start and end of arrays and array rows
            if (currentChar() === "{") {
                if (token.length > 0) {
                    // not expected
                    tokens.add(token, TOK_TYPE_UNKNOWN);
                    token = "";
                }
                tokenStack.push(tokens.add("ARRAY", TOK_TYPE_FUNCTION, TOK_SUBTYPE_START));
                tokenStack.push(tokens.add("ARRAYROW", TOK_TYPE_FUNCTION, TOK_SUBTYPE_START));
                offset += 1;
                continue;
            }

            if (currentChar() === ";") {
                if (root.excelFormulaUtilities.isEu) {
                    // If is EU then handle ; as list seperators
                    if (token.length > 0) {
                        tokens.add(token, TOK_TYPE_OPERAND);
                        token = "";
                    }
                    if (tokenStack.type() !== TOK_TYPE_FUNCTION) {
                        tokens.add(currentChar(), TOK_TYPE_OP_IN, TOK_SUBTYPE_UNION);
                    } else {
                        tokens.add(currentChar(), TOK_TYPE_ARGUMENT);
                    }
                    offset += 1;
                    continue;
                } else {
                    // Else if not Eu handle ; as array row seperator
                    if (token.length > 0) {
                        tokens.add(token, TOK_TYPE_OPERAND);
                        token = "";
                    }
                    tokens.addRef(tokenStack.pop());
                    tokens.add(",", TOK_TYPE_ARGUMENT);
                    tokenStack.push(tokens.add("ARRAYROW", TOK_TYPE_FUNCTION, TOK_SUBTYPE_START));
                    offset += 1;
                    continue;
                }
            }

            if (currentChar() === "}") {
                if (token.length > 0) {
                    tokens.add(token, TOK_TYPE_OPERAND);
                    token = "";
                }
                tokens.addRef(tokenStack.pop("ARRAYROWSTOP"));
                tokens.addRef(tokenStack.pop("ARRAYSTOP"));
                offset += 1;
                continue;
            }

            // trim white-space
            if (currentChar() === " ") {
                if (token.length > 0) {
                    tokens.add(token, TOK_TYPE_OPERAND);
                    token = "";
                }
                tokens.add("", TOK_TYPE_WSPACE);
                offset += 1;
                while ((currentChar() === " ") && (!EOF())) {
                    offset += 1;
                }
                continue;
            }

            // multi-character comparators
            if ((",>=,<=,<>,").indexOf("," + doubleChar() + ",") !== -1) {
                if (token.length > 0) {
                    tokens.add(token, TOK_TYPE_OPERAND);
                    token = "";
                }
                tokens.add(doubleChar(), TOK_TYPE_OP_IN, TOK_SUBTYPE_LOGICAL);
                offset += 2;
                continue;
            }

            // standard infix operators
            if (("+-*/^&=><").indexOf(currentChar()) !== -1) {
                if (token.length > 0) {
                    tokens.add(token, TOK_TYPE_OPERAND);
                    token = "";
                }
                tokens.add(currentChar(), TOK_TYPE_OP_IN);
                offset += 1;
                continue;
            }

            // standard postfix operators
            if (("%").indexOf(currentChar()) !== -1) {
                if (token.length > 0) {
                    tokens.add(token, TOK_TYPE_OPERAND);
                    token = "";
                }
                tokens.add(currentChar(), TOK_TYPE_OP_POST);
                offset += 1;
                continue;
            }

            // start subexpression or function
            if (currentChar() === "(") {
                if (token.length > 0) {
                    tokenStack.push(tokens.add(token, TOK_TYPE_FUNCTION, TOK_SUBTYPE_START));
                    token = "";
                } else {
                    tokenStack.push(tokens.add("", TOK_TYPE_SUBEXPR, TOK_SUBTYPE_START));
                }
                offset += 1;
                continue;
            }

            // function, subexpression, array parameters
            if (currentChar() === "," && !root.excelFormulaUtilities.isEu) {
                if (token.length > 0) {
                    tokens.add(token, TOK_TYPE_OPERAND);
                    token = "";
                }
                if (tokenStack.type() !== TOK_TYPE_FUNCTION) {
                    tokens.add(currentChar(), TOK_TYPE_OP_IN, TOK_SUBTYPE_UNION);
                } else {
                    tokens.add(currentChar(), TOK_TYPE_ARGUMENT);
                }
                offset += 1;
                continue;
            }

            // stop subexpression
            if (currentChar() === ")") {
                if (token.length > 0) {
                    tokens.add(token, TOK_TYPE_OPERAND);
                    token = "";
                }
                tokens.addRef(tokenStack.pop());
                offset += 1;
                continue;
            }

            // token accumulation
            token += currentChar();
            offset += 1;

        }

        // dump remaining accumulation
        if (token.length > 0 || inString || inPath || inRange || inError) {
            if (inString || inPath || inRange || inError) {
                if (inString) {
                    token = "\"" + token;
                } else if (inPath) {
                    token = "'" + token;
                } else if (inRange) {
                    token = "[" + token;
                } else if (inError) {
                    token = "#" + token;
                }

                tokens.add(token, TOK_TYPE_UNKNOWN);
            } else {
                tokens.add(token, TOK_TYPE_OPERAND);
            }
        }

        // move all tokens to a new collection, excluding all unnecessary white-space tokens
        var tokens2 = new F_tokens();

        while (tokens.moveNext()) {

            token = tokens.current();

            if (token.type.toString() === TOK_TYPE_WSPACE) {
                var doAddToken = (tokens.BOF()) || (tokens.EOF());
                //if ((tokens.BOF()) || (tokens.EOF())) {}
                doAddToken = doAddToken && (((tokens.previous().type.toString() === TOK_TYPE_FUNCTION) && (tokens.previous().subtype.toString() === TOK_SUBTYPE_STOP)) || ((tokens.previous().type.toString() === TOK_TYPE_SUBEXPR) && (tokens.previous().subtype.toString() === TOK_SUBTYPE_STOP)) || (tokens.previous().type.toString() === TOK_TYPE_OPERAND));
                //else if (!(
                //       ((tokens.previous().type === TOK_TYPE_FUNCTION) && (tokens.previous().subtype == TOK_SUBTYPE_STOP))
                //    || ((tokens.previous().type == TOK_TYPE_SUBEXPR) && (tokens.previous().subtype == TOK_SUBTYPE_STOP))
                //    || (tokens.previous().type == TOK_TYPE_OPERAND)))
                //  {}
                doAddToken = doAddToken && (((tokens.next().type.toString() === TOK_TYPE_FUNCTION) && (tokens.next().subtype.toString() === TOK_SUBTYPE_START)) || ((tokens.next().type.toString() === TOK_TYPE_SUBEXPR) && (tokens.next().subtype.toString() === TOK_SUBTYPE_START)) || (tokens.next().type.toString() === TOK_TYPE_OPERAND));
                //else if (!(
                //	((tokens.next().type == TOK_TYPE_FUNCTION) && (tokens.next().subtype == TOK_SUBTYPE_START))
                //	|| ((tokens.next().type == TOK_TYPE_SUBEXPR) && (tokens.next().subtype == TOK_SUBTYPE_START))
                //	|| (tokens.next().type == TOK_TYPE_OPERAND)))
                //	{}
                //else { tokens2.add(token.value, TOK_TYPE_OP_IN, TOK_SUBTYPE_INTERSECT)};
                if (doAddToken) {
                    tokens2.add(token.value.toString(), TOK_TYPE_OP_IN, TOK_SUBTYPE_INTERSECT);
                }
                continue;
            }

            tokens2.addRef(token);

        }

        // switch infix "-" operator to prefix when appropriate, switch infix "+" operator to noop when appropriate, identify operand
        // and infix-operator subtypes, pull "@" from in front of function names
        while (tokens2.moveNext()) {

            token = tokens2.current();

            if ((token.type.toString() === TOK_TYPE_OP_IN) && (token.value.toString() === "-")) {
                if (tokens2.BOF()) {
                    token.type = TOK_TYPE_OP_PRE.toString();
                } else if (((tokens2.previous().type.toString() === TOK_TYPE_FUNCTION) && (tokens2.previous().subtype.toString() === TOK_SUBTYPE_STOP)) || ((tokens2.previous().type.toString() === TOK_TYPE_SUBEXPR) && (tokens2.previous().subtype.toString() === TOK_SUBTYPE_STOP)) || (tokens2.previous().type.toString() === TOK_TYPE_OP_POST) || (tokens2.previous().type.toString() === TOK_TYPE_OPERAND)) {
                    token.subtype = TOK_SUBTYPE_MATH.toString();
                } else {
                    token.type = TOK_TYPE_OP_PRE.toString();
                }
                continue;
            }

            if ((token.type.toString() === TOK_TYPE_OP_IN) && (token.value.toString() === "+")) {
                if (tokens2.BOF()) {
                    token.type = TOK_TYPE_NOOP.toString();
                } else if (((tokens2.previous().type.toString() === TOK_TYPE_FUNCTION) && (tokens2.previous().subtype.toString() === TOK_SUBTYPE_STOP)) || ((tokens2.previous().type.toString() === TOK_TYPE_SUBEXPR) && (tokens2.previous().subtype.toString() === TOK_SUBTYPE_STOP)) || (tokens2.previous().type.toString() === TOK_TYPE_OP_POST) || (tokens2.previous().type.toString() === TOK_TYPE_OPERAND)) {
                    token.subtype = TOK_SUBTYPE_MATH.toString();
                } else {
                    token.type = TOK_TYPE_NOOP.toString();
                }
                continue;
            }

            if ((token.type.toString() === TOK_TYPE_OP_IN) && (token.subtype.length === 0)) {
                if (("<>=").indexOf(token.value.substr(0, 1)) !== -1) {
                    token.subtype = TOK_SUBTYPE_LOGICAL.toString();
                } else if (token.value.toString() === "&") {
                    token.subtype = TOK_SUBTYPE_CONCAT.toString();
                } else {
                    token.subtype = TOK_SUBTYPE_MATH.toString();
                }
                continue;
            }

            if ((token.type.toString() === TOK_TYPE_OPERAND) && (token.subtype.length === 0)) {
                if (isNaN(parseFloat(token.value))) {
                    if ((token.value.toString() === 'TRUE') || (token.value.toString() === 'FALSE')) {
                        token.subtype = TOK_SUBTYPE_LOGICAL.toString();
                    } else {
                        token.subtype = TOK_SUBTYPE_RANGE.toString();
                    }
                } else {
                    token.subtype = TOK_SUBTYPE_NUMBER.toString();
                }

                continue;
            }

            if (token.type.toString() === TOK_TYPE_FUNCTION) {
                if (token.value.substr(0, 1) === "@") {
                    token.value = token.value.substr(1).toString();
                }
                continue;
            }

        }

        tokens2.reset();

        // move all tokens to a new collection, excluding all noops
        tokens = new F_tokens();

        while (tokens2.moveNext()) {
            if (tokens2.current().type.toString() !== TOK_TYPE_NOOP) {
                tokens.addRef(tokens2.current());
            }
        }

        tokens.reset();

        return tokens;
    }

    excelFormulaUtilities.parseFormula = function(inputID, outputID) {

        var indentCount = 0;

        var indent = function() {
            var s = "|",
                i = 0;
            for (; i < indentCount; i += 1) {
                s += "&nbsp;&nbsp;&nbsp;|";
            }
            return s;
        };

        var formulaControl = document.getElementById(inputID);
        var formula = formulaControl.value;

        var tokens = getTokens(formula);

        var tokensHtml = "";

        tokensHtml += "<table cellspacing='0' style='border-top: 1px #cecece solid; margin-top: 5px; margin-bottom: 5px'>";
        tokensHtml += "<tr>";
        tokensHtml += "<td class='token' style='font-weight: bold; width: 50px'>index</td>";
        tokensHtml += "<td class='token' style='font-weight: bold; width: 125px'>type</td>";
        tokensHtml += "<td class='token' style='font-weight: bold; width: 125px'>subtype</td>";
        tokensHtml += "<td class='token' style='font-weight: bold; width: 150px'>token</td>";
        tokensHtml += "<td class='token' style='font-weight: bold; width: 300px'>token tree</td></tr>";

        while (tokens.moveNext()) {

            var token = tokens.current();

            if (token.subtype === TOK_SUBTYPE_STOP) {
                indentCount -= ((indentCount > 0) ? 1 : 0);
            }

            tokensHtml += "<tr>";

            tokensHtml += "<td class='token'>" + (tokens.index + 1) + "</td>";

            tokensHtml += "<td class='token'>" + token.type + "</td>";
            tokensHtml += "<td class='token'>" + ((token.subtype.length === 0) ? "&nbsp;" : token.subtype.toString()) + "</td>";
            tokensHtml += "<td class='token'>" + ((token.value.length === 0) ? "&nbsp;" : token.value).split(" ").join("&nbsp;") + "</td>";
            tokensHtml += "<td class='token'>" + indent() + ((token.value.length === 0) ? "&nbsp;" : token.value).split(" ").join("&nbsp;") + "</td>";

            tokensHtml += "</tr>";

            if (token.subtype === TOK_SUBTYPE_START) {
                indentCount += 1;
            }

        }

        tokensHtml += "</table>";

        document.getElementById(outputID).innerHTML = tokensHtml;

        formulaControl.select();
        formulaControl.focus();

    };

    // Pass a range such as A1:B2 along with a
    // delimiter to get back a full list of ranges.
    //
    // Example:
    //    breakOutRanges("A1:B2", "+"); //Returns A1+A2+B1+B2
    function breakOutRanges(rangeStr, delimStr) {

        //Quick Check to see if if rangeStr is a valid range
        if (!RegExp("[a-z]+[0-9]+:[a-z]+[0-9]+", "gi").test(rangeStr)) {
            throw "This is not a valid range: " + rangeStr;
        }
        var endRow, endCol, endColDec, totalRows, totalCols, curCol, curRow, curCell, retStr;

        //Make the rangeStr lowercase to deal with looping.
        var range = rangeStr.split(":"),

            startRow = parseInt(range[0].match(/[0-9]+/gi)[0]),
            startCol = range[0].match(/[A-Z]+/gi)[0],
            startColDec = fromBase26(startCol);

        endRow = parseInt(range[1].match(/[0-9]+/gi)[0]),
            endCol = range[1].match(/[A-Z]+/gi)[0],
            endColDec = fromBase26(endCol),

            // Total rows and cols
            totalRows = endRow - startRow + 1,
            totalCols = fromBase26(endCol) - fromBase26(startCol) + 1,

            // Loop vars
            curCol = 0,
            curRow = 1,
            curCell = "",

            //Return String
            retStr = ""; // jshint ignore:line

        for (; curRow <= totalRows; curRow += 1) {
            for (; curCol < totalCols; curCol += 1) {
                // Get the current cell id
                curCell = toBase26(startColDec + curCol) + "" + (startRow + curRow - 1);
                retStr += curCell + (curRow === totalRows && curCol === totalCols - 1 ? "" : delimStr);
            }
            curCol = 0;
        }

        return retStr;

    }

    //Modified from function at http://en.wikipedia.org/wiki/Hexavigesimal
    var toBase26 = excelFormulaUtilities.toBase26 = function(value) {

        value = Math.abs(value);

        var converted = "",
            iteration = false,
            remainder;

        // Repeatedly divide the numerb by 26 and convert the
        // remainder into the appropriate letter.
        do {
            remainder = value % 26;

            // Compensate for the last letter of the series being corrected on 2 or more iterations.
            if (iteration && value < 25) {
                remainder--;
            }

            converted = String.fromCharCode((remainder + 'A'.charCodeAt(0))) + converted;
            value = Math.floor((value - remainder) / 26);

            iteration = true;
        } while (value > 0);

        return converted;
    };

    // This was Modified from a function at http://en.wikipedia.org/wiki/Hexavigesimal
    // Pass in the base 26 string, get back integer
    var fromBase26 = excelFormulaUtilities.fromBase26 = function(number) {
        number = number.toUpperCase();

        var s = 0,
            i = 0,
            dec = 0;

        if (
            number !== null && typeof number !== "undefined" && number.length > 0) {
            for (; i < number.length; i++) {
                s = number.charCodeAt(number.length - i - 1) - "A".charCodeAt(0);
                dec += (Math.pow(26, i)) * (s + 1);
            }
        }

        return dec - 1;
    };

    function applyTokenTemplate(token, options, indent, lineBreak, override) {

        var indt = indent;

        var lastToken = typeof arguments[5] === undefined || arguments[5] === null ? null : arguments[5];

        var replaceTokenTmpl = function(inStr) {
            return inStr.replace(/\{\{token\}\}/gi, "{0}").replace(/\{\{autoindent\}\}/gi, "{1}").replace(/\{\{autolinebreak\}\}/gi, "{2}");
        };

        var tokenString = "";

        if (token.subtype === "text" || token.type === "text") {
            tokenString = token.value.toString();
        } else if (token.type === 'operand' && token.subtype === 'range') {
            tokenString = token.value.toString();
        } else {
            tokenString = ((token.value.length === 0) ? " " : token.value.toString()).split(" ").join("").toString();
        }

        if (typeof override === 'function') {
            var returnVal = override(tokenString, token, indent, lineBreak);

            tokenString = returnVal.tokenString;

            if (!returnVal.useTemplate) {
                return tokenString;
            }
        }

        switch (token.type) {

            case "function":
                //-----------------FUNCTION------------------
                switch (token.value) {
                    case "ARRAY":
                        tokenString = formatStr(replaceTokenTmpl(options.tmplFunctionStartArray), tokenString, indt, lineBreak);
                        break;
                    case "ARRAYROW":
                        tokenString = formatStr(replaceTokenTmpl(options.tmplFunctionStartArrayRow), tokenString, indt, lineBreak);
                        break;
                    default:
                        if (token.subtype.toString() === "start") {
                            tokenString = formatStr(replaceTokenTmpl(options.tmplFunctionStart), tokenString, indt, lineBreak);
                        } else {
                            tokenString = formatStr(replaceTokenTmpl(options.tmplFunctionStop), tokenString, indt, lineBreak);
                        }
                        break;
                }
                break;
            case "operand":
                //-----------------OPERAND------------------
                switch (token.subtype.toString()) {
                    case "error":
                        tokenString = formatStr(replaceTokenTmpl(options.tmplOperandError), tokenString, indt, lineBreak);
                        break;
                    case "range":
                        tokenString = formatStr(replaceTokenTmpl(options.tmplOperandRange), tokenString, indt, lineBreak);
                        break;
                    case "logical":
                        tokenString = formatStr(replaceTokenTmpl(options.tmplOperandLogical), tokenString, indt, lineBreak);
                        break;
                    case "number":
                        tokenString = formatStr(replaceTokenTmpl(options.tmplOperandNumber), tokenString, indt, lineBreak);
                        break;
                    case "text":
                        tokenString = formatStr(replaceTokenTmpl(options.tmplOperandText), tokenString, indt, lineBreak);
                        break;
                    case "argument":
                        tokenString = formatStr(replaceTokenTmpl(options.tmplArgument), tokenString, indt, lineBreak);
                        break;
                    default:
                        break;
                }
                break;
            case "operator-infix":
                tokenString = formatStr(replaceTokenTmpl(options.tmplOperandOperatorInfix), tokenString, indt, lineBreak);
                break;
            case "logical":
                tokenString = formatStr(replaceTokenTmpl(options.tmplLogical), tokenString, indt, lineBreak);
                break;
            case "argument":
                if (lastToken.type !== "argument") {
                    tokenString = formatStr(replaceTokenTmpl(options.tmplArgument), tokenString, indt, lineBreak);
                } else {
                    tokenString = formatStr(replaceTokenTmpl("{{autoindent}}" + options.tmplArgument), tokenString, indt, lineBreak);
                }
                break;
            case "subexpression":
                if (token.subtype.toString() === "start") {
                    tokenString = formatStr(replaceTokenTmpl(options.tmplSubexpressionStart), tokenString, indt, lineBreak);
                } else {
                    tokenString = formatStr(replaceTokenTmpl(options.tmplSubexpressionStop), tokenString, indt, lineBreak);
                }
                break;
            default:

                break;
        }
        return tokenString;
    }

    /**
     *
     * @memberof excelFormulaUtilities.parser
     * @function
     * @param {string} formula
     * @param {object} options optional param
     *<pre>
     *   TEMPLATE VALUES
     *  {{autoindent}} - apply auto indent based on current tree level
     *  {{token}} - the named token such as FUNCTION_NAME or "string"
     *  {{autolinebreak}} - apply linbreak automaticly. tests for next element only at this point
     *
     * Options include:
     *  tmplFunctionStart           - template for the start of a function, the {{token}} will contain the name of the function.
     *  tmplFunctionStop            - template for when the end of a function has been reached.
     *  tmplOperandError            - template for errors.
     *  tmplOperandRange            - template for ranges and variable names.
     *  tmplOperandLogical          - template for logical operators such as + - = ...
     *  tmplOperandNumber           - template for numbers.
     *  tmplOperandText             - template for text/strings.
     *  tmplArgument				- template for argument seperators such as ,.
     *  tmplFunctionStartArray      - template for the start of an array.
     *  tmplFunctionStartArrayRow   - template for the start of an array row.
     *  tmplFunctionStopArrayRow    - template for the end of an array row.
     *  tmplFunctionStopArray       - template for the end of an array.
     *  tmplSubexpressionStart      - template for the sub expresson start
     *  tmplSubexpressionStop       - template for the sub expresson stop
     *  tmplIndentTab               - template for the tab char.
     *  tmplIndentSpace             - template for space char.
     *  autoLineBreak               - when rendering line breaks automaticly which types should it break on. "TOK_SUBTYPE_STOP | TOK_SUBTYPE_START | TOK_TYPE_ARGUMENT"
     *  newLine                     - used for the {{autolinebreak}} replacement as well as some string parsing. if this is not set correctly you may get undesired results. usually \n for text or <br /> for html
     *  trim: true                  - trim the output.
     *	customTokenRender: null     - this is a call back to a custom token function. your call back should look like
     *                                EXAMPLE:
     *
     *                                    customTokenRender: function(tokenString, token, indent, linbreak){
     *                                        var outstr = token,
     *                                            useTemplate = true;
     *                                        // In the return object "useTemplate" tells formatFormula()
     *                                        // weather or not to apply the template to what your return from the "tokenString".
     *                                        return {tokenString: outstr, useTemplate: useTemplate};
     *                                    }
     *
     *</pre>
     * @returns {string}
     */
    var formatFormula = excelFormulaUtilities.formatFormula = function(formula, options) {
        //Quick fix for trailing space after = sign
        formula = formula.replace(/^\s*=\s+/, "=");

        var isFirstToken = true,
            defaultOptions = {
                tmplFunctionStart: '{{autoindent}}{{token}}(\n',
                tmplFunctionStop: '\n{{autoindent}}{{token}})',
                tmplOperandError: ' {{token}}',
                tmplOperandRange: '{{autoindent}}{{token}}',
                tmplLogical: '{{token}}{{autolinebreak}}',
                tmplOperandLogical: '{{autoindent}}{{token}}',
                tmplOperandNumber: '{{autoindent}}{{token}}',
                tmplOperandText: '{{autoindent}}"{{token}}"',
                tmplArgument: '{{token}}\n',
                tmplOperandOperatorInfix: ' {{token}}{{autolinebreak}}',
                tmplFunctionStartArray: '',
                tmplFunctionStartArrayRow: '{',
                tmplFunctionStopArrayRow: '}',
                tmplFunctionStopArray: '',
                tmplSubexpressionStart: '{{autoindent}}(\n',
                tmplSubexpressionStop: '\n)',
                tmplIndentTab: '\t',
                tmplIndentSpace: ' ',
                autoLineBreak: 'TOK_TYPE_FUNCTION | TOK_TYPE_ARGUMENT | TOK_SUBTYPE_LOGICAL | TOK_TYPE_OP_IN ',
                newLine: '\n',
                //trim: true,
                customTokenRender: null,
                prefix: "",
                postfix: ""
            };

        if (options) {
            options = core.extend(true, defaultOptions, options);
        } else {
            options = defaultOptions;
        }

        var indentCount = 0;

        var indent = function() {
            var s = "",
                i = 0;

            for (; i < indentCount; i += 1) {
                s += options.tmplIndentTab;
            }
            return s;
        };

        var tokens = getTokens(formula);

        var outputFormula = "";

        var autoBreakArray = options.autoLineBreak.replace(/\s/gi, "").split("|");

        //Tokens
        var isNewLine = true;

        var testAutoBreak = function(nextToken) {
            var i = 0;
            for (; i < autoBreakArray.length; i += 1) {
                if (nextToken !== null && typeof nextToken !== 'undefined' && (types[autoBreakArray[i]] === nextToken.type.toString() || types[autoBreakArray[i]] === nextToken.subtype.toString())) {
                    return true;
                }
            }
            return false;
        };

        var lastToken = null;

        while (tokens.moveNext()) {

            var token = tokens.current();
            var nextToken = tokens.next();

            if (token.subtype.toString() === TOK_SUBTYPE_STOP) {
                indentCount -= ((indentCount > 0) ? 1 : 0);
            }

            var matchEndNewLine = new RegExp(options.newLine + '$', ''),
                autoBreak = testAutoBreak(nextToken),
                autoIndent = isNewLine,
                indt = autoIndent ? indent() : options.tmplIndentSpace,
                lineBreak = autoBreak ? options.newLine : "";

            // TODO this strips out spaces which breaks part of issue 28.  'Data Sheet' gets changed to DataSheet
            outputFormula += applyTokenTemplate(token, options, indt, lineBreak, options.customTokenRender, lastToken);

            if (token.subtype.toString() === TOK_SUBTYPE_START) {
                indentCount += 1;

            }

            isNewLine = autoBreak || matchEndNewLine.test(outputFormula);
            isFirstToken = false;

            lastToken = token;
        }

        outputFormula = options.prefix + trim(outputFormula) + options.postfix;

        return outputFormula;
    };
    /**
     * This function calls {@link excelFormulaUtilities.parser.formatFormula}
     *
     * @memberof excelFormulaUtilities.parser
     * @function
     * @param {string} formula
     * @param {object} options optional param
     */
    excelFormulaUtilities.formatFormulaHTML = function(formula, options) {
        var tokRender = function(tokenStr, token) {
            var outstr = tokenStr;
            switch (token.type.toString()) {
                case TOK_TYPE_OPERAND:
                    if (token.subtype === TOK_SUBTYPE_TEXT) {
                        outstr = tokenStr.replace(/</gi, "&lt;").replace(/>/gi, "&gt;");
                    }
                    break;
            }

            return {
                tokenString: outstr,
                useTemplate: true
            };
        };
        var defaultOptions = {
            tmplFunctionStart: '{{autoindent}}<span class="function">{{token}}</span><span class="function_start">(</span><br />',
            tmplFunctionStop: '<br />{{autoindent}}{{token}}<span class="function_stop">)</span>',
            tmplOperandText: '{{autoindent}}<span class="quote_mark">"</span><span class="text">{{token}}</span><span class="quote_mark">"</span>',
            tmplArgument: '{{token}}<br />',
            tmplSubexpressionStart: '{{autoindent}}(',
            tmplSubexpressionStop: ' )',
            tmplIndentTab: '<span class="tabbed">&nbsp;&nbsp;&nbsp;&nbsp;</span>',
            tmplIndentSpace: '&nbsp;',
            newLine: '<br />',
            autoLineBreak: 'TOK_TYPE_FUNCTION | TOK_TYPE_ARGUMENT | TOK_SUBTYPE_LOGICAL | TOK_TYPE_OP_IN ',
            trim: true,
            prefix: "=",
            customTokenRender: tokRender
        };

        if (options) {
            options = core.extend(true, defaultOptions, options);
        } else {
            options = defaultOptions;
        }

        return formatFormula(formula, options);
    };

    /**
     *
     * @memberof excelFormulaUtilities.convert
     * @function
     * @param {string} formula
     * @returns {string}
     */
    var formula2CSharp = excelFormulaUtilities.formula2CSharp = function(formula, options) {

        //Custom callback to format as c#
        var functionStack = [];
        var outStr;

        var tokRender = function(tokenStr, token) {
            var outstr = "",
                /*tokenString = (token.value.length === 0) ? "" : token.value.toString(),*/
                tokenString = tokenStr,
                directConversionMap = {
                    "=": "==",
                    "<>": "!=",
                    "MIN": "Math.min",
                    "MAX": "Math.max",
                    "ABS": "Math.abs",
                    "SUM": "",
                    "IF": "",
                    "&": "+",
                    "AND": "",
                    "OR": ""
                },
                currentFunctionOnStack = functionStack[functionStack.length - 1],
                useTemplate = false;

            switch (token.type.toString()) {

                case TOK_TYPE_FUNCTION:

                    switch (token.subtype) {

                        case TOK_SUBTYPE_START:

                            functionStack.push({
                                name: tokenString,
                                argumentNumber: 0
                            });
                            outstr = typeof directConversionMap[tokenString.toUpperCase()] === "string" ? directConversionMap[tokenString.toUpperCase()] : tokenString;
                            useTemplate = true;

                            break;

                        case TOK_SUBTYPE_STOP:

                            useTemplate = true;
                            switch (currentFunctionOnStack.name.toLowerCase()) {
                                case "if":
                                    outstr = currentFunctionOnStack.argumentNumber === 1 ? ":0)" : ")";
                                    useTemplate = false;
                                    break;
                                default:
                                    outstr = typeof directConversionMap[tokenString.toUpperCase()] === "string" ? directConversionMap[tokenString.toUpperCase()] : tokenString;
                                    break;
                            }
                            functionStack.pop();
                            break;
                    }

                    break;

                case TOK_TYPE_ARGUMENT:
                    switch (currentFunctionOnStack.name.toLowerCase()) {
                        case "if":
                            switch (currentFunctionOnStack.argumentNumber) {
                                case 0:
                                    outstr = "?";
                                    break;
                                case 1:
                                    outstr = ":";
                                    break;
                            }
                            break;
                        case "sum":
                            outstr = "+";
                            break;
                        case "and":
                            outstr = "&&";
                            break;
                        case "or":
                            outstr = "||";
                            break;
                        default:
                            outstr = typeof directConversionMap[tokenString.toUpperCase()] === "string" ? directConversionMap[tokenString.toUpperCase()] : tokenString;
                            useTemplate = true;
                            break;
                    }

                    currentFunctionOnStack.argumentNumber += 1;

                    break;

                case TOK_TYPE_OPERAND:

                    switch (token.subtype) {

                        case TOK_SUBTYPE_RANGE:
                            //Assume '=' sign
                            if (!currentFunctionOnStack) {
                                break;
                            }
                            switch (currentFunctionOnStack.name.toLowerCase()) {
                                // If in the sum function break aout cell names and add
                                case "sum":
                                    //TODO make sure this is working
                                    if (RegExp(":", "gi").test(tokenString)) {
                                        outstr = breakOutRanges(tokenString, "+");
                                    } else {
                                        outStr = tokenString;
                                    }

                                    break;
                                case "and":
                                    //TODO make sure this is working
                                    if (RegExp(":", "gi").test(tokenString)) {
                                        outstr = breakOutRanges(tokenString, "&&");
                                    } else {
                                        outStr = tokenString;
                                    }

                                    break;
                                case "or":
                                    //TODO make sure this is working
                                    if (RegExp(":", "gi").test(tokenString)) {
                                        outstr = breakOutRanges(tokenString, "||");
                                    } else {
                                        outStr = tokenString;
                                    }

                                    break;
                                    // By Default return an array containing all cell names in array
                                default:
                                    // Create array for ranges
                                    if (RegExp(":", "gi").test(tokenString)) {
                                        outstr = "[" + breakOutRanges(tokenString, ",") + "]";
                                    } else {
                                        outstr = tokenString;
                                    }
                                    //debugger;
                                    break;
                            }

                            break;

                        default:
                            break;
                    }
                    /* falls through */
                default:
                    if (outstr === "") {
                        outstr = typeof directConversionMap[tokenString.toUpperCase()] === "string" ? directConversionMap[tokenString.toUpperCase()] : tokenString;
                    }
                    useTemplate = true;
                    break;
            }

            return {
                tokenString: outstr,
                useTemplate: useTemplate
            };
        };

        var defaultOptions = {
            tmplFunctionStart: '{{token}}(',
            tmplFunctionStop: '{{token}})',
            tmplOperandError: '{{token}}',
            tmplOperandRange: '{{token}}',
            tmplOperandLogical: '{{token}}',
            tmplOperandNumber: '{{token}}',
            tmplOperandText: '"{{token}}"',
            tmplArgument: '{{token}}',
            tmplOperandOperatorInfix: '{{token}}',
            tmplFunctionStartArray: "",
            tmplFunctionStartArrayRow: "{",
            tmplFunctionStopArrayRow: "}",
            tmplFunctionStopArray: "",
            tmplSubexpressionStart: "(",
            tmplSubexpressionStop: ")",
            tmplIndentTab: "\t",
            tmplIndentSpace: " ",
            autoLineBreak: "TOK_SUBTYPE_STOP | TOK_SUBTYPE_START | TOK_TYPE_ARGUMENT",
            trim: true,
            customTokenRender: tokRender
        };

        if (options) {
            options = core.extend(true, defaultOptions, options);
        } else {
            options = defaultOptions;
        }

        var cSharpOutput = formatFormula(formula, options);
        return cSharpOutput;
    };

    /**
     * Both the csharp and javascript are the same when converted, this is just an alias for convert2CSharp. there are some subtle differences such as == vrs ===, this will be addressed in a later version.
     * @memberof excelFormulaUtilities.convert
     * @function
     * @param {string} formula
     * @returns {string}
     */
    excelFormulaUtilities.formula2JavaScript = function(formula, options) {
        return formula2CSharp(formula, options).replace('==', '===');
    };

    /**
     *
     * @memberof excelFormulaUtilities.convert
     * @function
     * @param {string} formula
     * @returns {string}
     */
    excelFormulaUtilities.formula2Python = function(formula, options) {

        //Custom callback to format as c#
        var functionStack = [];

        var tokRender = function(tokenStr, token) {
            var outstr = "",
                outStr,
                /*tokenString = (token.value.length === 0) ? "" : token.value.toString(),*/
                tokenString = tokenStr,
                directConversionMap = {
                    "=": "==",
                    "<>": "!=",
                    "MIN": "min",
                    "MAX": "max",
                    "ABS": "math.fabs",
                    "SUM": "",
                    "IF": "",
                    "&": "+",
                    "AND": "",
                    "OR": "",
                    "NOT": "!",
                    "TRUE": "True",
                    "FALSE": "False"
                },
                currentFunctionOnStack = functionStack[functionStack.length - 1],
                useTemplate = false;

            switch (token.type.toString()) {

                case TOK_TYPE_FUNCTION:

                    switch (token.subtype) {

                        case TOK_SUBTYPE_START:

                            functionStack.push({
                                name: tokenString,
                                argumentNumber: 0
                            });
                            outstr = typeof directConversionMap[tokenString.toUpperCase()] === "string" ? directConversionMap[tokenString.toUpperCase()] : tokenString;
                            useTemplate = true;

                            break;

                        case TOK_SUBTYPE_STOP:

                            useTemplate = true;
                            switch (currentFunctionOnStack.name.toLowerCase()) {
                                case "if":
                                    outstr = ",))[0]";
                                    if (currentFunctionOnStack.argumentNumber === 1) {
                                        outstr = " or (0" + outstr;
                                    }
                                    useTemplate = false;
                                    break;
                                default:
                                    outstr = typeof directConversionMap[tokenString.toUpperCase()] === "string" ? directConversionMap[tokenString.toUpperCase()] : tokenString;
                                    break;
                            }
                            functionStack.pop();
                            break;
                    }

                    break;

                case TOK_TYPE_ARGUMENT:
                    switch (currentFunctionOnStack.name.toLowerCase()) {
                        case "if":
                            switch (currentFunctionOnStack.argumentNumber) {
                                case 0:
                                    outstr = " and (";
                                    break;
                                case 1:
                                    outstr = ",) or (";
                                    break;
                            }
                            break;
                        case "sum":
                            outstr = "+";
                            break;
                        case "and":
                            outstr = " and ";
                            break;
                        case "or":
                            outstr = " or ";
                            break;
                        default:
                            outstr = typeof directConversionMap[tokenString.toUpperCase()] === "string" ? directConversionMap[tokenString.toUpperCase()] : tokenString;
                            useTemplate = true;
                            break;
                    }

                    currentFunctionOnStack.argumentNumber += 1;

                    break;

                case TOK_TYPE_OPERAND:

                    switch (token.subtype) {

                        case TOK_SUBTYPE_RANGE:
                            //Assume '=' sign
                            if (!currentFunctionOnStack) {
                                break;
                            }

                            if (RegExp("true|false", "gi").test(tokenString)) {
                                outstr = typeof directConversionMap[tokenString.toUpperCase()] === "string" ? directConversionMap[tokenString.toUpperCase()] : tokenString;
                                break;
                            }

                            switch (currentFunctionOnStack.name.toLowerCase()) {
                                // If in the sum function break aout cell names and add
                                case "sum":
                                    //TODO make sure this is working
                                    if (RegExp(":", "gi").test(tokenString)) {
                                        outstr = breakOutRanges(tokenString, "+");
                                    } else {
                                        outStr = tokenString;
                                    }

                                    break;
                                case "and":
                                    //TODO make sure this is working
                                    if (RegExp(":", "gi").test(tokenString)) {
                                        outstr = breakOutRanges(tokenString, " and ");
                                    } else {
                                        outStr = tokenString;
                                    }

                                    break;
                                case "or":
                                    //TODO make sure this is working
                                    if (RegExp(":", "gi").test(tokenString)) {
                                        outstr = breakOutRanges(tokenString, " or ");
                                    } else {
                                        outStr = tokenString;
                                    }

                                    break;
                                    // By Default return an array containing all cell names in array
                                default:
                                    // Create array for ranges
                                    if (RegExp(":", "gi").test(tokenString)) {
                                        outstr = "[" + breakOutRanges(tokenString, ",") + "]";
                                    } else {
                                        outstr = tokenString;
                                    }
                                    //debugger;
                                    break;
                            }

                            break;

                        default:
                            break;
                    }
                    /* falls through */

                default:
                    if (outstr === "") {
                        outstr = typeof directConversionMap[tokenString.toUpperCase()] === "string" ? directConversionMap[tokenString.toUpperCase()] : tokenString;
                    }
                    useTemplate = true;
                    break;
            }

            return {
                tokenString: outstr,
                useTemplate: useTemplate
            };
        };

        var defaultOptions = {
            tmplFunctionStart: '{{token}}(',
            tmplFunctionStop: '{{token}})',
            tmplOperandError: '{{token}}',
            tmplOperandRange: '{{token}}',
            tmplOperandLogical: '{{token}}',
            tmplOperandNumber: '{{token}}',
            tmplOperandText: '"{{token}}"',
            tmplArgument: '{{token}}',
            tmplOperandOperatorInfix: '{{token}}',
            tmplFunctionStartArray: "",
            tmplFunctionStartArrayRow: "{",
            tmplFunctionStopArrayRow: "}",
            tmplFunctionStopArray: "",
            tmplSubexpressionStart: "(",
            tmplSubexpressionStop: ")",
            tmplIndentTab: "\t",
            tmplIndentSpace: " ",
            autoLineBreak: "TOK_SUBTYPE_STOP | TOK_SUBTYPE_START | TOK_TYPE_ARGUMENT",
            trim: true,
            customTokenRender: tokRender
        };

        if (options) {
            options = core.extend(true, defaultOptions, options);
        } else {
            options = defaultOptions;
        }

        var pythonOutput = formatFormula(formula, options);

        return pythonOutput;
    };

    excelFormulaUtilities.getTokens = getTokens;

}(window));


// FILE: js/libraries/GeneralMethods.js
function GeneralMethods(localStorageEngine, ZHPromise, ScreenService) {

    var self = this;

    this.salvaUltimoUsuario = function() {
        if (!localStorageEngine.getLocalVar('DEMO')) {
            localStorageEngine.getDataSource('/User').then(function(user) {
                localStorageEngine.setDataSource('LastUser', user);
                limpaUnidadesUsuario();
            });
        }
    };

    function limpaUnidadesUsuario() {
        var promises = [];
        promises.push(localStorageEngine.remove('/User', []));
        promises.push(localStorageEngine.remove('/Units', []));
        return ZHPromise.all(promises);
    }

    function removeDadosLocaisDoUsuario() {
        var promises = [];
        promises.push(localStorageEngine.setLocalVar('__CDOPERADOR', ""));
        promises.push(localStorageEngine.setLocalVar('__CDFILIAL', ""));
        promises.push(localStorageEngine.setLocalVar('__NRORGTRAB', ""));
        return ZHPromise.all(promises);
    }

    this.verificaNovoUsuario = function(CDOPERADOR, EMAILOPERADOR) {
        ScreenService.hideLoader();

        return localStorageEngine.getDataSource('LastUser').then(function(lastUser) {
            var promises = [];

            if (lastUser[0] && lastUser[0].CDOPERADOR !== CDOPERADOR) {
                ScreenService.confirmMessage("Existem dados de outro usuário no sistema, deseja sobrepor?", 'question', function() {

                    return self.limpaDadosImportados().then(function() {
                        localStorageEngine.setLocalVar('UltimoOperador', EMAILOPERADOR);
                        ScreenService.openWindow('Dashboard');
                    });

                }, function() {

                    promises.push(limpaUnidadesUsuario());
                    promises.push(removeDadosLocaisDoUsuario());

                    return ZHPromise.all(promises);

                });

            } else {

                localStorageEngine.setLocalVar('UltimoOperador', EMAILOPERADOR);
                ScreenService.openWindow('Dashboard');
            }
        });
    };

    this.limpaDadosImportados = function() {
        var promises = [];
        var dataSourceNames = ['/ClientesDoServicoEfetivos', '/DatasCardapioEfetivos',
            '/DatasEntregaSolicitacao', '/DatasInventario', '/Justificativas',
            '/Movopsol', '/NovaSolicitacao', '/ProdutosSolicitacao', '/Solicitacao',
            '/ProximoCodigoSolicitacao', '/Produtos', '/TiposCardapioEfetivos',
            '/ServicosCardapioEfetivos', '/UltimoInventario', '/ProximoCodigoInventario',
            '/ProximoCodigoProdutoInventario', '/ProdutosInventario', '/TrocarUnidade',
            '/ProdutoDatasEntregaSolicitacao', '/ProdutosExcluidos', '/Inventarios',
            '/ProdutosEditados', 'LastUser', '/Parametros', '/DataFinalInicial',
            '/InventarioProdutos', '/ServicosEfetivosUnicos', '/CardapioEfetivos'
        ];

        dataSourceNames.forEach(function(name) {
            promises.push(localStorageEngine.remove(name, []));
        });

        localStorageEngine.setLocalVar('ROWNUM', false);
        localStorageEngine.setLocalVar('ROWNUM_INVENT', false);
        localStorageEngine.setLocalVar('ROWNUM_PRODUTO_INVENT', false);
        localStorageEngine.setLocalVar('ROWNUM_LOG', false);

        return ZHPromise.all(promises);
    };

    this.getLang = function() {
        var currentLang = localStorageEngine.getLocalVar('lang');
        var lang;
        if (currentLang === "pt_br") {
            lang = 'BRAZILIAN PORTUGUESE';
        } else if (currentLang === "es_AR") {
            lang = 'SPANISH';
        } else {
            lang = 'BRAZILIAN PORTUGUESE';
        }
        return lang;
    };

    this.converteDataEmObjeto = function(data, spltiString) {
        if (spltiString === '/') {
            return formatoBrasileiroParaObjeto(data);
        } else if (spltiString === '-') {
            return formatoAmericanoParaObjeto(data);
        }
    };

    function formatoBrasileiroParaObjeto(data) {
        var dataSplit = data.split('/');
        var dataObjeto = new Date(dataSplit[2], dataSplit[1] - 1, dataSplit[0]);
        return dataObjeto;
    }

    function formatoAmericanoParaObjeto(data) {
        var dataSplit = data.split('-');
        var dataObjeto = new Date(dataSplit[0], dataSplit[1] - 1, dataSplit[2]);
        return dataObjeto;
    }

    this.retornaDataDeHojeObjeto = function() {
        var date = new Date();
        var dia = date.getDate();
        var mes = date.getMonth();
        var ano = date.getFullYear();

        var hoje = new Date(ano, mes, dia);
        return hoje;
    };

    this.retornaDataHoraDeHojeString = function() {
        var date = new Date();
        var dia = colocaDoisDigitos(date.getDate());
        var mes = colocaDoisDigitos(date.getMonth() + 1);
        var ano = date.getFullYear();
        var hora = colocaDoisDigitos(date.getHours());
        var minutos = colocaDoisDigitos(date.getMinutes());
        var segundos = colocaDoisDigitos(date.getSeconds());

        return ano + "-" + mes + "-" + dia + " " + hora + ":" + minutos + ":" + segundos;
    };

    this.retornaDataDeHojeString = function() {
        var date = new Date();
        var dia = colocaDoisDigitos(date.getDate());
        var mes = colocaDoisDigitos(date.getMonth() + 1);
        var ano = date.getFullYear();

        return dia + "/" + mes + "/" + ano;
    };

    function colocaDoisDigitos(value) {
        if (value < 10) {
            value = '0' + value;
        }
        return value;
    }

    this.valorCorreto = function(value) {
        var index = value.toString().indexOf(',');
        var valueFloat = parseFloat(value);

        /* global Util*/
        if (valueFloat <= 0 || Util.isEmptyOrBlank(value) || isNaN(valueFloat) || index !== -1) {
            return false;
        }
        return true;
    };

    this.valorCorretoMaiorIgualZero = function(value) {
        var index = value.toString().indexOf(',');
        var valueFloat = parseFloat(value);
        if (valueFloat < 0 || Util.isEmptyOrBlank(value) || isNaN(valueFloat) || index !== -1) {
            return false;
        }
        return true;
    };

    this.criarLogMovopsol = function(currentRow, IDOPERAMOVS, QTANTOPER, QTDEPOPER) {

        var ROWNUM = localStorageEngine.getLocalVar('ROWNUM_LOG');

        if (!ROWNUM) {
            ROWNUM = 'L1';
        } else {
            var split = ROWNUM.split('L');
            split[1]++;
            var novaROWNUM = split[1];
            ROWNUM = 'L' + novaROWNUM;
        }

        var objetoMovopsol = {
            ROWNUM: ROWNUM,
            CDFILSOLMOV: currentRow.CDFILSOLI,
            CDFILLANMOV: currentRow.CDFILLAN,
            NRSOLIMOV: currentRow.NRSOLICMP,
            CDPRODUTO: currentRow.CDPRODUTO,
            DTPROGENTR: currentRow.DTPROGENTR,
            DTOPERSOL: this.retornaDataHoraDeHojeString(),
            CDOPERADOR: localStorageEngine.getLocalVar('__CDOPERADOR'),
            DSJUSTOPER: currentRow.DSJUSTOPER || null,
            QTANTOPER: QTANTOPER,
            QTDEPOPER: QTDEPOPER,
            NRORG: currentRow.NRORG,
            IDOPERAMOVS: IDOPERAMOVS,
            changed: true
        };

        localStorageEngine.setLocalVar('ROWNUM_LOG', ROWNUM);

        return localStorageEngine.getDataSource('/Movopsol').then(function(movopsol) {
            if (!movopsol) {
                movopsol = [];
            }

            movopsol.push(objetoMovopsol);

            return localStorageEngine.setDataSource('/Movopsol', movopsol);
        });
    };

    this.concatenaDataComDiaDaSemana = function(data) {
        var dataSplit = data.split('/');
        var date = new Date(dataSplit[2], dataSplit[1] - 1, dataSplit[0]);
        var diaDaSemana = date.getDay();
        if (diaDaSemana === 0) {
            data = data + ' - ' + 'DOMINGO';
        } else if (diaDaSemana === 1) {
            data = data + ' - ' + 'SEGUNDA-FEIRA';
        } else if (diaDaSemana === 2) {
            data = data + ' - ' + 'TERÇA-FEIRA';
        } else if (diaDaSemana === 3) {
            data = data + ' - ' + 'QUARTA-FEIRA';
        } else if (diaDaSemana === 4) {
            data = data + ' - ' + 'QUINTA-FEIRA';
        } else if (diaDaSemana === 5) {
            data = data + ' - ' + 'SEXTA-FEIRA';
        } else if (diaDaSemana === 6) {
            data = data + ' - ' + 'SÁBADO';
        }
        return data;
    };

    this.lastTwoWeeks = function(format) {

        return {
            start: moment().subtract(15, 'days').format(format),
            end: moment().format(format),
            value: "1"
        };
    };

    this.nextTwoWeeks = function(format) {

        return {
            start: moment().format(format),
            end: moment().add(15, 'days').format(format),
            value: "2"
        };
    };

    this.lastThirtyDays = function(format) {

        return {
            start: moment().subtract(30, 'days').format(format),
            end: moment().format(format),
            value: "3"
        };
    };

    this.nextThirtyDays = function(format) {

        return {
            start: moment().format(format),
            end: moment().add(30, 'days').format(format),
            value: "4"
        };
    };

    this.focus = function(selector) {

        setTimeout(function() {

            var elem = document.querySelector(selector);

            if (elem) {
                elem.focus();
                elem.click();
            }

        }, 300);

    };

    this.onlyIntergerNumbers = function(selector) {
        setTimeout(function() {
            var elem = $(selector);

            elem.keypress(function(e) {
                var intergerNumbers = [48, 49, 50, 51, 52, 53, 54, 55, 56, 57];
                var numberCode = e.which;

                if (intergerNumbers.indexOf(numberCode) < 0) {
                    e.preventDefault();
                }
            });

            onpaste(elem, 0);

        }, 300);
    };

    this.onlyFloatNumbers = function(selector) {
        setTimeout(function() {

            var elem = $(selector);

            elem.keypress(function(e) {
                var intergerNumbers = [46, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57];
                var numberCode = e.which;

                if ((intergerNumbers.indexOf(numberCode) < 0) ||
                    (numberCode === 46 && e.currentTarget.value.indexOf(String.fromCharCode(numberCode)) >= 0)) {
                    e.preventDefault();
                }
            });

            onpaste(elem, 0);

        }, 300);
    };

    var onpaste = function(elem, i) {
        if (elem.hasOwnProperty(i)) {
            elem[i].onpaste = function(e) {
                e.preventDefault();
            };
            i++;
            onpaste(i);
        }
    };

    this.onBack = function() {
        if (ScreenService.isPopupOpen()) {
            var popup = ScreenService.getPopupOpen();
            if (popup.getActionsByVisibility('back') &&
                popup.getActionsByVisibility('back').length) {
                popup.getActionsByVisibility('back')[0].click();
            } else {
                ScreenService.closePopup();
            }
        } else if (ScreenService.isWidgetEditing()) {
            ScreenService.stopEditWidget();
        } else if (ScreenService.isSideMenuOpen()) {
            ScreenService.toggleSideMenu();
        } else if (ScreenService.isAlertShown()) {
            // close alert only if there is just the ok button
            var alertContainer = $('.zh-container-alert').parent();
            if (alertContainer.find('.zh-alert-button-ok').length === 1) {
                alertContainer.hide();
            }
        } else if ($(':focus').is('.inline-edit')) {
            // inline editable grid -> navigate
            $(':focus').blur();
        } else if ($('.zh-footer-left .previous:visible').length > 0) {
            // click on the back button if it is visible
            $('.zh-footer-left .previous:visible').click();
        } else {
            ScreenService.goBack();
        }
    };

    this.updateItemsPerPage = function(widget) {
        if (Util.isPhone() || Util.isTablet() || Util.isMobile()) {
            widget.itemsPerPage = 15;
        } else if (Util.isDesktop()) {
            widget.itemsPerPage = 40;
        }
    };

    this.gerarProdutoFiltroSolicitacao = function(produtosSolicitacao) {
        var produtos = [];
        for (var i = 0; i < produtosSolicitacao.length; i++) {
            var produto = {
                CDFILSOLI: produtosSolicitacao[i].CDFILSOLI,
                CDPRODUTO: produtosSolicitacao[i].CDPRODUTO,
                NMPRODUTO: produtosSolicitacao[i].NMPRODUTO
            };
            produtos.push(produto);
        }
        var produtosUnicos = removerProdutosRedundantes(produtos);

        return localStorageEngine.setDataSource("/ProdutoDatasEntregaSolicitacao", produtosUnicos);
    };

    function removerProdutosRedundantes(produtos) {
        var produtosunicos = [];
        var valor = '';

        var pesquisaItem = function(pesquisa, arrayPesquisar) {
            for (var fkey = 0; fkey < arrayPesquisar.length; fkey++) {
                if (arrayPesquisar[fkey].CDPRODUTO === pesquisa.CDPRODUTO &&
                    arrayPesquisar[fkey].CDFILSOLI === pesquisa.CDFILSOLI) {
                    return true;
                }
            }
            return false;
        };

        for (var key = 0; key < produtos.length; key++) {
            valor = produtos[key];
            if (false === pesquisaItem(valor, produtosunicos)) {
                produtosunicos.push(valor);
            }
        }

        return produtosunicos;
    }

    this.setDefaultTab = function(widget) {
        widget.parent.defaultTab = widget.name;
    };

    this.updateActionsAndFields = function(grid, formEdit) {
        var dataset = grid.dataSource.data;
        var currentRow = grid.currentRow;
        var actionNext = formEdit.getAction('next');
        var actionBack = formEdit.getAction('back');
        var fieldSaveAndNext = formEdit.getField('saveAndNext');
        var fieldSaveAndBack = formEdit.getField('saveAndBack');
        var elementSaveAndNext = $('#saveAndNext');
        var elementSaveAndBack = $('#saveAndBack');
        if (dataset.length === 1) {
            actionNext.readOnly = true;
            actionBack.readOnly = true;
            fieldSaveAndNext.readOnly = true;
            fieldSaveAndBack.readOnly = true;
            addClass('#saveAndNext', 'save-and-next-last-item');
            addClass('#saveAndBack', 'save-and-next-last-item');
        } else if ((dataset.length - 1) === dataset.indexOf(currentRow)) {
            actionNext.readOnly = true;
            actionBack.readOnly = false;
            fieldSaveAndNext.readOnly = true;
            fieldSaveAndBack.readOnly = false;
            addClass('#saveAndNext', 'save-and-next-last-item');
            elementSaveAndBack.removeClass('save-and-next-last-item');
        } else if (dataset.indexOf(currentRow) === 0) {
            actionNext.readOnly = false;
            actionBack.readOnly = true;
            fieldSaveAndNext.readOnly = false;
            fieldSaveAndBack.readOnly = true;
            elementSaveAndNext.removeClass('save-and-next-last-item');
            addClass('#saveAndBack', 'save-and-next-last-item');
        } else {
            actionNext.readOnly = false;
            actionBack.readOnly = false;
            fieldSaveAndNext.readOnly = false;
            fieldSaveAndBack.readOnly = false;
            elementSaveAndNext.removeClass('save-and-next-last-item');
            elementSaveAndBack.removeClass('save-and-next-last-item');
        }
    };

    function addClass(elem, cssClass) {
        var elementSaveAndNext = $(elem);
        if (elementSaveAndNext[0]) {
            elementSaveAndNext.addClass(cssClass);
        } else {
            setTimeout(function() {
                elementSaveAndNext = $(elem);
                elementSaveAndNext.addClass(cssClass);
            }, 300);
        }
    }

    this.sortArray = function() {
        var array = arguments[0];
        var orderByColumns = arguments[1];
        var orderByTypes = arguments[2] || [];
        array.sort(function(obj, nextObj) {
            var index = 0;
            return compare(obj, nextObj, orderByColumns, index, orderByTypes);
        });
        return array;
    };

    var orderTypes = {
        "ASC": 1,
        "DESC": -1,
        "SAME": 0
    };

    var compare = function(obj, nextObj, orderByColumns, index, orderByTypes) {
        if (obj[orderByColumns[index]] === nextObj[orderByColumns[index]]) {
            index++;
            if (orderByColumns.hasOwnProperty(index)) {
                return compare(obj, nextObj, orderByColumns, index, orderByTypes);
            } else {
                return orderTypes.SAME;
            }
        } else if (obj[orderByColumns[index]] > nextObj[orderByColumns[index]]) {
            if (orderByTypes.hasOwnProperty(index) && orderTypes.hasOwnProperty(orderByTypes[index])) {
                return orderTypes[orderByTypes[index]];
            } else {
                return orderTypes.ASC;
            }
        } else if (obj[orderByColumns[index]] < nextObj[orderByColumns[index]]) {
            if (orderByTypes.hasOwnProperty(index) && orderTypes.hasOwnProperty(orderByTypes[index])) {
                return orderTypes[orderByTypes[index]] * -1;
            } else {
                return orderTypes.DESC;
            }
        }
    };

}
Configuration(function(ContextRegister) {
    ContextRegister.register('GeneralMethods', GeneralMethods);
});


// FILE: js/util/CRUD.js
var CRUD = function(localStorageEngine, ZHPromise) {
    var self = this;

    this.init = function(dataSourceName) {
        this.dataSourceName = dataSourceName;
        return this;
    };

    this.find = function(filter, findDeleted, dataSourceName) {
        if (findDeleted === undefined || findDeleted === null) {
            findDeleted = false;
        }
        if (dataSourceName) {
            this.dataSourceName = dataSourceName;
        }
        var defer = ZHPromise.defer();
        localStorageEngine.getDataSource(this.dataSourceName, filter).then(
            function success(rows) {
                defer.resolve(rows.filter(function(row) {
                    return (findDeleted || !row.localChanges || !row.localChanges.remove);
                }));
            },
            function error(e) {
                defer.reject(e);
            }
        );
        return defer.promise;
    };

    this.saveRow = function(row, dataSourceName) {
        if (dataSourceName) {
            this.dataSourceName = dataSourceName;
        }
        if (row.localChanges && row.localChanges.remove) {
            delete row.localChanges.remove;
        }
        return localStorageEngine.save(this.dataSourceName, row);
    };
    this.saveDataset = function(dataset, dataSourceName) {
        var promises = [];
        for (var i = 0; i < dataset.length; i++) {
            var rowToSave = dataset[i];
            promises.push(self.saveRow(rowToSave, dataSourceName));
        }
        return ZHPromise.all(promises);
    };

    this.removeRow = function(row, dataSourceName) {
        if (dataSourceName) {
            this.dataSourceName = dataSourceName;
        }
        row.localChanges = row.localChanges || {};
        row.localChanges.remove = true;
        row.changed = true;
        return localStorageEngine.save(this.dataSourceName, row);
    };
    this.removeDataset = function(dataset, dataSourceName) {
        var promises = [];
        for (var i = 0; i < dataset.length; i++) {
            var rowToRemove = dataset[i];
            promises.push(self.removeRow(rowToRemove, dataSourceName));
        }
        return ZHPromise.all(promises);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('CRUD', CRUD);
});


// FILE: js/util/Environment.js
var Environment = function(templateManager, localStorageEngine, ScreenService, ZHPromise, dataSourceHolder, GeneralMethods) {

    var storey;
    var hall;
    var bed;
    var currentPatient;
    var dataSourceMain;
    var currentPrescription;
    var prescriptionDataSource;
    var currentAppointment;
    var appointmentDataSource;

    this.windowReload = function(container) {

        container.eachWidget(function(widget) {

            widget.reload().then(function() {
                templateManager.updateTemplate();
            });

        });

    };

    this.ZHAll = function(promises) {
        return ZHPromise.all(promises);
    };

    // CURRENT PATIENT
    this.setCurrentPatientId = function(id) {
        localStorageEngine.setLocalVar('__patient', id);
    };

    this.getCurrentPatientId = function() {
        return localStorageEngine.getLocalVar('__patient');
    };

    this.getCurrentPatient = function() {
        return currentPatient;
    };

    this.setCurrentPatient = function(patient) {
        currentPatient = patient;
        localStorageEngine.setLocalVar('__hospitalization', patient.hospitalization);
    };

    this.getDataSource = function(name, filtro, page, itemsPerPage) {
        return localStorageEngine.getDataSource(name, filtro, page, itemsPerPage);
    };

    this.getDataSourceMain = function() {
        return dataSourceMain;
    };

    this.setDataSourceMain = function(dataSource) {
        dataSourceMain = dataSource;
    };

    // CURRENT PRESCRIPTION
    this.setCurrentPrescriptionDate = function(date) {
        localStorageEngine.setLocalVar('__prescription', date);
    };

    this.getCurrentPrescription = function() {
        return currentPrescription;
    };

    this.setCurrentPrescription = function(patient) {
        currentPrescription = patient;
    };

    this.getPrescriptionDataSource = function() {
        return prescriptionDataSource;
    };

    this.setPrescriptionDataSource = function(dataSource) {
        prescriptionDataSource = dataSource;
    };

    // CURRENT CONSULTATION
    this.setCurrentAppointmentDate = function(date) {
        localStorageEngine.setLocalVar('__appointment', date);
    };

    this.getCurrentAppointmentDate = function() {
        return localStorageEngine.getLocalVar('__appointment');
    };

    this.getCurrentAppointment = function() {
        return currentAppointment;
    };

    this.setCurrentAppointment = function(patient) {
        currentAppointment = patient;
    };

    this.getAppointmentDataSource = function() {
        return appointmentDataSource;
    };

    this.setAppointmentDataSource = function(dataSource) {
        appointmentDataSource = dataSource;
    };

    // ANTHROPOMETRIC EVALUATION

    this.setEvaluationMethods = function(methods) {
        localStorageEngine.setLocalVar('__evaluationMethods', methods);
    };

    this.getEvaluationMethods = function() {
        return localStorageEngine.getLocalVar('__evaluationMethods');
    };

    this.getLocalVar = function(name) {
        return localStorageEngine.getLocalVar(name);
    };

    this.setLocalVar = function(name, value) {
        localStorageEngine.setLocalVar(name, value);
    };

    this.remove = function(name, value) {
        if (!value) {
            value = [];
        }
        return localStorageEngine.remove(name, value);
    };

    // this.getDataSource = function(name) {
    //     return localStorageEngine.getDataSource(name);
    // };

    this.setLabelToStripe = function(widget) {

        if (widget.name === 'WidgetMain') {
            widget.__filterPopup.fields.forEach(function(field) {

                if (field.name === 'unit') {

                    field.label = ScreenService.i18n("Unidade") + ": " + localStorageEngine.getLocalVar('__CDFILIAL');

                }

                if (field.name === 'storey') {

                    if (field.value()) {
                        templateManager.container.getWidget('WidgetStripe').getField("storey").label = ScreenService.i18n("Andar") + ": " + field.value();
                    } else {
                        templateManager.container.getWidget('WidgetStripe').getField("storey").label = ScreenService.i18n("Andar") + ": " + ScreenService.i18n("Todos");
                    }

                }

                if (field.name === 'hall') {

                    if (field.value()) {
                        templateManager.container.getWidget('WidgetStripe').getField("hall").label = ScreenService.i18n("Ala") + ": " + field.value();
                    } else {
                        templateManager.container.getWidget('WidgetStripe').getField("hall").label = ScreenService.i18n("Ala") + ": " + ScreenService.i18n("Todos");
                    }

                }

                if (field.name === 'bed') {

                    if (field.value()) {
                        templateManager.container.getWidget('WidgetStripe').getField("bed").label = ScreenService.i18n("Leito") + ": " + field.value();
                    } else {
                        templateManager.container.getWidget('WidgetStripe').getField("bed").label = ScreenService.i18n("Leito") + ": " + ScreenService.i18n("Todos");
                    }

                }

                field.reload();

            });
        } else {
            widget.fields.forEach(function(field) {

                if (field.name === 'unit') {

                    field.label = ScreenService.i18n("Unidade") + ": " + localStorageEngine.getLocalVar('__CDFILIAL');

                }

                if (field.name === 'storey') {

                    if (storey) {
                        field.label = ScreenService.i18n("Andar") + ": " + storey;
                    } else {
                        field.label = ScreenService.i18n("Andar") + ": " + ScreenService.i18n("Todos");
                    }

                }

                if (field.name === 'hall') {

                    if (hall) {
                        field.label = ScreenService.i18n("Ala") + ": " + hall;
                    } else {
                        field.label = ScreenService.i18n("Ala") + ": " + ScreenService.i18n("Todos");
                    }

                }

                if (field.name === 'bed') {

                    if (bed) {
                        field.label = ScreenService.i18n("Leito") + ": " + bed;
                    } else {
                        field.label = ScreenService.i18n("Leito") + ": " + ScreenService.i18n("Todos");
                    }

                }

                field.reload();

            });
        }
    };

    this.initStructureParams = function() {

        storey = localStorageEngine.getLocalVar('__storey');
        hall = localStorageEngine.getLocalVar('__hall');
        bed = localStorageEngine.getLocalVar('__bed');

    };

    this.setStructureParams = function(fields) {

        localStorageEngine.setLocalVar('__storey', fields.storey);
        localStorageEngine.setLocalVar('__hall', fields.hall);
        localStorageEngine.setLocalVar('__bed', fields.bed);

    };

    var _prepareFilter = function(filterName, filterValue) {

        return {
            name: filterName,
            operator: '=',
            value: filterValue
        };

    };

    this.setDataSourceFilter = function(widget) {

        widget.dataSourceFilter = [];

        if (storey) {
            widget.dataSourceFilter.push(_prepareFilter('storey', storey));
        }

        if (hall) {
            widget.dataSourceFilter.push(_prepareFilter('hall', hall));
        }

        if (bed) {
            widget.dataSourceFilter.push(_prepareFilter('bed', bed));
        }

        widget.dataSourceFilter.push({
            name: 'situation',
            operator: '=',
            value: 'INTERNADO'
        }, {
            name: 'unit',
            operator: '=',
            value: localStorageEngine.getLocalVar('__CDFILIAL')
        });

        widget.reload().then(function() {
            templateManager.updateTemplate();
        });

    };

    this.afterLogin = function() {
        ScreenService.openWindow('MapNutritionist');
    };

    this.lastPatientVisit = function() {

        var filter = [{
                'name': 'id',
                'operator': '=',
                'value': localStorageEngine.getLocalVar('__patient')
            },
            {
                'name': 'unit',
                'operator': '=',
                'value': localStorageEngine.getLocalVar('__CDFILIAL')
            }
        ];

        localStorageEngine.getDataSource('/Patients', filter).then(function(patient) {
            patient[0].lastVisit = moment().format("DD/MM/YYYY HH:MM:ss");
            localStorageEngine.save('/Patients', patient[0]);

        });

    };

    var ALL_RECORDS = 999999999;

    this.setDataSource = function(name, value) {
        return localStorageEngine.setDataSource(name, value);
    };
    this.save = function(name, object) {
        return this.getDataSource(name, [], null, ALL_RECORDS).then(function(dataset) {
            var clearedName = dataSourceHolder.clearName(name);
            var dataToSave = angular.isArray(object) ? object : [object];
            var willBeSaved = false;
            for (var i = 0; i < dataToSave.length; i++) {
                var newData = dataToSave[i];
                for (var index = 0; index < dataset.length; index++) {
                    var row = dataset[index];
                    if (equalByLocalKey(row, newData, clearedName)) {
                        dataset[index] = newData;
                        willBeSaved = true;
                    }
                }
                if (!willBeSaved) {
                    dataset.push(newData);
                }
            }
            dataset = orderByLocalOrder(name, dataset);
            return this.setDataSource(name, dataset);
        }.bind(this));
    };

    var equalByLocalKey = function(row, object, name) {
        var dataSource = getDataSourceByName(name);
        return dataSource.localKey.every(function(localKey) {
            return row[localKey] === object[localKey];
        });
    };

    var orderByLocalOrder = function(name, dataset) {
        var dataSource = getDataSourceByName(name);
        if (dataSource.localOrder) {
            return GeneralMethods.sortArray(dataset, dataSource.localOrder);
        } else {
            return dataset;
        }
    };

    var getDataSourceByName = function(name) {
        name = dataSourceHolder.clearName(name);
        var dataSources = dataSourceHolder.dataSources;
        if (!dataSources[name]) {
            throw 'dataSource ' + name + 'not found';
        }
        return dataSources[name][name];
    };

    this.saveInServer = function(name, callback, dataSource, retryCount, timeout) {
        return localStorageEngine.saveInServer(name, callback, dataSource, retryCount, timeout);
    };

    this.defer = function() {
        return ZHPromise.defer();
    };

    this.showLoader = function() {
        $rootScope.$broadcast("request.start");
    };
    this.hideLoader = function() {
        $rootScope.$broadcast("request.stop");
    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('Environment', Environment);
});


// FILE: js/util/messages.js
function messages(restEngine) {

    var messages = {};

    /**
     * Global method to get a message by its code
     * @param  {String} code - Code of message
     * @return {String} Message corresponding to the code
     */
    this.getMessage = function(code) {
        if (!messages.hasOwnProperty(code)) {
            return messages.messageNotFound;
        }
        return messages[code];
    };

    /**
     * Get messages file
     * @return {Object} Messages object
     */
    function requestMessagesFile() {
        var messagesPath = "json/messages.json";
        return restEngine.requestMetaData(messagesPath).then(function(response) {
            return response.data;
        });
    }

    /**
     * Set messages object in global messages variable
     * @param {Object} requestedMessages - Messages object
     */
    function setMessages(requestedMessages) {
        messages = requestedMessages;
    }

    /**
     * Create default messages object if the messages file was not found
     */
    function messagesFileNotFound() {
        messages = {
            "messageNotFound": "Message not found"
        };
    }

    window.getMessage = this.getMessage;

    requestMessagesFile()
        .then(setMessages)
        .catch(messagesFileNotFound);
}
/*global Configuration*/
Configuration(function(ContextRegister) {
    ContextRegister.register('Messages', messages);
});


// FILE: js/util/Validation.js
var Validation = function() {

    /*global excelFormulaUtilities*/

    var self = this;

    this.isValidField = function(field) {

        if (field) {
            return true;
        } else {
            return false;
        }

    };

    this.formulaToJS = function(FORMULA) {

        var result = FORMULA.split("AND")
            .join("&")
            .replace(/SEXO/g, 'window.SEXO')
            .replace(/IDADE/g, 'window.IDADE')
            .replace(/RESULTADO/g, 'window.RESULTADO');
        var formulaJS = excelFormulaUtilities.formula2JavaScript(result);

        return eval(formulaJS); // jshint ignore:line

    };

    this.strPad = function(i, l, s) {

        // to use
        // _strPad(12, 5); // returns 00012
        // _strPad('abc', 6, '#'); // returns ###abc

        var o = i.toString();

        if (!s) {
            s = '0';
        }

        while (o.length < l) {
            o = s + o;
        }

        return o;

    };

    this.now = function() {

        var now = new Date();
        var day = now.getDate();
        var month = now.getMonth() + 1;
        var year = now.getFullYear();
        var hours = now.getHours();
        var minutes = now.getMinutes();
        var seconds = now.getSeconds();

        return self.strPad(day, 2) + '/' + self.strPad(month, 2) + '/' + year + ' ' + self.strPad(hours, 2) + ':' + self.strPad(minutes, 2) + ':' + self.strPad(seconds, 2);

    };

    this.getDateObject = function(date) {

        var reggie = /(\d{2})\/(\d{2})\/(\d{4}) (\d{2}):(\d{2}):(\d{2})/;

        var dateArray = reggie.exec(date);

        var dateObject = new Date(
            (+dateArray[3]),
            (+dateArray[2]) - 1,
            (+dateArray[1]),
            (+dateArray[4]),
            (+dateArray[5]),
            (+dateArray[6])
        );

        return dateObject;

    };

    this.getCurrentDateByFormat = function(format) {
        return moment().format(format);
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("Validation", Validation);
});


// FILE: js/repositories/AppointmentRepository.js
Configuration(function(RepositoryFactory, ContextRegister, Query) {
    var AppointmentRepository = RepositoryFactory.factory('/Appointments', 'LOCAL');

    AppointmentRepository.getAppointmentByIdAndDate = function(id, date) {
        return this.findOne(Query.build()
            .where('id').equals(id)
            .where('date').equals(date)
        );
    };

    ContextRegister.register('AppointmentRepository', AppointmentRepository);

});


// FILE: js/repositories/BedsRepository.js
var BedsRepository = function(RepositoryFactory, ContextRegister, Query) {
    var VacantBedsRepository = RepositoryFactory.factory('/BedsPrescriptions', 'LOCAL');
    var OccupiedBedsRepository = RepositoryFactory.factory('/Beds', 'LOCAL');

    this.findFromVacantBeds = function() {
        return VacantBedsRepository.find(Query.build());
    };
    this.findOneFromVacantBeds = function(bedId) {
        return VacantBedsRepository.findOne(Query.build().where('bed').equals(bedId));
    };
    this.saveInVacantBeds = function(bed) {
        return VacantBedsRepository.save(bed);
    };
    this.removeFromVacantBeds = function(bed) {
        return VacantBedsRepository.remove(bed);
    };

    this.findFromOccupiedBeds = function() {
        return OccupiedBedsRepository.find(Query.build());
    };
    this.findOneFromOccupiedBeds = function(bedId) {
        return OccupiedBedsRepository.findOne(Query.build().where('bed').equals(bedId));
    };
    this.saveInOccupiedBeds = function(bed) {
        return OccupiedBedsRepository.save(bed);
    };
    this.removeFromOccupiedBeds = function(bed) {
        return OccupiedBedsRepository.remove(bed);
    };
};
Configuration(function(ContextRegister) {
    ContextRegister.register('BedsRepository', BedsRepository);
});


// FILE: js/repositories/BioEvaluationRepository.js
Configuration(function(RepositoryFactory, ContextRegister) {

    var BioEvaluationRepository = RepositoryFactory.factory('/BioEvaluations', 'LOCAL');
    ContextRegister.register('BioEvaluationRepository', BioEvaluationRepository);

});


// FILE: js/repositories/DataInfoEvaluationsRepository.js
Configuration(function(RepositoryFactory, ContextRegister) {
    var DataInfoEvaluationsRepository = RepositoryFactory.factory('/DataInfoEvaluations', 'LOCAL');
    ContextRegister.register('DataInfoEvaluationsRepository', DataInfoEvaluationsRepository);

});


// FILE: js/repositories/EnteralFeedingsRepository.js
Configuration(function(RepositoryFactory, ContextRegister) {
    var EnteralFeedingsRepository = RepositoryFactory.factory('/EnteralFeedings', 'LOCAL');

    ContextRegister.register('EnteralFeedingsRepository', EnteralFeedingsRepository);

});


// FILE: js/repositories/EvaluationDataRepository.js
Configuration(function(RepositoryFactory, ContextRegister, Query) {
    var EvaluationDataRepository = RepositoryFactory.factory('/EvaluationFields', 'LOCAL');

    EvaluationDataRepository.getMethods = function(method, type) {

        return this.findAll(Query.build()
            .where('method').equals(method)
            .where('type').equals(type)
        );

    };

    ContextRegister.register('EvaluationDataRepository', EvaluationDataRepository);

});


// FILE: js/repositories/EvaluationFieldsRepository.js
Configuration(function(RepositoryFactory, ContextRegister, Query) {
    var EvaluationFieldsRepository = RepositoryFactory.factory('/EvaluationsFields', 'LOCAL');

    EvaluationFieldsRepository.getEvaluationByIdAndDate = function(id, date) {
        return this.find(Query.build()
            .where('id').equals(id)
            .where('date').equals(date)
        );
    };

    ContextRegister.register('EvaluationFieldsRepository', EvaluationFieldsRepository);

});


// FILE: js/repositories/FoodsRepository.js
Configuration(function(RepositoryFactory, ContextRegister, Query) {
    var FoodsRepository = RepositoryFactory.factory('/Foods', 'LOCAL');

    FoodsRepository.getFoods = function() {
        return this.download(Query.build());
    };

    FoodsRepository.getFoodById = function(id) {
        return this.findOne(Query.build().where('id').equals(id));
    };

    FoodsRepository.removeFood = function(food) {
        return this.remove(Query.build().where('id').equals(food.id));
    };

    ContextRegister.register('FoodsRepository', FoodsRepository);

});


// FILE: js/repositories/LactaryFeedingsRepository.js
Configuration(function(RepositoryFactory, ContextRegister) {
    var LactaryFeedingsRepository = RepositoryFactory.factory('/LactaryFeedings', 'LOCAL');

    ContextRegister.register('LactaryFeedingsRepository', LactaryFeedingsRepository);

});


// FILE: js/repositories/LoginRepository.js
Configuration(function(RepositoryFactory, ContextRegister, Query) {
    var LoginRepository = RepositoryFactory.factory('/Login', 'LOCAL');

    LoginRepository.auth = function(email, password) {
        return this.download(Query.build()
            .where('email').equals(email)
            .where('password').equals(password)
        );
    };

    ContextRegister.register('LoginRepository', LoginRepository);

});


// FILE: js/repositories/MapNutritionistRepository.js
Configuration(function(RepositoryFactory, ContextRegister, Query) {
    var MapNutritionistRepository = {}; //RepositoryFactory.factory('/Patients', 'LOCAL');

    MapNutritionistRepository.initDataSource = function() {
        return this.download(Query.build());
    };

    ContextRegister.register('MapNutritionistRepository', MapNutritionistRepository);

});


// FILE: js/repositories/NormalFeedingsRepository.js
Configuration(function(RepositoryFactory, ContextRegister, Query) {
    var NormalFeedingsRepository = RepositoryFactory.factory('/NormalFeedings', 'LOCAL');

    NormalFeedingsRepository.getRow = function(key) {
        return this.findOne(Query.build().where('key').equals(key));
    };

    ContextRegister.register('NormalFeedingsRepository', NormalFeedingsRepository);

});


// FILE: js/repositories/PatientsRepository.js
Configuration(function(RepositoryFactory, ContextRegister, Query) {
    var PatientsRepository = RepositoryFactory.factory('/Patients', 'LOCAL');

    PatientsRepository.getPatients = function() {
        return this.download(Query.build());
    };

    PatientsRepository.getPatientById = function(id) {
        return this.findOne(Query.build().where('id').equals(id));
    };

    ContextRegister.register('PatientsRepository', PatientsRepository);

});


// FILE: js/repositories/PrescriptionRepository.js
Configuration(function(RepositoryFactory, ContextRegister, Query) {
    var PrescriptionRepository = RepositoryFactory.factory('/Prescriptions', 'LOCAL');

    PrescriptionRepository.getPrescription = function(id) {
        return this.findOne(Query.build()
            .where('id').equals(id));
    };

    PrescriptionRepository.getPrescriptions = function(id) {
        return this.find(Query.build()
            .where('id').equals(id));
    };

    ContextRegister.register('PrescriptionRepository', PrescriptionRepository);

});


// FILE: js/repositories/ProfilesRepository.js
Configuration(function(RepositoryFactory, ContextRegister, Query) {
    var ProfilesRepository = RepositoryFactory.factory('/Profiles', 'LOCAL');

    ProfilesRepository.getProfileById = function(id) {
        return this.findOne(Query.build().where('id').equals(id));
    };

    ContextRegister.register('ProfilesRepository', ProfilesRepository);

});


// FILE: js/repositories/ServerRepository.js
Configuration(function(ContextRegister, RepositoryFactory) {
    var ServerRepository = RepositoryFactory.factory('/ServerRepository', 'LOCAL');
    ContextRegister.register('ServerRepository', ServerRepository);
});


// FILE: js/repositories/StructuresRepository.js
Configuration(function(RepositoryFactory, ContextRegister, Query) {
    var StructuresRepository = RepositoryFactory.factory('/Structures', 'LOCAL');

    StructuresRepository.getStructures = function() {
        return this.download(Query.build());
    };

    ContextRegister.register('StructuresRepository', StructuresRepository);

});


// FILE: js/repositories/UserRepository.js
var UserRepository = function(RepositoryFactory) {
    var UserRepository = RepositoryFactory.factory('/User', 'LOCAL');

    this.save = function(user) {
        return UserRepository.save(user);
    };
};
Configuration(function(ContextRegister) {
    ContextRegister.register('UserRepository', UserRepository);
});


// FILE: js/services/AppointmentService.js
var AppointmentService = function(localStorageEngine, AppointmentRepository, Environment, BioEvaluationRepository, ZHPromise, restEngine, ScreenService, metaDataFactory) {

    // EVALUATION DATA
    this.setEvaluationDataMethods = function(methods) {
        localStorageEngine.setLocalVar('__evaluationDataMethods', methods);
    };

    this.addFields = function() {
        return localStorageEngine.getLocalVar('__evaluationDataMethods');
    };

    this.save = function(row) {
        return AppointmentRepository.save(row);
    };

    this.bioEvaluationSave = function(row) {
        return BioEvaluationRepository.save(row);
    };

    this.anthDataEvaluation = function(widget) {

        if (widget.dataSource.checkedRows.length > 0) {

            var fields = [];
            var promise = [];

            promise = widget.dataSource.checkedRows.map(function(checkedRow) {

                var methodsFilter = [];
                if (checkedRow.method === undefined) {
                    methodsFilter = [{
                        "name": "methodEvaluationId",
                        "operator": "=",
                        "value": checkedRow.methodEvaluationId
                    }];
                } else {
                    methodsFilter = [{
                        "name": "methodEvaluationId",
                        "operator": "=",
                        "value": checkedRow.method
                    }];
                }

                return localStorageEngine.getDataSource('/DataEvaluations', methodsFilter).then(function(dataEvaluations) {

                    dataEvaluations.forEach(function(field) {

                        fields.push({
                            id: field.dataEvaluationId,
                            method: field.methodEvaluationId
                        });

                    });

                });

            });

            var _initValue = function(field) {

                var dataInfoFilter = [{
                        "name": "id",
                        "operator": "=",
                        "value": localStorageEngine.getLocalVar('__patient')
                    },
                    {
                        "name": "appointmentDate",
                        "operator": "=",
                        "value": localStorageEngine.getLocalVar('__appointment')
                    },
                    {
                        "name": "dataTypeEvaluationId",
                        "operator": "=",
                        "value": field.id
                    },
                    {
                        "name": "methodEvaluationId",
                        "operator": "=",
                        "value": field.method
                    }
                ];

                return localStorageEngine.getDataSource('/DataInfoEvaluations', dataInfoFilter).then(function(dataInfoEvaluations) {

                    if (dataInfoEvaluations.length > 0) {
                        field.value = dataInfoEvaluations[0].value;
                    } else {
                        field.value = null;
                    }

                });

            };

            var _initName = function(field) {

                var typeFilter = [{
                    "name": "dataTypeEvaluationId",
                    "operator": "=",
                    "value": field.id
                }];

                return localStorageEngine.getDataSource('/DataTypeEvaluations', typeFilter).then(function(dataTypeEvaluations) {
                    field.name = dataTypeEvaluations[0].dataTypeEvaluationName;
                });

            };

            var addField = function() {

                var aux = [];
                var obj = {
                    "isVisible": true,
                    "showOnView": true,
                    "template": "field/text-edit.html",
                    "validations": {
                        "required": true
                    }
                };

                fields.map(function(field) {

                    _initValue(field).then(function() {

                        _initName(field).then(function() {

                            if (aux.indexOf(field.id) === -1) {

                                obj.isVisible = true;
                                aux.push(field.id);

                            } else {
                                obj.isVisible = false;
                            }

                            obj.id = field.method;
                            obj.name = field.id;
                            obj.label = field.name;
                            obj.defaultValue = field.value;

                            metaDataFactory.fieldFactory(obj, widget.widgets[0].widgets[0], true);
                            widget.widgets[0].widgets[0].getField(obj.name).defaultValue = field.value;
                            widget.widgets[0].widgets[0].getField(obj.name).value = field.value;
                            widget.widgets[0].widgets[0].getField(obj.name).applyDefaultValue();

                        });

                    });

                });
                ScreenService.openSwipe(widget.widgets[0]);

            };

            return ZHPromise.all(promise).then(function() {
                return addField();
            });

        }

    };
    this.anthCalculate = function(widget) {
        var antevalselectedRows = widget.selectedRows;
        var antrAvaliationPopup = widget.container.getWidget('antrAvaliationPopup');
        var fields = [];
        var appointmentDateFix = localStorageEngine.getLocalVar('__normalizedAppointment');

        if (antrAvaliationPopup.isValid()) {
            antevalselectedRows.forEach(function(selecteds) {
                widget.localDataSourceLoaded.DataEvaluations.forEach(function(todos) {
                    if (selecteds.methodEvaluationId === todos.methodEvaluationId) {
                        fields.push({
                            id: todos.dataEvaluationId,
                            method: selecteds.methodEvaluationId,
                            value: antrAvaliationPopup.currentRow[todos.dataEvaluationId]
                        });
                    }
                });
            });
            var filtro = [{
                    "name": "date",
                    "operator": "=",
                    "value": localStorageEngine.getLocalVar('__normalizedAppointment')
                },
                {
                    "name": "id",
                    "operator": "=",
                    "value": localStorageEngine.getLocalVar('__patient')
                },
                {
                    "name": "unit",
                    "operator": "=",
                    "value": localStorageEngine.getLocalVar('__CDFILIAL')
                }
            ];
            localStorageEngine.getDataSource('/Appointments', filtro).then(function(appointments) {

                appointments[0].changed = true;
                if (antrAvaliationPopup.currentRow['01']) {
                    appointments[0].weight = antrAvaliationPopup.currentRow['01'];
                }
                if (antrAvaliationPopup.currentRow['02']) {
                    appointments[0].height = antrAvaliationPopup.currentRow['02'];
                }

                localStorageEngine.save('/Appointments', appointments).then(function() {
                    widget.curPatient.Appointments.forEach(function(aux) {
                        if (aux.normalizedDate === appointments[0].date) {
                            aux.changed = true;
                            if (antrAvaliationPopup.currentRow['01']) {
                                aux.weight = antrAvaliationPopup.currentRow['01'];
                            }
                            if (antrAvaliationPopup.currentRow['02']) {
                                aux.height = antrAvaliationPopup.currentRow['02'];
                            }
                        }
                    });
                });
            });

            var promises = [];
            var toSave = [];
            fields.forEach(function(field) {
                var filter = [{
                        "name": "id",
                        "operator": "=",
                        "value": localStorageEngine.getLocalVar('__patient')
                    },
                    {
                        "name": "appointmentDate",
                        "operator": "=",
                        "value": appointmentDateFix
                    },
                    {
                        "name": "dataTypeEvaluationId",
                        "operator": "=",
                        "value": field.id
                    },
                    {
                        "name": "methodEvaluationId",
                        "operator": "=",
                        "value": field.method
                    }
                ];
                promises.push(localStorageEngine.getDataSource('/DataInfoEvaluations', filter).then(function(dataInfoEvaluations) {
                    var now = new Date();
                    var key = now.getTime();
                    if (dataInfoEvaluations.length > 0) {
                        if (dataInfoEvaluations[0].value !== field.value) {
                            dataInfoEvaluations[0].changed = true;
                            dataInfoEvaluations[0].value = field.value;
                            dataInfoEvaluations[0].__is_new = true;
                            dataInfoEvaluations[0].key = key;
                            toSave.push(dataInfoEvaluations[0]);
                        }
                    } else {
                        if (field.value !== null) {
                            dataInfoEvaluations.push({
                                __is_new: true,
                                changed: true,
                                id: localStorageEngine.getLocalVar('__patient'),
                                appointmentDate: appointmentDateFix,
                                dataTypeEvaluationId: field.id,
                                key: key,
                                methodEvaluationId: field.method,
                                unit: localStorageEngine.getLocalVar('__CDFILIAL'),
                                value: field.value
                            });
                            toSave.push(dataInfoEvaluations[0]);
                        }
                    }
                }));
            });
            return ZHPromise.all(promises).then(function() {

                return localStorageEngine.saveInServer('/Appointments').then(function() {

                    return localStorageEngine.save('/DataInfoEvaluations', toSave).then(function() {

                        return localStorageEngine.saveInServer('/AnthEvaluationMethods').then(function() {

                            return restEngine.post('/DataInfoEvaluations/save', {
                                "requestType": "DataSet",
                                "dataset": toSave
                            }).then(function() {

                                var ids = "";
                                for (var i = 0; i < fields.length; i++) {
                                    if (ids.search(fields[i].method) === -1) {
                                        ids = ids.concat("_", fields[i].method);
                                    }
                                }
                                ids += "_";
                                var patient = widget.curPatient;
                                restEngine.post('/AnthEvaluationResult', {
                                    "requestType": "Row",
                                    "row": {
                                        "id": patient.id,
                                        "unit": patient.unit,
                                        "age": patient.age,
                                        "yearAndMonth": patient.yearAndMonth,
                                        "appointmentDate": appointmentDateFix,
                                        "avaliations": ids
                                    }
                                }, function(success) {
                                    var deferred = ZHPromise.defer();
                                    if (success.dataset.error) {
                                        ScreenService.showMessage(ScreenService.i18n(success.dataset.error.GENERIC_MESSAGE));
                                        deferred.reject();
                                        return deferred;
                                    } else {
                                        deferred.resolve();
                                        return restEngine.post('/Structures', {
                                            "requestType": "Row",
                                            "row": {
                                                "unitID": localStorageEngine.getLocalVar('__CDFILIAL'),
                                                "nutritionistID": localStorageEngine.getLocalVar('__CDOPERADOR')
                                            }
                                        }, function() {
                                            antrAvaliationPopup.resultadoAuth = success.dataset.Result;
                                        }, undefined, function(error) {
                                            console.log(error);
                                        });
                                    }
                                }, undefined, function(error) {
                                    console.log(error);
                                });
                            });
                        });
                    });
                });
            });
        }
    };

    this.saveCheckedRows = function(checkedRows) {

        var rows = [];

        checkedRows.map(function(checkedRow) {

            if (checkedRow.__isSelected === undefined) {
                checkedRow.__isSelected = false;
            }

            checkedRow.changed = true;
            checkedRow.id = localStorageEngine.getLocalVar('__patient');
            checkedRow.unit = localStorageEngine.getLocalVar('__CDFILIAL');
            checkedRow.__is_new = true;
            checkedRow.appointmentDate = checkedRow.date;

            rows.push(checkedRow);

        });
        var count = 0;

        var save = function() {
            if (count < rows.length) {
                return localStorageEngine.save('/AnthEvaluationMethods', rows).then(function() {
                    count++;
                    return save();
                });
            }
        };

        return ZHPromise.all(rows).then(function() {
            return save();
        });

    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('AppointmentService', AppointmentService);
});


// FILE: js/services/AuthService.js
function authService(restEngine) {

    /**
     * Validate session using the token in cookie
     * @return {Promise} Validate session response
     */
    this.validateSession = function() {
        return restEngine.request('/validate_session', {
            "requestType": "Empty"
        });
    };
}
Configuration(function(ContextRegister) {
    ContextRegister.register("AuthService", authService);
});


// FILE: js/services/BedsService.js
var BedsService = function(BedsRepository, ZHPromise) {

    this.getVacantBeds = function() {
        return BedsRepository.findFromVacantBeds();
    };
    this.getOccupiedBeds = function() {
        return BedsRepository.findFromOccupiedBeds();
    };

    this.getAllBeds = function() {
        var defer = ZHPromise.defer();
        ZHPromise.all([
            BedsRepository.findFromVacantBeds(),
            BedsRepository.findFromOccupiedBeds()
        ]).then(
            function success(values) {
                var vacantBeds = values[0] || [];
                var occupiedBeds = values[1] || [];
                var allBeds = vacantBeds.concat(occupiedBeds);
                defer.resolve(allBeds);
            },
            function error(e) {
                defer.reject(e);
            }
        );
        return defer.promise;
    };

    this.getBed = function(bed) {
        var defer = ZHPromise.defer();
        ZHPromise.all([
            BedsRepository.findOneFromVacantBeds(bed),
            BedsRepository.findOneFromOccupiedBeds(bed)
        ]).then(
            function success(values) {
                var vacantBed = values[0] || [];
                var occupiedBed = values[1] || [];
                if (vacantBed && vacantBed.bed === bed) {
                    defer.resolve(vacantBed);
                } else if (occupiedBed && occupiedBed.bed === bed) {
                    defer.resolve(occupiedBed);
                } else {
                    defer.resolve([]);
                }
            },
            function error(e) {
                defer.reject(e);
            }
        );
        return defer.promise;
    };

    this.localSwapBeds = function(vacantBedToOccupy, occupiedBedToVacate) {
        var vacantBed = Object.assign({}, vacantBedToOccupy);
        var occupiedBed = Object.assign({}, occupiedBedToVacate);

        return ZHPromise.all([
            BedsRepository.removeFromVacantBeds(vacantBed).then(function() {
                BedsRepository.saveInVacantBeds(occupiedBed);
            }),
            BedsRepository.removeFromOccupiedBeds(occupiedBed).then(function() {
                BedsRepository.saveInOccupiedBeds(vacantBed);
            })
        ]);
    };

    this.changeVacantBedToOccupied = function(vacantBedToOccupy) {
        var vacantBed = Object.assign({}, vacantBedToOccupy);

        return ZHPromise.all([
            BedsRepository.removeFromVacantBeds(vacantBed).then(function() {
                BedsRepository.saveInOccupiedBeds(vacantBed);
            })
        ]);
    };

    this.changeOccupiedBedToVacate = function(occupiedBedToVacate) {
        var occupiedBed = Object.assign({}, occupiedBedToVacate);

        return ZHPromise.all([
            BedsRepository.removeFromOccupiedBeds(occupiedBed).then(function() {
                BedsRepository.saveInVacantBeds(occupiedBed);
            })
        ]);
    };
};
Configuration(function(ContextRegister) {
    ContextRegister.register('BedsService', BedsService);
});


// FILE: js/services/ControlFeedingService.js
var ControlFeedingService = function(localStorageEngine, EnteralFeedingsRepository, LactaryFeedingsRepository, NormalFeedingsRepository, ZHPromise, Validation) {

    var SERVICES = {
        NORMAL: "0",
        LACTARY: "1",
        ENTERAL: "2",
        ROOM_SERVICE: "3",
        ESPECIAL: "4"
    };

    this.save = function(currentRow) {

        if (currentRow.serviceId === SERVICES.ENTERAL) {
            return EnteralFeedingsRepository.save(currentRow);
        }

        if (currentRow.serviceId === SERVICES.LACTARY) {
            return LactaryFeedingsRepository.save(currentRow);
        }

        if (currentRow.service === SERVICES.NORMAL || currentRow.serviceId === SERVICES.NORMAL) {
            return NormalFeedingsRepository.save(currentRow);
        }

    };

    this.saveCheckedRowsNormal = function(checkedRows) {
        return localStorageEngine.save('/NormalFeedings', checkedRows);
    };

    this.saveCheckedRows = function(checkedRows) {

        var rows = [];

        checkedRows.map(function(checkedRow) {

            if (checkedRow.__isSelected === undefined) {
                checkedRow.__isSelected = false;
            }

            rows.push(checkedRow);

        });

        var count = 0;

        var save = function() {

            if (count < rows.length) {

                return NormalFeedingsRepository.save(rows[count]).then(function() {
                    count++;
                    return save();
                });

            }

        };

        return ZHPromise.all(rows).then(function() {
            return save();
        });

    };

    this.saveControlFeeding = function(widget, isNew) {
        var currentRow;
        if (!isNew) {
            currentRow = angular.copy(widget.container.getWidget('WidgetControlFeeding').currentRow);
        } else {
            currentRow = widget.currentRow;
        }

        var now = new Date();
        var key = now.getTime();
        var serviceId = '';
        var serviceName = '';
        if (currentRow.service === SERVICES.LACTARY) {
            serviceId = SERVICES.ENTERAL;
            serviceName = "ENTERAL";
        }

        if (currentRow.service === SERVICES.ENTERAL) {
            serviceId = SERVICES.LACTARY;

            serviceName = "LACTARIO";
        }

        if ((currentRow.service === SERVICES.NORMAL) || (currentRow.serviceId === SERVICES.NORMAL)) {
            serviceId = SERVICES.NORMAL;
            serviceName = "NORMAL";

            var objects = [];
            var unique = 0;
            var rows = angular.element('#footer').scope().templateManager.container.getWidget('WidgetNormalFeedingSwipe').dataSource.data;

            if (currentRow.__is_new) {
                rows.forEach(function(row) {

                    if (row.__isSelected === undefined) {
                        row.__isSelected = false;
                    }

                    unique++;
                    var key = rows.length + 1;

                    objects.push({
                        "id": localStorageEngine.getLocalVar('__patient'),
                        "unit": row.unit,
                        "serviceId": currentRow.serviceId,
                        "key": key,
                        "unique": Validation.strPad(unique, 4),
                        "plateName": row.plateName,
                        "plateTypeName": row.plateTypeName,
                        "serviceDate": currentRow.date,
                        "plateNumbers": "1",
                        "plateAuxId": row.plateAuxId,
                        "formulationId": row.formulationId,
                        "observation": null,
                        "dietId": currentRow.dietId,
                        "changed": true,
                        "__isSelected": row.__isSelected,
                        "isSelected": row.__isSelected
                    });

                });

                this.saveCheckedRowsNormal(objects);
            } else {
                rows.forEach(function(row) {

                    if (row.__isSelected === true) {
                        row.isSelected = true;
                    }
                    row.changed = true;

                });

                return this.saveCheckedRowsNormal(rows);
            }

        }

        if (currentRow.__is_new) {
            var obj = {
                __is_new: true,
                changed: true,
                dietName: currentRow.dietName,
                dietId: currentRow.dietId,
                id: localStorageEngine.getLocalVar('__patient'),
                sequential: key,
                service: currentRow.serviceName,
                serviceDate: currentRow.date,
                dietGroup: currentRow.dietGroup,
                serviceId: serviceId,
                serviceCode: currentRow.serviceId,
                serviceName: serviceName,
                unit: localStorageEngine.getLocalVar('__CDFILIAL'),
                nutritionist: localStorageEngine.getLocalVar('__CDOPERADOR'),
                hospitalization: localStorageEngine.getLocalVar('__hospitalization')
            };
            isNew = false;
            return localStorageEngine.save('/ControlFeedings', obj);
        }
    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('ControlFeedingService', ControlFeedingService);
});


// FILE: js/services/DashboardService.js
function DashboardService(Environment) {

    this.buildLogItem = function(container) {
        return Environment.getDataSource('/LogSolicitacao', [], null, 999999).then(function(logSolicitacao) {
            var widgetLog = container.getWidget('logSolicitacoes');
            if (logSolicitacao.length) {
                var firstRow = logSolicitacao[0];
                widgetLog.getField('INFO').label = "Último erro";
                widgetLog.dataSource.data = [{
                    INFO: firstRow.DATAHORA
                }];
            } else {
                widgetLog.getField('INFO').label = "Nenhum erro encontrado";
                widgetLog.dataSource.data = [];
            }
        });
    };

    this.buildUnitItem = function(container) {
        var template = "widget/dashboard/grid.html";
        var highlight = getLocalVar('__CDFILIAL') + ' - ' + getLocalVar('__NMFILIAL');
        var id = 123;
        var name = "unidade";
        var label = "Unidade Logada";
        var color = 13;
        var width = 3;
        var events = [{
            "name": "WidgetOnDashboardClick",
            "code": "TrocarUnidadeController.inicializaTrocaUnidade();"
        }];

        return getUnitItemData().then(function(data) {

            var showHeader;
            var dataset;
            var height;

            if (!data.length) {

                showHeader = false;
                height = 1;
                dataset = [{
                    INFO: "Nenhuma informação a ser exibida"
                }];

            } else if (data.length === 1) {

                dataset = data;
                showHeader = true;
                height = 1;

            } else {

                dataset = data;
                showHeader = true;
                height = 2;

            }

            var item = buildItem(id, name, label, color, width, height, showHeader, dataset, template, highlight, events, container);

            var existItem = container.widgets.filter(function(widget) {
                return widget.name === name;
            })[0];

            if (existItem) {

                container.widgets.splice(container.widgets.indexOf(existItem), 1);

            }
            return container.widgets.push(item);

        });

    };

    this.buildEffectiveItem = function() {
        // creating a way to show those informations
    };

    var buildItem = function(id, name, label, color, width, height, showHeader, dataset, template, highlight, events, container) {

        return metaDataFactory.widgetFactory({
            "id": id,
            "name": name,
            "color": color,
            "width": width,
            "height": height,
            "isVisible": true,
            "showHeader": showHeader,
            "label": label,
            "highlight": highlight,
            "template": template,
            "events": events,
            "container": container,
            "fields": [{
                "id": 1,
                "name": "INFO",
                "label": "Informações",
                "align": "left"
            }],
            "dataSource": {
                "data": dataset
            }
        });

    };

    var getUnitItemData = function() {

        var data = [];
        var defaultData = [{
            INFO: "Nenhuma informação a ser exibida"
        }];
        var filtro = [{
            name: "CDFILIAL",
            operator: "=",
            value: getLocalVar('__CDFILIAL')
        }];

        return Environment.getDataSource('/Parametros', filtro, null, 999999).then(function(parametros) {

            if (getLocalVar('importacaoRealizada')) {

                if (getLocalVar('__CDFILIAL') === getLocalVar('__CDFILCENTPME')) {

                    data.push({
                        INFO: 'É uma central de merenda'
                    });

                } else if (!getLocalVar('__CDFILCENTPME')) {

                    data.push({
                        INFO: 'Não possui central de merenda'
                    });

                }

                if (parametros.length) {

                    if (parametros[0].IDUTILALMOX === 'S') {

                        data.push({
                            INFO: 'Possui controle de estoque por almoxarifado'
                        });
                    }

                    if (parametros[0].IDUTILLOTE === 'S') {

                        data.push({
                            INFO: 'Possui controle de estoque por lote'
                        });
                    }
                }

            } else {

                data = defaultData;
            }

            return data;

        });
    };

}
Configuration(function(ContextRegister) {
    ContextRegister.register('DashboardService', DashboardService);
});


// FILE: js/services/EvaluationDataService.js
var EvaluationDataService = function(localStorageEngine) {

    this.setMethods = function(methods) {
        localStorageEngine.setLocalVar('__evaluationDataMethods', methods);
    };

    this.getFields = function() {

        localStorageEngine.getLocalVar('__evaluationDataMethods');

        localStorageEngine.getDataSource('/EvaluationFields');

    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('EvaluationDataService', EvaluationDataService);
});


// FILE: js/services/LoginService.js
var LoginService = function(LoginRepository, Environment, ZHPromise, $rootScope) {

    var USER_ID = "__CDOPERADOR";
    var USER_NAME = "__NMOPERADOR";
    var BRANCH_OFFICE_ID = "__CDFILIAL";
    var ORGANIZATION_ID = "__NRORGTRAB";
    var USER_MAIL = "__DSEMAILOPER";

    this.getCurrentUser = function() {

        return getLocalVar(USER_ID);
    };

    this.auth = function(email, password) {
        return LoginRepository.auth(email, password).then(function(session) {
            return _initCurrentUser(session.User).then(function() {
                return session;
            }.bind(this));
        });
    };

    var _initCurrentUser = function(user) {

        var promises = [];

        promises.push(setLocalVar(USER_ID, user.CDOPERADOR));
        promises.push(setLocalVar(USER_NAME, user.NMOPERADOR));
        promises.push(setLocalVar(BRANCH_OFFICE_ID, user.CDFILIAL));
        promises.push(setLocalVar(ORGANIZATION_ID, user.NRORG));
        promises.push(setLocalVar(USER_MAIL, user.DSEMAILOPER));

        return ZHPromise.all(promises);

    };

    this.login = function(session) {

        return _initCurrentUser(session.User).then(function() {
            return Environment.afterLogin(session);
        });

    };

    /**
     * Remove user data from local storage
     */
    this.removeUserData = function() {
        localStorage.clear();
    };

    this.logout = function() {

        var promises = [];

        promises.push(setLocalVar(USER_ID, ""));
        promises.push(setLocalVar(USER_NAME, ""));
        promises.push(setLocalVar(BRANCH_OFFICE_ID, ""));
        promises.push(setLocalVar(ORGANIZATION_ID, ""));

        return ZHPromise.all(promises);

    };

    this.setLanguage = function(lang) {

        $rootScope.$broadcast("__words.change", lang);

    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('LoginService', LoginService);
});


// FILE: js/services/MapKitchenMaidService.js
var MapKitchenMaidService = function(localStorageEngine) {
    this.saveFoods = function(food, index) {
        return localStorageEngine.getDataSource('/Foods', []).then(function(foods) {
            foods[index] = food;
            return localStorageEngine.save("/Foods", foods);
        });
    };
};
Configuration(function(ContextRegister) {
    ContextRegister.register('MapKitchenMaidService', MapKitchenMaidService);
});


// FILE: js/services/MapNutritionistService.js
var MapNutritionistService = function(PatientsRepository, BedsService, ZHPromise, Validation, localStorageEngine) {

    var DISCHARGE = 0;
    var DEATH = 1;

    this.initDataSource = function() {

        return PatientsRepository.saveInServer().then(function() {
            return PatientsRepository.initDataSource();
        });

    };

    this.getPatientById = function(patientId) {
        return PatientsRepository.getPatientById(patientId).then(function(patient) {
            return patient;
        });
    };

    var _patientSituation = function(checkedPatients, situation) {

        var patients = [];
        var promises = checkedPatients.map(function(patient) {
            return PatientsRepository.getPatientById(patient.id)
                .then(function(patient) {

                    patient.dischargeDate = Validation.now();
                    patient.situation = situation;
                    patient.changed = true;
                    patient.oldSituation = situation;
                    if (situation === DEATH) {
                        patient.isDead = true;
                    }
                    patients.push(patient);

                });

        });

        var currentPatient = 0;

        var updateCurrentPatient = function() {

            if (currentPatient < patients.length) {
                BedsService.getBed(patients[currentPatient].bed).then(function(bed) {
                    patients[currentPatient].oldBed = bed;
                });
                return PatientsRepository.save(patients[currentPatient]).then(function() {
                    currentPatient++;
                    return updateCurrentPatient();
                });

            }
        };

        return ZHPromise.all(promises).then(function() {
            return updateCurrentPatient();
        });

    };

    this.confirmationDischarge = function(patientsDischarge) {
        return _patientSituation(patientsDischarge, DISCHARGE);
    };

    this.confirmationDeath = function(patientsDeath) {
        return _patientSituation(patientsDeath, DEATH);
    };

    this.confirmationPatientStatus = function(patientData) {
        return _patientSituation(patientData.patient, patientData.situation);
    };

    this.savePrescription = function(rowsToSave) {

        return localStorageEngine.save("/ControlFeedings", rowsToSave);

    };

    this.saveOralPrescriptionNormalFeedings = function(rowsToSave, changed) {
        if (changed === false) {
            if (Array.isArray(rowsToSave)) {
                rowsToSave.map(function(row) {
                    row.changed = false;
                    row.__is_new = false;
                });
            }
        }
        return localStorageEngine.save("/NormalFeedings", rowsToSave);
    };

    this.saveLactaryFeeding = function(rowsToSave, changed) {
        if (changed === false) {
            if (Array.isArray(rowsToSave)) {
                rowsToSave.map(function(row) {
                    row.changed = false;
                    row.__is_new = false;
                });
            }
        }
        return localStorageEngine.save("/LactaryFeedings", rowsToSave);

    };

    this.saveEnteralFeeding = function(rowsToSave) {

        localStorageEngine.save("/EnteralFeedings", rowsToSave);

    };

    this.savePatient = function(rowsToSave) {
        return localStorageEngine.save("/Patients", rowsToSave);
    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('MapNutritionistService', MapNutritionistService);
});


// FILE: js/services/PrescriptionService.js
var PrescriptionService = function(localStorageEngine, PrescriptionRepository, PatientsRepository, LactaryFeedingsRepository) {

    this.save = function(prescription) {
        PrescriptionRepository.save(prescription);
    };

    this.saveLactaryPrescription = function(newLactaryPrescription) {

        LactaryFeedingsRepository.save(newLactaryPrescription);

    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('PrescriptionService', PrescriptionService);
});


// FILE: js/services/ProfileService.js
var ProfileService = function(ProfilesRepository) {

    this.save = function(row) {

        return ProfilesRepository.getProfileById(row.id).then(function(profile) {

            profile.confidentiality = row.confidentiality;
            profile.eatingHabits = row.eatingHabits;
            profile.foodIntolerance = row.foodIntolerance;
            profile.observation = row.observation;
            profile.lastVisit = moment().format("DD/MM/YYYY HH:MM:ss");
            profile.changed = true;

            return ProfilesRepository.save(profile);

        });

    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('ProfileService', ProfileService);
});


// FILE: js/services/ServerService.js
function ServerService(Query, ServerRepository, Environment) {

    // var FRONT_END_VERSION = '1.0';
    var messages = {
        OK: {
            message: "Conexão realizada com sucesso!",
            type: "information"
        },
        NEW_VERSION: {
            message: "Existe uma nova versão disponível da aplicação. Acesse a loja e atualize!",
            type: "information"
        },
        SERVER_OUT: {
            message: "Servidor desatualizado. Favor entrar em contato com o administrador.",
            type: "error"
        }
    };

    this.testConnection = function() {
        var query = Query.build();
        return ServerRepository.download(query);
    };

    this.verifieVersionStatus = function() {

        return Environment.getDataSource('/connection', [], null, 999999).then(function(connection) {

            if (connection.length !== 0) {

                return this.compareVersion(connection[0].version);

            } else {

                return this.testConnection().then(function(connection) {

                    return this.compareVersion(connection[0].version);

                }.bind(this));
            }

        }.bind(this));

    };

    this.compareVersion = function(version) {

        var status = "OK";

        var frontEndVersion = this.getFrontEndVersion();

        if (parseFloat(version) > parseFloat(frontEndVersion)) {

            status = "NEW_VERSION";

        } else if (parseFloat(version) < parseFloat(frontEndVersion)) {

            status = "SERVER_OUT";

        }

        return status;
    };

    this.getFrontEndVersion = function() {
        return window.applicationVersion ? window.applicationVersion : '';
    };

    this.getObjectMessage = function(messageId) {
        return messages[messageId];
    };
}
Configuration(function(ContextRegister) {
    ContextRegister.register('ServerService', ServerService);
});


// FILE: js/services/SynchronizeService.js
var SynchronizeService = function(restEngine, $window, localStorageEngine, StructuresRepository, PatientsRepository, ZHPromise, ScreenService) {

    var _initStructures = function(unitID, widget) {
        var filterFunc = function(data) {
            return data.id === widget.curPatient.id;
        };
        var nutritionistID = getLocalVar('__CDOPERADOR');
        return restEngine.post('/Structures', {
            "requestType": "Row",
            "row": {
                "unitID": unitID,
                "nutritionistID": nutritionistID
            }
        }, null, null, null, null, 600000).then(function(success) {

            for (var key in success.dataset) {

                var name = '/' + key;
                var value = success.dataset[key];

                localStorageEngine.setDataSource(name, value);
                if (widget && widget.localDataSourceLoaded) {
                    var nameDataSourceLoaded = name.replace('/', '');
                    widget.localDataSourceLoaded[nameDataSourceLoaded] = angular.copy(value);
                    var dataToPatient = widget.localDataSourceLoaded[nameDataSourceLoaded].filter(filterFunc);
                    widget.curPatient[nameDataSourceLoaded] = dataToPatient;
                }

            }

        });

    };

    var _initPatients = function(unitID, widget) {
        var filterFunc = function(data) {
            return data.id === widget.curPatient.id;
        };
        var nutritionistID = getLocalVar('__CDOPERADOR');

        return restEngine.post('/Patients', {
            "requestType": "Row",
            "row": {
                "unitID": unitID,
                "nutritionistID": nutritionistID
            }
        }, null, null, null, null, 600000).then(function(success) {

            for (var key in success.dataset) {

                var name = '/' + key;
                var value = success.dataset[key];

                localStorageEngine.setDataSource(name, value);
                if (widget && widget.localDataSourceLoaded) {
                    var nameDataSourceLoaded = name.replace('/', '');
                    widget.localDataSourceLoaded[nameDataSourceLoaded] = angular.copy(value);
                    var dataToPatient = widget.localDataSourceLoaded[nameDataSourceLoaded].filter(filterFunc);
                    widget.curPatient[nameDataSourceLoaded] = dataToPatient;
                }
            }

        });
    };

    var _save = function(dataSet, dataSetsParentErrorDelete, conditionIdentification) {
        return localStorageEngine.getDataSource(dataSet).then(
            function success(rows) {
                var rowsToSave = [];
                var rowsToRemove = [];
                for (var i = 0; i < rows.length; i++) {
                    var row = rows[i];
                    if (!row.successToRemoveOnline) {
                        if (row.localChanges && row.localChanges.remove) {
                            rowsToRemove.push(row);
                        } else {
                            rowsToSave.push(row);
                        }
                    }
                }
                var promises = [];

                if (rowsToSave.length) {
                    promises.push(
                        localStorageEngine.saveInServer(dataSet, null, rowsToSave)
                    );
                }
                rowsToRemove = clearInconsistenciesToDelete(dataSetsParentErrorDelete, rowsToRemove, conditionIdentification);
                if (rowsToRemove.length) {
                    promises.push(
                        localStorageEngine.removeFromServer(dataSet, null, rowsToRemove)
                    );
                }

                return ZHPromise.all(promises);
            }
        );
    };

    function isValidArray(array) {
        return Array.isArray(array) && array.length;
    }

    function replaceAllStr(string, oldStr, newStr) {
        if (typeof string === 'string' && typeof oldStr === 'string' && typeof newStr === 'string') {
            return string.split(oldStr).join(newStr);
        }
        return string;
    }

    function clearInconsistenciesToDelete(dataSetsParentErrorDelete, dataSetChildren, conditionIdentification) {
        if (isValidArray(dataSetsParentErrorDelete) && isValidArray(dataSetChildren) && conditionIdentification && conditionIdentification.IDENTIFICATION) {
            conditionIdentification = conditionIdentification.IDENTIFICATION;
            var deletePropagatesDataSetChildren = [];
            deletePropagatesDataSetChildren = dataSetChildren.filter(function(children) {
                return !children.manualDelete;
            });
            if (isValidArray(deletePropagatesDataSetChildren)) {
                deletePropagatesDataSetChildren.filter(function(childrenToDelete) { // jshint ignore:line
                    var foundedRelateParentError;
                    try {
                        foundedRelateParentError = dataSetsParentErrorDelete.filter(function(errorToDeleteParent) { // jshint ignore:line
                            var bkpCondition = conditionIdentification;
                            bkpCondition = replaceAllStr(bkpCondition, 'PARENT', 'errorToDeleteParent');
                            bkpCondition = replaceAllStr(bkpCondition, 'CHILDREN', 'childrenToDelete');
                            return eval(bkpCondition); // jshint ignore:line
                        })[0];
                        if (foundedRelateParentError) {
                            childrenToDelete.inconsistenceToDelete = true;
                        }
                    } catch (error) {
                        return null;
                    }
                });
            }
        }
        return dataSetChildren;
    }

    var _get = function(widget) {
        var unitID = localStorageEngine.getLocalVar('__CDFILIAL');

        var promises = [];

        if (widget) {
            promises.push(_initStructures(unitID, widget));
            promises.push(_initPatients(unitID, widget));
        } else {
            promises.push(_initStructures(unitID));
            promises.push(_initPatients(unitID));
        }
        return ZHPromise.all(promises);

    };

    this.synchronizedIntolerance = function(widget, filter) {
        var patient = widget.curPatient;
        var deferred = ZHPromise.defer();
        syncRestriction(widget).then(function() {
            var arrayParamsProcedure = [];
            var filterRowPatient = widget.curPatient.ControlFeedings;
            if (filter) {
                filterRowPatient = widget.curPatient.ControlFeedings.filter(function(controlFeeding) {
                    return controlFeeding.id === filter.id && controlFeeding.serviceDate === filter.serviceDate && controlFeeding.DTINCLUSAO === filter.DTINCLUSAO;
                });
            }
            filterRowPatient.forEach(function(currentRow) {
                if (currentRow.idtpservhosp === '0' && currentRow.idformdiet !== '4') {
                    var paramsProcedure = {
                        'P_CDFILIAL': currentRow.unit,
                        'P_CDSERVFILI': currentRow.cdservfili,
                        'P_CDPACIENTE': patient.id,
                        'P_DTSERVINTE': currentRow.serviceDate,
                        'P_NRSEQUSERV': currentRow.sequential,
                        'P_DTINTERNA': moment(patient.hospitalization, 'DD/MM/YYYY HH:mm:ss').format('DD/MM/YYYY')
                    };
                    arrayParamsProcedure.push(paramsProcedure);
                }
            });
            if (isValidArray(arrayParamsProcedure)) {
                restEngine.post('/Patients/updateFoodRestriction', {
                    "requestType": "DataSet",
                    "dataset": arrayParamsProcedure
                }, null, null, null, null, 500000).then(function() {
                    _get(widget).then(function() {
                        deferred.resolve(true);
                    }, function() {
                        deferred.reject(false);
                    });
                }, function() {
                    deferred.reject(false);
                });
            } else {
                deferred.resolve(true);
            }
        }, function() {
            deferred.reject(false);
        });
        return deferred.promise;
    };

    function verifyOccurredErrorCallbackArray(callback) {
        var errorsHaveOccurred = false;
        if (typeof callback === 'object') {
            var objectCallback = Object.entries(callback);
            if (isValidArray(objectCallback)) {
                objectCallback.forEach(function(dataSourceError) {
                    if (isValidArrayOfError(dataSourceError[1], dataSourceError[0])) {
                        errorsHaveOccurred = true;
                    }
                });
            }
        }
        return errorsHaveOccurred;
    }

    function syncRestriction(widget) {
        var deferred = ZHPromise.defer();
        var relationshipId = getIdentificationOfRelationship();
        return _save('/ControlFeedings').then(function(CONTROL_FEEDING) {
            var arrayErrorControl = getArrayOfError(CONTROL_FEEDING, 'CONTROL_FEEDING', 'DELETE');
            return _save('/Patients').then(function(PATIENT) {
                return _save('/Profiles').then(function(PROFILE) {
                    return _save('/NormalFeedings', arrayErrorControl, relationshipId.CONTROL_FEEDING).then(function(NORMALFEEDING) {
                        return _save('/FoodIntoleranceGroup').then(function(FOODINTOLERANCEGROUP) {
                            var callback = {
                                'CONTROL_FEEDING': CONTROL_FEEDING,
                                'PATIENT': PATIENT,
                                'PROFILE': PROFILE,
                                'NORMALFEEDING': NORMALFEEDING,
                                'FOODINTOLERANCEGROUP': FOODINTOLERANCEGROUP
                            };
                            var error = verifyOccurredErrorCallbackArray(callback);
                            if (!error) {
                                _get(widget).then(function() {
                                    deferred.resolve();
                                });
                            } else {
                                deferred.reject();
                            }
                            return deferred;
                        });
                    });
                });
            });
        });
    }

    function getMessageInfo(typeStruct, arrayInfo, propertyToDisplay) {
        var msg = "<p class='titleStructError'>" + i18n(typeStruct) + ':</p><br />';
        var subcategory = {};
        arrayInfo.forEach(function(currentInfo, indexInfo) {
            var message = '';
            if (currentInfo.SUBCATEGORY) {
                if (!subcategory[currentInfo.SUBCATEGORY]) {
                    subcategory[currentInfo.SUBCATEGORY] = true;
                    message = message.concat("<p class='subCategory'>" + i18n(currentInfo.SUBCATEGORYNAME) + "</p><br />");
                }
            }
            if (currentInfo.id) {
                message = message.concat(i18n('Código Paciente: ') + currentInfo.id + "<br />");
            }
            if (currentInfo.GENERIC_MESSAGE) {
                message = message.concat(i18n(currentInfo.GENERIC_MESSAGE));
            } else {
                propertyToDisplay.forEach(function(property, index) {
                    message = message.concat(i18n(currentInfo[property]));
                    if (index !== propertyToDisplay.length - 1) {
                        message = message.concat(' - ');
                    }
                });
            }
            if (indexInfo < arrayInfo.length - 1) {
                message = message.concat('<br />');
            }
            if (msg.indexOf(message) === -1) {
                msg = msg.concat(message);
            }
        });
        return msg;
    }

    function isValidArrayOfError(callback, nameArray) {
        return Array.isArray(callback) && callback.length && callback[0].dataset && callback[0].dataset.error && Array.isArray(callback[0].dataset.error[nameArray]) && callback[0].dataset.error[nameArray].length;
    }

    function i18n(string) {
        if (typeof string === 'string') {
            return ScreenService.i18n(string);
        }
        return "";
    }

    function concatCallbackError(callback, nameArray) {
        var fullCallback;
        if (isValidArray(callback)) {
            callback.forEach(function(currentCallback) {
                if (isValidArrayOfError([currentCallback], nameArray)) {
                    if (!fullCallback) {
                        fullCallback = [angular.copy(currentCallback)];
                    } else {
                        fullCallback[0].dataset.error[nameArray] = fullCallback[0].dataset.error[nameArray].concat(currentCallback.dataset.error[nameArray]);
                    }
                }
            });
            return fullCallback;
        }
        return [];
    }

    function isValidObjectError(object) {
        return object && object.dataset && object.dataset.error;
    }

    function getArrayOfError(callback, nameArray, typeOperation) {
        if (isValidArray(callback)) {
            if (typeOperation && typeof typeOperation === 'string') {
                var filtredCallback = callback.filter(function(data) {
                    if (isValidArray(data) || isValidObjectError(data)) {
                        return data.dataset.error.OPERATION === typeOperation;
                    } else {
                        return false;
                    }
                })[0];
                if (filtredCallback) {
                    return filtredCallback.dataset.error[nameArray];
                }
            } else if (isValidArrayOfError(callback, nameArray)) {
                return callback[0].dataset.error[nameArray];
            }
        }
        return [];
    }

    function errorHandling(typeStruct, callback, nameArray, propertyToDisplay, referenceMainMessage) {
        var normalizedCallBack = concatCallbackError(callback, nameArray);
        if (isValidArrayOfError(normalizedCallBack, nameArray)) {
            var struct = ScreenService.i18n(typeStruct);
            var arrayOfError = getArrayOfError(normalizedCallBack, nameArray);
            var message = getMessageInfo(struct, arrayOfError, propertyToDisplay);
            if (referenceMainMessage.numberOfErrors === 0) {
                referenceMainMessage.msg = referenceMainMessage.msg.concat('<br /><br />');
            }
            if (referenceMainMessage.numberOfErrors > 0) {
                referenceMainMessage.msg = referenceMainMessage.msg.concat("<hr class='lineWrap'>");
            }
            referenceMainMessage.msg = referenceMainMessage.msg.concat(message);
            referenceMainMessage.msg = referenceMainMessage.msg.concat('<br />');
            referenceMainMessage.numberOfErrors++;
            return true;
        }
        return false;
    }

    function getStructureDataError() {
        var dataSets = {
            '/ControlFeedings': {
                'name': 'CONTROL_FEEDING',
                'typeData': 'Prescrições do Controle Alimentar',
                'propertyToDisplay': ['serviceDate', 'serviceName', 'dietName']
            },
            '/User': {
                'name': 'USER',
                'typeData': 'Usuários',
                'propertyToDisplay': []
            },
            '/Patients': {
                'name': 'PATIENT',
                'typeData': 'Pacientes',
                'propertyToDisplay': []
            },
            '/Profiles': {
                'name': 'PROFILE',
                'typeData': 'Perfil Alimentar',
                'propertyToDisplay': ['nmgroupsub'],
            },
            '/Prescriptions': {
                'name': 'PRESCRIPTION',
                'typeData': 'Prescrições Dietéticas',
                'propertyToDisplay': ['date', 'dietName', 'bed']
            },
            '/Appointments': {
                'name': 'APPOINTMENT',
                'typeData': 'Consultas',
                'propertyToDisplay': []
            },
            '/BioEvaluations': {
                'name': 'BIOEVALUATION',
                'typeData': 'Avaliações Bioquímicas',
                'propertyToDisplay': []
            },
            '/NormalFeedings': {
                'name': 'NORMALFEEDING',
                'typeData': 'Pratos Prescrição Oral',
                'propertyToDisplay': ['serviceDate', 'plateTypeName', 'plateName']
            },
            '/EnteralFeedings': {
                'name': 'ENTERALFEEDING',
                'typeData': 'Pratos Prescrição Enteral',
                'propertyToDisplay': []
            },
            '/LactaryFeedings': {
                'name': 'LACTARYFEEDING',
                'typeData': 'Pratos Prescrição Lactária',
                'propertyToDisplay': ['serviceDate', 'formulationName']
            },
            '/FoodIntoleranceGroup': {
                'name': 'FOODINTOLERANCEGROUP',
                'typeData': 'Grupos de Restrição Alimentar',
                'propertyToDisplay': []
            }
        };
        return dataSets;
    }

    this.persistOnline = function(dataSourceName, route, type, param, timeout) {
        var isOnline = localStorageEngine.getLocalVar('__ONLINE');
        var defer = ZHPromise.defer();
        if (isOnline === true) {
            if (typeof route === 'string' && typeof type === 'string') {
                var typeLow = type.toLowerCase();
                var objectRequest = {
                    requestType: type
                };
                objectRequest[typeLow] = param;
                restEngine.post(route, objectRequest, null, null, null, null, timeout).then(function(callback) {
                    if (isValidArrayOfError(callback, dataSourceName)) {
                        var structure = getStructureDataError();
                        var currentStructure = structure[dataSourceName];
                        var msgError = {
                            'msg': ScreenService.i18n("Ocorreram erros ao salvar alterações:"),
                            'numberOfErrors': 0
                        };
                        errorHandling(currentStructure.typeData, callback, currentStructure.name, currentStructure.propertyToDisplay, msgError);
                        ScreenService.showMessage(msgError.msg);
                        defer.reject(false);
                    }
                    defer.resolve(callback);
                    return defer;

                }, function() {
                    ScreenService.showMessage(i18n('Falha na conexão. Sincronize para salvar dados locais.'));
                    defer.reject(null);
                });

            } else {
                defer.reject(null);
            }
        } else {
            defer.reject(null);
        }
        return defer;
    };

    function crudPromisse(dataSources, typeOperation) {
        var promise = [];
        if (typeOperation === 'clear', isValidArray(dataSources)) {
            dataSources.forEach(function(dataSource) {
                promise.push(
                    localStorageEngine.setDataSource(dataSource, [])
                );
            });
        }
        return ZHPromise.all(promise);
    }

    function getIdentificationOfRelationship() {
        var parentsAndId = {
            'CONTROL_FEEDING': {
                'IDENTIFICATION': 'PARENT.nrsequserv === CHILDREN.sequential && PARENT.id === CHILDREN.id'
            }
        };
        return parentsAndId;
    }
    this.sync = function(changedUnit) {
        var msgError = {
            'msg': ScreenService.i18n("Ocorreram erros ao sincronizar os seguintes dados:"),
            'numberOfErrors': 0
        };
        var relationshipId = getIdentificationOfRelationship();
        var errorsHaveOccurred = false;
        return _save('/User').then(function(USER) {
            return _save('/Patients').then(function(PATIENT) {
                return _save('/Profiles').then(function(PROFILE) {
                    return _save('/ControlFeedings').then(function(CONTROL_FEEDING) {
                        var arrayErrorControl = getArrayOfError(CONTROL_FEEDING, 'CONTROL_FEEDING', 'DELETE');
                        return _save('/Prescriptions').then(function(PRESCRIPTION) {
                            return _save('/Appointments').then(function(APPOINTMENT) {
                                return _save('/BioEvaluations').then(function(BIOEVALUATION) {
                                    return _save('/NormalFeedings', arrayErrorControl, relationshipId.CONTROL_FEEDING).then(function(NORMALFEEDING) {
                                        return _save('/EnteralFeedings', arrayErrorControl, relationshipId.CONTROL_FEEDING).then(function(ENTERALFEEDING) {
                                            return _save('/LactaryFeedings', arrayErrorControl, relationshipId.CONTROL_FEEDING).then(function(LACTARYFEEDING) {
                                                return _save('/FoodIntoleranceGroup').then(function(FOODINTOLERANCEGROUP) {
                                                    var deferred = ZHPromise.defer();
                                                    var dataSets = Object.values(getStructureDataError());
                                                    var callback = {
                                                        'CONTROL_FEEDING': CONTROL_FEEDING,
                                                        'USER': USER,
                                                        'PATIENT': PATIENT,
                                                        'PROFILE': PROFILE,
                                                        'PRESCRIPTION': PRESCRIPTION,
                                                        'APPOINTMENT': APPOINTMENT,
                                                        'BIOEVALUATION': BIOEVALUATION,
                                                        'NORMALFEEDING': NORMALFEEDING,
                                                        'ENTERALFEEDING': ENTERALFEEDING,
                                                        'LACTARYFEEDING': LACTARYFEEDING,
                                                        'FOODINTOLERANCEGROUP': FOODINTOLERANCEGROUP
                                                    };
                                                    dataSets.forEach(function(dataSet) {
                                                        var currentErro = errorHandling(dataSet.typeData, callback[dataSet.name], dataSet.name, dataSet.propertyToDisplay, msgError);
                                                        errorsHaveOccurred = errorsHaveOccurred ? true : currentErro;
                                                    });
                                                    if (changedUnit) {
                                                        var dataSource = ['/AccessRoads', '/Beds', '/DataEvaluations', '/ControlFeedings', '/DataTypeEvaluations', '/Examinations', '/Formulations', '/HealthInsurance', '/NutritionalStatus', '/Patients', '/Prescriptions', '/Profiles', '/Services', '/Appointments'];
                                                        crudPromisse(dataSource, 'clear').then(function() {
                                                            _get().then(function() {
                                                                if (errorsHaveOccurred) {
                                                                    ScreenService.showMessage(msgError.msg);
                                                                    deferred.resolve(true);
                                                                } else {
                                                                    deferred.resolve(false);
                                                                }
                                                            });
                                                        });
                                                    } else {
                                                        _get().then(function() {
                                                            if (errorsHaveOccurred) {
                                                                ScreenService.showMessage(msgError.msg);
                                                                deferred.resolve(true);
                                                            } else {
                                                                deferred.resolve(false);
                                                            }
                                                        });
                                                    }
                                                    return deferred;
                                                });

                                            });
                                        });
                                    });
                                });
                            });
                        });
                    });
                });
            });
        });
    };
};
Configuration(function(ContextRegister) {
    ContextRegister.register('SynchronizeService', SynchronizeService);
});


// FILE: js/services/UserService.js
var UserService = function(localStorageEngine, UserRepository) {

    this.save = function(user) {

        return UserRepository.save(user).then(function() {

            localStorageEngine.setLocalVar('__CDFILIAL', user.CDFILIAL);
            localStorageEngine.setLocalVar('__NRORGTRAB', user.NRORGTRAB);

            // temporary
            localStorageEngine.setLocalVar('__storey', "");
            localStorageEngine.setLocalVar('__hall', "");
            localStorageEngine.setLocalVar('__bed', "");

        });

    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('UserService', UserService);
});


// FILE: js/controllers/AppointmentController.js
var AppointmentController = function(eventAggregator, localStorageEngine, templateManager, ScreenService, Environment, AppointmentService, EvaluationDataService, Validation, ZHPromise) {

    var containerMain;
    var widgetNavigationStripe;
    var widgetMain;
    var prevValue;

    var MALE = 'M';
    var FEMALE = 'F';

    // LIST

    this.widgetAfterMoveRow = function(dataSource, appointment) {

        Environment.setCurrentAppointment(appointment);
        Environment.setAppointmentDataSource(dataSource);
        Environment.setCurrentAppointmentDate(appointment.date);

        ScreenService.openWindow('Appointment');

    };

    // VIEW

    this.navigationStripeAfterMoveRow = function(appointment) {

        Environment.setCurrentAppointment(appointment);
        Environment.setCurrentAppointmentDate(appointment.date);

        widgetMain.reload().then(function() {
            widgetMain.moveToFirst();
        });

    };

    var _initNavigationStripe = function() {
        widgetNavigationStripe.currentRow = Environment.getCurrentAppointment();
        widgetNavigationStripe.dataSource.data = Environment.getAppointmentDataSource();
    };

    var _initContainer = function(container) {

        containerMain = container;
        widgetNavigationStripe = container.getWidget('WidgetNavigationStripe');
        widgetMain = container.getWidget('WidgetMain');

    };

    this.containerAfterinit = function(container) {

        _initContainer(container);
        _initNavigationStripe();

    };

    // ACTIONS
    this.addEvaluation = function(appointment, evaluation) {

        Environment.setCurrentAppointment(appointment);
        ScreenService.openWindow(evaluation);

    };

    // ANTHROPOMETRIC EVALUATION

    this.openWindow = function(methods) {

        EvaluationDataService.setMethods(methods);
        ScreenService.openWindow('EvaluationData');

    };

    this.closeSwipe = function(widget) {

        ScreenService.goBack();
        widget.activate();

    };

    // EDIT
    this.openSwipe = function(container) {

        var widgetMain = container.getWidget('WidgetAppointment');
        var widgetSwipe = container.getWidget('WidgetAppointmentSwipe');

        widgetSwipe.widgets[0].template = "widget/form-view.html";
        widgetSwipe.widgets[0].activate();
        widgetSwipe.widgets[0].view();

        widgetSwipe.widgets.forEach(function(widget) {
            widget.currentRow = widgetMain.currentRow;
        });

        Environment.setCurrentAppointmentDate(widgetMain.currentRow.date);

        ScreenService.openSwipe(widgetSwipe);

    };

    this.view = function(widget) {

        widget.template = "widget/form-view.html";

    };

    this.edit = function(widget) {

        widget.template = "widget/form.html";
        widget.edit();

    };

    this.add = function(widget) {

        widget.widgets[0].template = "widget/form.html";
        widget.widgets[0].edit();
        widget.widgets[0].newRow();

        widget.widgets[0].getField('date').value(Validation.now());

        ScreenService.openSwipe(widget);

    };

    this.editBio = function(widget) {

        var filterPatient = [{
                "name": "id",
                "operator": "=",
                "value": localStorageEngine.getLocalVar('__patient')
            },
            {
                "name": "unit",
                "operator": "=",
                "value": localStorageEngine.getLocalVar('__CDFILIAL')
            }
        ];

        localStorageEngine.getDataSource('/Patients', filterPatient).then(function() {
            widget.edit();
            widget.getField('examinationDate').readOnly = true;

        });

    };

    eventAggregator.subscribe('addBioAva', function(dataForNewAva) {

        var widget = dataForNewAva[0];
        var avaliationFields = widget.AvaliationFields;
        var dataSource = widget.localDataSourceLoaded;
        var patient = widget.curPatient;
        var currentAppointment = dataForNewAva[1];
        var now = new Date();
        var key = now.getTime();

        var newAvaliation = {
            'key': key,
            'age': patient.age.match(/[0-9]+/g),
            'gender': patient.gender,
            'appointmentDate': currentAppointment.date,
            'id': patient.id,
            'unit': patient.unit,
            '__is_new': true
        };

        if (patient.gender === '0') {
            newAvaliation.pregnancyId = 'N';
            avaliationFields.forEach(function(avfd) {
                if (avfd.name === "pregnancyWeek" || avfd.name === "pregnancyName") {
                    avfd.isVisible = false;
                }
            });
        } else {
            avaliationFields.forEach(function(avfd) {
                if (avfd.name === "pregnancyWeek" || avfd.name === "pregnancyName") {
                    avfd.isVisible = true;
                }
            });
        }

        var priority = dataSource.BioEvaluations.length + 1;
        newAvaliation.priority = Validation.strPad(priority, 3);
        // widget.widgets[0].getField('examinationName').readOnly = false;
        // widget.widgets[0].getField('examinationDate').readOnly = false;
        // console.log(newAvaliation);
        widget.edit();
        widget.avaliationExam = newAvaliation;
        widget.showAddEditExam = true;
    });

    this.cancelBio = function(widget) {
        var WidgetBiochemicalEvaluationGrid = widget.container.getWidget('WidgetBiochemicalEvaluationGrid');
        ScreenService.closeSwipe();
        WidgetBiochemicalEvaluationGrid.activate();
    };

    this.backBio = function(widget) {

        ScreenService.closeSwipe();
        widget.activate();

    };

    this.save = function(widget) {
        widget.currentRow.changed = true;
        if (widget.currentRow.dietName === "") {
            widget.currentRow.dietId = "";
            widget.currentRow.dietName = "sem descrição";
        }
        if (widget.currentRow.healthInsuranceName === "") {
            widget.currentRow.healthInsuranceId = "";
        }
        if (widget.currentRow.medicalTeamName === "") {
            widget.currentRow.medicalTeamId = "";
        }
        if (widget.currentRow.nutritionalStatusName === "") {
            widget.currentRow.nutritionalStatusId = "";
        }
        AppointmentService.save(widget.currentRow).then(function() {

            // ScreenService.showMessage('Salvo com sucesso.');
            if (widget.name === 'WidgetBiochemicalEvaluationGrid') {
                ScreenService.closeSwipe();
            } else {
                widget.template = "widget/form-view.html";
                ScreenService.closeSwipe();
                widget.container.getWidget('WidgetAppointment').reload();
                widget.container.getWidget('WidgetAppointment').activate();

            }
            Environment.lastPatientVisit();
        });

    };

    this.bioEvaluationSave = function(widget) {
        var WidgetBiochemicalEvaluationGrid = widget.container.getWidget('WidgetBiochemicalEvaluationGrid');
        var row = widget.currentRow;
        widget.currentRow.changed = true;
        if ((row.examinationDate) && (row.examinationName) && (row.examinationValue)) {
            AppointmentService.bioEvaluationSave(widget.currentRow).then(function() {

                // ScreenService.showMessage('Salvo com sucesso.');
                ScreenService.closeSwipe();
                WidgetBiochemicalEvaluationGrid.reload();
                WidgetBiochemicalEvaluationGrid.activate();
                Environment.lastPatientVisit();
            });
        } else {
            ScreenService.showMessage('Preencha os campos obrigatórios');
        }

    };

    function removeDuplicates(originalArray, objKey) {
        var trimmedArray = [];
        var values = [];
        var value;

        for (var i = 0; i < originalArray.length; i++) {
            value = originalArray[i][objKey];

            if (values.indexOf(value) === -1) {
                trimmedArray.push(originalArray[i]);
                values.push(value);
            }
        }

        return trimmedArray;

    }

    this.openSwipeAnthropometric = function(widget) {
        var widgetList = widget.container.getWidget('WidgetAnthropometricEvaluationList');
        var widgetTabbed = widget.container.getWidget('WidgetAnthropometricEvaluation');
        var filterPatient = [];
        if (widget.currentRow.__is_new) {
            var age = parseInt(widget.container.getWidget('WidgetNavigationStripe').currentRow.age.substring(0, 2));

            filterPatient = [{
                    name: "dataIni",
                    operator: "<",
                    value: age
                },
                {
                    name: "dataFim",
                    operator: ">",
                    value: age
                }
            ];

            localStorageEngine.getDataSource('/AnthEvaluationGeneral', filterPatient).then(function(methods) {

                methods = removeDuplicates(methods, 'name');

                widgetList.dataSource.data = methods;
                ScreenService.closePopup(true);
                ScreenService.openSwipe(widgetTabbed);
            });

        } else {
            filterPatient = [{
                    "name": "id",
                    "operator": "=",
                    "value": getLocalVar('__patient')
                },
                {
                    "name": "date",
                    "operator": "=",
                    "value": getLocalVar('__appointment')
                }
            ];

            localStorageEngine.getDataSource('/AnthEvaluationMethods', filterPatient).then(function(methods) {

                methods = removeDuplicates(methods, 'name');

                widgetList.dataSource.data = methods;
                ScreenService.closePopup(true);
                ScreenService.openSwipe(widgetTabbed);
            });

        }

    };

    this.openSwipeBiochemical = function(widget) {

        ScreenService.closePopup(true);
        ScreenService.openSwipe(widget.widgets[0]);

    };

    this.openSubWidgetSwipe = function(widget) {

        widget.widgets[0].widgets[0].activate();
        widget.widgets[0].widgets[0].currentRow = widget.currentRow;
        ScreenService.openSwipe(widget.widgets[0]);

    };

    this.classification = function(widget) {

        window.IDADE = widget.currentRow.age;
        window.RESULTADO = widget.currentRow.examinationValue;
        var formula = widget.currentRow.examinationFormula;

        if (widget.currentRow.gender === 0) {
            window.SEXO = MALE;
        } else {
            window.SEXO = FEMALE;
        }

        var status = Validation.formulaToJS(formula);

        widget.getField('examinationStatus').value(status);

    };

    this.anthDataEvaluation = function(widget) {

        widget.widgets[0].widgets[0].fields = [];
        widget.widgets[0].widgets[0].fieldGroups = "";
        AppointmentService.anthDataEvaluation(widget).then(function() {
            ScreenService.openSwipe(widget.widgets[0]);
        });

    };

    this.anthCalculate = function(widget) {
        AppointmentService.anthCalculate(widget).then(function() {
            ScreenService.openSwipe(widget.widgets[0]);
        });
    };

    this.widgetAfterSelectRow = function(checkedRows) {
        AppointmentService.saveCheckedRows(checkedRows);
    };

    this.backResult = function(widget) {

        ScreenService.closeSwipe();
        widget.container.getWidget('WidgetAnthropometricEvaluationSwipeForm').activate();

    };

    this.backEvaluation = function(widget) {

        ScreenService.closeSwipe();
        widget.container.getWidget('WidgetAnthropometricEvaluationList').activate();

    };

    this.backMethods = function(widget) {
        ScreenService.closeSwipe();
        widget.activate();
    };

    this.backOrCancel = function(args) {
        var deferred = ZHPromise.defer();
        if (args.row.__is_new) {
            args.owner.widget.list(true);
            args.owner.widget.dataSource.data.pop();
            args.owner.widget.currentRow = {};
            deferred.resolve();
        } else if (args.owner.widget.mode === 'edit') {
            args.owner.widget.view();
            for (var i in args.owner.widget.backupRow) {
                args.owner.widget.currentRow[i] = args.owner.widget.backupRow[i];
            }
            deferred.resolve();
            return deferred.promise;
        }
        ScreenService.goBack();
        return deferred.promise;
    };

    this.onlyNumbers = function(field) {
        if (isNaN(Number(field.value()))) {
            field.value(prevValue);
        } else {
            if (field.value() === "") {
                prevValue = "";
                field.value(0);
                field.value("");
                return false;
            }
            if (parseInt(Number(field.value())) === Number(field.value())) {
                field.value(parseInt(Number(field.value())).toString());
                field.value(Math.abs(field.value()));
                prevValue = field.value();
            } else {
                field.value(prevValue);
            }
        }
    };

    this.setPrevValue = function(field) {
        prevValue = field.value();
    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('AppointmentController', AppointmentController);
});


// FILE: js/controllers/ControlFeedingController.js
var ControlFeedingController = function(localStorageEngine, Validation, ScreenService, Environment, ControlFeedingService) {

    var SERVICES = {
        NORMAL: "0",
        LACTARY: "1",
        ENTERAL: "2",
        ROOM_SERVICE: "3",
        ESPECIAL: "4"
    };
    var isNew = false;

    this.verify = function(widget) {
        if (widget.selectedRow.__alwaysSelected) {
            ScreenService.showMessage(ScreenService.i18n('Não é possível desmarcar o prato não opcional'));
            widget.selectedRow.__isSelected = true;
        }
    };

    this.verifyAll = function(widget) {
        widget.dataSource.data.forEach(function(row) {
            if (row.__alwaysSelected) {
                row.__isSelected = true;
            }
        });
    };

    this.back = function(widget) {
        ScreenService.closeSwipe();
        if (widget.name === 'WidgetLactaryFeedingFormSwipe') {
            widget.container.getWidget("WidgetLactaryFeedingSwipe").activate();

        } else if (widget.name === 'WidgetNormalFeedingSwipe') {
            widget.container.getWidget("WidgetNormalFeedingSwipe").activate();
        } else if (widget.name === 'WidgetNormalFeedingFormSwipe') {
            widget.container.getWidget("WidgetNormalFeedingSwipe").activate();
        } else {
            widget.container.getWidget("WidgetEnteralFeedingSwipe").activate();
        }
    };

    this.view = function(widget) {
        widget.template = "widget/form-view.html";
        widget.list();
    };

    this.backEvaluation = function(widget) {
        widget.unSelectAll();
        ScreenService.closeSwipe();
        widget.container.getWidget('WidgetControlFeeding').activate();
        // widget.container.getWidget('WidgetAddEvaluationFeeding').widgets[0].activate();
    };

    this.cancel = function(widget) {
        if (widget.name === 'WidgetEnteralFeedingFormSwipe') {
            ScreenService.closeSwipe();
            widget.container.getWidget('WidgetEnteralFeedingSwipe').activate();
        } else {
            widget.template = "widget/form-view.html";
            widget.list();
            ScreenService.closeSwipe();
        }
    };

    this.addEvaluation = function(widget, evaluation) {

        widget.widgets[0].newRow();
        var evaluationService = '';

        if (evaluation === '2') {
            widget.widgets[0].label = ScreenService.i18n('Adicionar Enteral/Parenteral');
            evaluationService = '1';
            widget.widgets[0].getField('date').readOnly = true;
            widget.widgets[0].getAction('next').isVisible = false;
            widget.widgets[0].getAction('confirm').isVisible = true;
        }

        if (evaluation === '1') {
            widget.widgets[0].label = ScreenService.i18n('Adicionar Lactário');
            evaluationService = '2';
            widget.widgets[0].getField('date').readOnly = true;
            widget.widgets[0].getAction('next').isVisible = false;
            widget.widgets[0].getAction('confirm').isVisible = true;
        }

        if (evaluation === '0') {
            widget.widgets[0].label = ScreenService.i18n('Adicionar Oral');
            evaluationService = '0';
            widget.widgets[0].getField('date').readOnly = false;
            widget.widgets[0].getAction('next').isVisible = true;
            widget.widgets[0].getAction('confirm').isVisible = false;
            isNew = true;
        }

        widget.widgets[0].getField('serviceName').dataSourceFilter = [{
                "name": "serviceType",
                "operator": "=",
                "value": "D"
            },
            {
                "name": "service",
                "operator": "=",
                "value": evaluationService
            },
            {
                "name": "unit",
                "operator": "=",
                "value": Environment.getLocalVar('__CDFILIAL')
            }
        ];

        widget.widgets[0].getField('serviceName').reload();
        widget.widgets[0].getField('date').value(moment().format("DD/MM/YYYY HH:MM:ss"));
        widget.widgets[0].getField('userName').value(Environment.getLocalVar('__NMOPERADOR'));

        ScreenService.closePopup();
        ScreenService.openSwipe(widget);

    };

    this.nextEvaluation = function(widget) {

        try {

            _validateFields(widget.currentRow);

            var widgetNormalFeeding = widget.container.getWidget('WidgetNormalFeeding');
            var widgetNormalFeedingSwipe = widget.container.getWidget('WidgetNormalFeedingSwipe');
            var menuDate = widget.currentRow.date.split(" ");

            var filter = [{
                    "name": "menuDate",
                    "operator": "=",
                    "value": menuDate[0] + ' 00:00:00'
                },
                {
                    "name": "unit",
                    "operator": "=",
                    "value": localStorageEngine.getLocalVar('__CDFILIAL')
                }
            ];

            widgetNormalFeedingSwipe.dataSource.data = [];
            localStorageEngine.getDataSource('/NormalFeedingsNow', filter).then(function(feedings) {

                if (feedings.length > 0) {

                    feedings.forEach(function(feeding) {
                        if (feeding.optional === 'N') {
                            feeding.__isSelected = true;
                        }
                        widgetNormalFeedingSwipe.dataSource.data.push(feeding);
                        widgetNormalFeedingSwipe.getAction('confirm').isVisible = true;
                    });

                    widgetNormalFeedingSwipe.reload().then(function() {
                        ScreenService.openSwipe(widgetNormalFeeding);
                    });

                } else {

                    var msg = ScreenService.i18n('Não existe cardápio cadastrado para a dieta e dia informado.');
                    ScreenService.showMessage(msg);
                }
            });
        } catch (exceptionMessage) {
            ScreenService.showMessage(ScreenService.i18n(exceptionMessage));
        }

    };

    this.serviceOnChange = function(widget) {
        var service = "";

        // temporary
        if (widget.currentRow.service === '1') {
            service = SERVICES.ENTERAL;
        }

        if (widget.currentRow.service === '2') {
            service = SERVICES.LACTARY;
        }

        if (widget.currentRow.service === '0') {
            service = SERVICES.NORMAL;
        }

        widget.getField('dietName').dataSourceFilter = [{
                "name": "serviceId",
                "operator": "=",
                "value": widget.currentRow.serviceId
            },
            {
                "name": "unit",
                "operator": "=",
                "value": localStorageEngine.getLocalVar('__CDFILIAL')
            },
            {
                "name": "service",
                "operator": "=",
                "value": service
            }
        ];

        widget.getField('dietName').reload().then(function() {
            widget.getField('dietName').value('');
        });

    };

    var _validateFields = function(fields) {

        if (!fields.date) {
            throw "Informe uma data.";
        }

        if (!fields.serviceName) {
            throw "Informe um serviço.";
        }

        if (!fields.dietName) {
            throw "Informe uma dieta.";
        }

    };

    this.saveNormalControlFeeding = function(widget) {
        ControlFeedingService.saveControlFeeding(widget, isNew).then(function(success) {
            if (success) {
                widget.container.getWidget('WidgetControlFeeding').reload().then(function() {
                    ScreenService.closeSwipe(0);
                    widget.container.getWidget('WidgetControlFeeding').activate();
                });
            }
            isNew = false;
        });
    };

    this.saveControlFeeding = function(widget) {

        try {

            _validateFields(widget.currentRow);

            ControlFeedingService.saveControlFeeding(widget.currentRow).then(function() {

                widget.container.getWidget('WidgetControlFeeding').reload().then(function() {
                    ScreenService.closeSwipe();
                    widget.container.getWidget('WidgetControlFeeding').activate();

                });

            });

        } catch (exceptionMessage) {
            ScreenService.showMessage(ScreenService.i18n(exceptionMessage));
        }

    };

    this.openSwipe = function(container) {

        var widgetMain = container.getWidget('WidgetControlFeeding');

        var widgetEnteralFeeding = container.getWidget('WidgetEnteralFeeding');
        var widgetEnteralFeedingSwipe = container.getWidget('WidgetEnteralFeedingSwipe');

        var widgetLactaryFeeding = container.getWidget('WidgetLactaryFeeding');
        var widgetLactaryFeedingSwipe = container.getWidget('WidgetLactaryFeedingSwipe');

        var widgetNormalFeeding = container.getWidget('WidgetNormalFeeding');
        var widgetNormalFeedingSwipe = container.getWidget('WidgetNormalFeedingSwipe');

        var filter = [{
                "name": "id",
                "operator": "=",
                "value": widgetMain.currentRow.id
            },
            {
                "name": "unit",
                "operator": "=",
                "value": widgetMain.currentRow.unit
            },
            {
                "name": "serviceDate",
                "operator": "=",
                "value": widgetMain.currentRow.serviceDate.substring(0, 10)
            },
            {
                "name": "sequential",
                "operator": "=",
                "value": widgetMain.currentRow.sequential
            }

        ];

        if (widgetMain.currentRow.serviceId === SERVICES.ENTERAL) {

            widgetEnteralFeedingSwipe.dataSourceFilter = filter;
            widgetEnteralFeedingSwipe.currentRow = widgetMain.currentRow;
            widgetEnteralFeedingSwipe.reload().then(function() {
                widgetEnteralFeedingSwipe.activate();
                ScreenService.openSwipe(widgetEnteralFeeding);
            });

        }

        if (widgetMain.currentRow.serviceId === SERVICES.LACTARY) {
            widgetLactaryFeedingSwipe.dataSourceFilter = filter;
            widgetLactaryFeedingSwipe.currentRow = widgetMain.currentRow;
            widgetLactaryFeedingSwipe.reload().then(function() {
                widgetLactaryFeedingSwipe.activate();
                ScreenService.openSwipe(widgetLactaryFeeding);
            });

        }

        if (widgetMain.currentRow.serviceId === SERVICES.NORMAL) {
            filter[2] = {
                "name": "serviceDate",
                "operator": "=",
                "value": widgetMain.currentRow.serviceDate.substring(0, 10)
            };
            filter.pop();
            widgetNormalFeedingSwipe.dataSource.data = [];
            localStorageEngine.getDataSource('/NormalFeedings', filter).then(function(feedings) {

                feedings.forEach(function(feeding) {
                    widgetNormalFeedingSwipe.dataSource.data.push(feeding);
                    if (feeding.isSelected) {
                        feeding.__isSelected = true;
                        feeding.__alwaysSelected = true;
                    } else {
                        feeding.__isSelected = false;
                    }
                });

                widgetNormalFeedingSwipe.reload().then(function() {
                    ScreenService.openSwipe(widgetNormalFeeding);
                });

            });

        }

    };

    this.openSubWidgetSwipe = function(widget) {

        widget.widgets[0].widgets[0].template = "widget/form-view.html";
        widget.widgets[0].widgets[0].currentRow = widget.currentRow;
        ScreenService.openSwipe(widget.widgets[0]);

    };

    this.editEnteral = function(widget) {

        widget.getField('formulationName').dataSourceFilter = [{
                "name": "preparationId",
                "operator": "=",
                "value": widget.currentRow.preparationId
            },
            {
                "name": "unit",
                "operator": "=",
                "value": localStorageEngine.getLocalVar('__CDFILIAL')
            }
        ];

        widget.getField('formulationName').reload();

        widget.getField('enteralDate').readOnly = true;
        widget.getField('time').readOnly = true;

        widget.template = "widget/form-view-edit.html";
        widget.edit();

    };

    this.edit = function(widget) {

        widget.template = "widget/form-view-edit.html";
        widget.edit();

    };

    this.add = function(widget) {

        var newWidget = widget.widgets[0].widgets[0];

        newWidget.fields.forEach(function(field) {
            field.readOnly = false;
        });

        newWidget.template = "widget/form.html";
        newWidget.edit();
        newWidget.newRow();

        newWidget.currentRow.unique = Validation.strPad(widget.dataSource.data.length + 1, 4);
        newWidget.currentRow.key = widget.dataSource.data.length + 1;
        newWidget.currentRow.serviceId = widget.currentRow.serviceId;
        newWidget.currentRow.id = widget.currentRow.id;
        newWidget.currentRow.unit = widget.currentRow.unit;
        newWidget.currentRow.serviceDate = widget.currentRow.serviceDate;
        newWidget.currentRow.sequential = widget.currentRow.sequential;

        ScreenService.openSwipe(widget.widgets[0]);

    };

    this.save = function(widget) {
        widget.currentRow.service = widget.container.getWidget('WidgetAddEvaluationFeedingSwipe').currentRow.service;
        widget.currentRow.changed = true;
        if (widget.name === "WidgetNormalFeedingFormSwipe") {
            ControlFeedingService.save(widget.currentRow).then(function() {
                widget.template = "widget/form-view.html";
                widget.list();
            });
        } else {
            widget.fields.forEach(function(field) {
                if ((field.validations) && (field.value() === "")) {
                    ScreenService.showMessage(ScreenService.i18n("Preencha os campos obrigatórios."));
                } else if ((field.validations) && (field.value() !== "")) {
                    ControlFeedingService.save(widget.currentRow).then(function() {

                        widget.template = "widget/form-view.html";
                        ScreenService.closeSwipe();

                        if (widget.name === 'WidgetLactaryFeedingFormSwipe') {
                            widget.container.getWidget('WidgetLactaryFeedingSwipe').reload();
                            widget.container.getWidget('WidgetLactaryFeedingSwipe').activate();

                        } else if (widget.name === 'WidgetEnteralFeedingFormSwipe') {
                            widget.container.getWidget('WidgetEnteralFeedingSwipe').reload();
                            widget.container.getWidget('WidgetEnteralFeedingSwipe').activate();

                        }
                        widget.list();
                        Environment.lastPatientVisit();

                    });
                }
            });
        }
    };

    this.preparationOnChange = function(preparationId, formulation) {
        formulation.value("");
        formulation.widget.currentRow[formulation.valueField] = "";
        formulation.widget.currentRow[formulation.descriptionField] = "";
        formulation.disabled = false;
    };

    this.onEnter = function(widget) {

        widget.reload().then(function() {
            widget.updateCheckedRows();
        });

    };

    this.widgetAfterSelectRow = function(checkedRows) {
        ControlFeedingService.saveCheckedRows(checkedRows);
    };

    this.widgetEnteralFeedingSwipeOnEnter = function() {

        var data = [{
                "preparationId": "A",
                "preparationName": ScreenService.i18n("Aberto")
            },
            {
                "preparationId": "F",
                "preparationName": ScreenService.i18n("Fechado")
            }
        ];

        localStorageEngine.setDataSource('/Preparations', data);

    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('ControlFeedingController', ControlFeedingController);
});


// FILE: js/controllers/EvaluationDataController.js
var EvaluationDataController = function(EvaluationDataService) {

    this.onEnter = function() {

        EvaluationDataService.getFields();

    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('EvaluationDataController', EvaluationDataController);
});


// FILE: js/controllers/LoginController.js
var LoginController = function(LoginService, ScreenService, templateManager, Environment, ServerService, GeneralMethods, DashboardService, restEngine, localStorageEngine) {

    var editarPerfil = function() {
        // ScreenService.openWindow('AlterarPerfil');
    };

    this.containerAfterinit = function(container) {

        var currentUser = LoginService.getCurrentUser();
        var widgetLogin = container.getWidget('WidgetLogin');

        onEnterLogin(widgetLogin);

        if (container.restoreSession && currentUser) {

            ScreenService.openWindow(container.openWindow);
            ScreenService.buildUserData(getLocalVar('__NMOPERADOR'), [getLocalVar('__CDFILIAL') + ' - ' + getLocalVar('__NMFILIAL')], editarPerfil);

        }

    };

    function onEnterLogin(widgetLogin) {
        var fieldDemonstracao = widgetLogin.getField('demonstracao');
        var fieldEmail = widgetLogin.getField('email');

        templateManager.historyItems = ['Login'];
        widgetLogin.newRow();
        Environment.remove('/connection').then(function() {

            var ip = Environment.getLocalVar('ip');
            var lang = Environment.getLocalVar('language');
            var ultimoOperador = Environment.getLocalVar('UltimoOperador');

            if (ip) {
                fieldDemonstracao.isVisible = false;
                templateManager.updateURL(ip);
            }

            if (lang) {
                LoginService.setLanguage(lang);
            }

            if (ultimoOperador) {
                fieldEmail.setValue(ultimoOperador);
            }

            if (window && window.location) {
                var ipSemPorta = window.location.hostname + window.location.pathname;
                var porta = window.location.port;
                var index = typeof ipSemPorta === 'string' ? ipSemPorta.lastIndexOf('/mobile') : ipSemPorta;
                if (typeof ipSemPorta === 'string' && index > -1) {
                    ipSemPorta = ipSemPorta.substring(0, index) + '/';
                    Environment.setLocalVar('ipSemPorta', ipSemPorta);
                }
                if (typeof porta === 'string') {
                    Environment.setLocalVar('porta', porta);
                }
            }

            templateManager.updateTemplate();

        }).catch(function(error) {
            console.log(error.message);
        });
    }

    this.iniciaDemonstracao = function() {
        var lang = Environment.getLocalVar('language');
        if (Environment.getLocalVar('language')) {

            LoginService.setLanguage(lang);

        }

        //templateManager.updateURL("http://apresentacao.teknisa.com/tecfood/portugues/schoolmeals/backend/service/index.php");
        templateManager.updateURL("https://apresentacao-tecdiet.teknisa.com/");

        LoginService.auth('tek@teknisa.com', '1').then(function() {

            Environment.setLocalVar('UltimoOperador', '');
            Environment.setLocalVar('DEMO', true);

        }, function() {

            ScreenService.showMessage(ScreenService.i18n('Verifique sua conexão'), 'error');

        });

    };

    this.onEnterHome = function(container) {
        templateManager.historyItems = ['Dashboard'];
        DashboardService.buildUnitItem(container);
        DashboardService.buildLogItem(container);
        if (!getLocalVar("__CDFILIAL") && getLocalVar('importacaoRealizada')) {
            ScreenService.openWindow('TrocarUnidade');
        }
    };

    this.serverIsConfigured = function(widgetLogin) {

        if (Environment.getLocalVar('ip') || Environment.getLocalVar('ipSemPorta')) {

            ServerService.verifieVersionStatus().then(function(serverStatus) {

                if (serverStatus === "OK") {

                    this.login(widgetLogin);

                } else {

                    var objMessage = ServerService.getObjectMessage(serverStatus);

                    ScreenService.showMessage(ScreenService.i18n(objMessage.message), objMessage.type);

                }

            }.bind(this));

        } else {

            ScreenService.showMessage(ScreenService.i18n('Configure o ip do servidor'), 'alert');
        }

    };

    this.login = function(widget) {

        if (widget.isValid()) {

            LoginService.auth(widget.currentRow.email, widget.currentRow.password).then(function(session) {
                afterLogin(session).then(function() {
                    Environment.remove('/GruposProduto').then(function() {});
                    restEngine.post('/Patients/patientSequentialAjusta', {
                        "requestType": "Row",
                        "row": {
                            'unit': localStorageEngine.getLocalVar('__CDFILIAL')
                        }
                    }).then(function(callback) {
                        if (callback && callback.dataset && callback.dataset.patientSequentialAjusta) {
                            if (callback.dataset.patientSequentialAjusta.IDCTRLCDPACI) {
                                localStorageEngine.setLocalVar('__ISSEQUENTIAL', callback.dataset.patientSequentialAjusta.IDCTRLCDPACI);
                            }
                            if (callback.dataset.patientSequentialAjusta.IDAJUSTAPACI) {
                                localStorageEngine.setLocalVar('__ISAJUSTA', callback.dataset.patientSequentialAjusta.IDAJUSTAPACI);
                            } else {
                                localStorageEngine.setLocalVar('__ISSEQUENTIAL', null);
                                localStorageEngine.setLocalVar('__ISAJUSTA', null);
                            }
                        }
                    });
                });

            }.bind(this), function(callback) {

                if (callback.data) {

                    ScreenService.showMessage(ScreenService.i18n(callback.data.error), 'error');

                } else {

                    ScreenService.showMessage(ScreenService.i18n("Conexão perdida! Tente novamente."), 'error');
                }

            });

        }

    };

    // função que recebe dados do login após o mesmo ser efetuado com sucesso
    function afterLogin(login) {
        Environment.setLocalVar('__NMOPERADOR', login.User.NMOPERADOR);
        Environment.setLocalVar('__NRORGTRAB', login.User.NRORG);
        Environment.setLocalVar('__CDFILIAL', login.User.CDFILIAL);
        Environment.setLocalVar('__NMFILIAL', login.User.NMFILIAL);

        ScreenService.buildUserData(getLocalVar('__NMOPERADOR'), [getLocalVar('__CDFILIAL') + ' - ' + getLocalVar('__NMFILIAL')], editarPerfil);

        return GeneralMethods.verificaNovoUsuario(login.User.CDOPERADOR, login.User.DSEMAILOPER, editarPerfil).then(function() {
            return login;
        });
    }

    this.logout = function() {

        ScreenService.confirmMessage("Deseja realmente sair do sistema?", 'question', function() {
            var ipData = backUpBackendData();

            LoginService.logout().then(onLogoutSuccess, onLogoutError);

            applayBackupIP(ipData);
        });

    };

    function onLogoutError() {}

    function onLogoutSuccess() {
        LoginService.removeUserData();
        ScreenService.toggleSideMenu();
        ScreenService.openWindow("login");
    }

    function backUpBackendData() {
        var ipdata = {
            'saveConnectedIPSemPorta': getLocalVar("ipSemPorta"),
            'saveConnectedPorta': getLocalVar("porta"),
            'saveConnectedDiretorioIP': getLocalVar("diretorio"),
            'saveConnectedIP': getLocalVar("ip")
        };
        return ipdata;
    }

    function applayBackupIP(ipData) {
        setTimeout(function() {
            setLocalVar("ipSemPorta", ipData.saveConnectedIPSemPorta);
            setLocalVar("porta", ipData.saveConnectedPorta);
            setLocalVar("diretorio", ipData.saveConnectedDiretorioIP);
            setLocalVar("ip", ipData.saveConnectedIP);
        }, 300);
    }

    this.changeLanguage = function(lang) {
        Environment.setLocalVar('language', lang);
        ScreenService.setLanguage(lang);
        LoginService.setLanguage(lang);
        ScreenService.successNotification(ScreenService.i18n('Idioma alterado'));
        ScreenService.closePopup();
    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('LoginController', LoginController);
});


// FILE: js/controllers/MapDietController.js
var MapDietController = function(ApplicationContext, eventAggregator) {

    this.onEnterWidgetMain = function(widget) {

        var filter = [];
        widget.reload(filter);
    };

    this.reloadScreenAndClearFilters = function(widget) {
        eventAggregator.publish("reloadMapDiet", widget);
    };

    eventAggregator.subscribe("reloadMapDiet", function(widget) {
        widget.typeSearch = '';
        widget.dataSourceFilter = [];
        widget.reload();
    });
};

Configuration(function(ContextRegister) {
    ContextRegister.register('MapDietController', MapDietController);
});


// FILE: js/controllers/MapKitchenMaidController.js
var MapKitchenMaidController = function(ApplicationContext, eventAggregator, SynchronizeService, ZHPromise, Environment, localStorageEngine, ScreenService, templateManager, MapNutritionistService) {
    var foodsRepository = ApplicationContext.FoodsRepository;

    this.onEnterWidgetMain = function(widget) {

        var filter = [{
                name: [{
                    name: "patientName",
                    direction: "ASC"
                }],
                operator: "ORDER_BY",
                value: ""
            },
            {
                name: "unit",
                operator: "=",
                value: getLocalVar("__CDFILIAL")
            }
        ];
        widget.reload(filter);
    };

    this.reloadScreenAndClearFilters = function(widget) {
        eventAggregator.publish("reloadMapKitchenMaid", widget);
    };

    eventAggregator.subscribe("reloadMapKitchenMaid", function(widget) {
        widget.typeSearch = '';
        widget.dataSourceFilter = [];
        widget.reload();
    });

    eventAggregator.subscribe('sendDeliverFood', function(serviceData) {
        var widget = serviceData.widget;
        var service = serviceData.service;
        // var index = serviceData.index;
        var promise = ZHPromise.defer();
        service.changed = true;
        service.situation = 'Recolher';
        service.deliveryDate = moment().format('DD/MM/YYYY HH:mm:ss');
        service.deliveryOperator = localStorageEngine.getLocalVar('__CDOPERADOR');
        SynchronizeService.persistOnline('/Foods', '/Foods/delivery', 'DataSet', [service]).then(function() {
            service.changed = false;
            service.__is_new = false;
            foodsRepository.save(service).then(function() {
                widget.reload();
            });
        }, function() {}).finally(function() {
            promise.resolve();
        });
    });

    eventAggregator.subscribe('sendCollectFood', function(serviceData) {
        var widget = serviceData.widget;
        var service = serviceData.service;
        var promise = ZHPromise.defer();
        service.changed = true;
        service.situation = null;
        service.collectDate = moment().format('DD/MM/YYYY HH:mm:ss');
        service.collectOperator = localStorageEngine.getLocalVar('__CDOPERADOR');
        SynchronizeService.persistOnline('/Foods', '/Foods/collect', 'DataSet', [service]).then(function() {
            service.changed = false;
            service.__is_new = false;
            foodsRepository.save(service).then(function() {
                widget.reload();
            });
            foodsRepository.removeFood(service);
        }, function() {}).finally(function() {
            // widget.reload();
            promise.resolve();
        });
    });

    this.closePopup = function(widget) {
        closePopupPrescription(widget);
    };

    function closePopupPrescription(widgetPopup) {
        widgetPopup.currentRow = {};
        widgetPopup.filtredFeedingsForWidget = [];
        ScreenService.closePopup();
        widgetPopup.isVisible = false;
        widgetPopup.container.showMenu = true;
    }

    this.saveNewPrescription = function(widget) {
        var widgetPrescricao = widget.container.getWidget('widgetPopupPrescricao');
        var currentRow = widgetPrescricao.currentRow;

        if (currentRow.isNewRegister) {
            currentRow.DTINCLUSAO = moment().format('DD/MM/YYYY HH:mm:ss');
        }
        var msg;
        var filterPatient = [{
                "name": "unit",
                "operator": "=",
                "value": localStorageEngine.getLocalVar('__CDFILIAL')
            },
            {
                "name": "id",
                "operator": "=",
                "value": widget.currentRow.id,
            },
            {
                "name": [{
                    "name": "formattedName",
                    "direction": "ASC"
                }],
                "operator": "ORDER_BY",
                "value": ""
            }
        ];
        if (widget.isValid()) {
            localStorageEngine.getDataSource('/Patients', filterPatient).then(function(patient) {
                patient[0].ControlFeedings = [widget.currentRow];
                widget.curPatient = patient[0];
                if (widget.filtredFeedingsForWidget && widget.filtredFeedingsForWidget.length) {
                    var count = widget.filtredFeedingsForWidget.filter(function(feeding) {
                        return feeding.selectedByUser === true;
                    }).length;
                    if (count === 0 && widget.currentRow.dietService !== '4' && widget.currentRow.idformdiet !== '4') {
                        msg = ScreenService.i18n("Selecione algum prato para salvar a prescrição!");
                        ScreenService.showMessage(msg);
                    } else {
                        saveOralPrescription(widget);
                    }
                } else if ((currentRow.idtpservhosp === '0' || currentRow.idtpservhosp === '3' ||
                        currentRow.idtpservhosp === '4') || !currentRow.idtpservhosp) {
                    if (widget.currentRow.dietService === '4' || widget.currentRow.idformdiet === '4') {
                        msg = ScreenService.i18n("Selecione algum prato para salvar a prescrição!");
                        ScreenService.showMessage(msg);
                    } else {
                        msg = ScreenService.i18n("Não há cardápio para a configuração informada!");
                        ScreenService.showMessage(msg);
                    }
                }
            });
        }
    };

    function saveOralPrescription(widget) {
        // var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');
        // var nonFormatedDate = widget.currentRow.serviceDate;
        var formatedData = moment(widget.currentRow.serviceDate, 'YYYY-MM-DD').format("DD/MM/YYYY");
        widget.currentRow.serviceDate = formatedData !== "Invalid date" ? formatedData : widget.currentRow.serviceDate;
        templateManager.bkpCurrentRowPrescription = angular.copy(widget.currentRow);
        // var currentRow = widget.currentRow;
        // var normalFeedings = widget.filtredFeedingsForWidget;
        if (widget.currentRow.idformdiet === '4' || widget.currentRow.dietService === '4') {
            saveSpecialDiet(widget);
        } else {
            saveNormalFeedingsEdit(widget);
        }
        closePopupPrescription(widget);
    }

    function saveNormalFeedingsEdit(widget) {
        // var widgetPopupPrescricao = widget.container.getWidget('widgetPopupPrescricao');
        var widgetPopupPrescricao = widget;
        // var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');
        var currentRow = angular.copy(widgetPopupPrescricao.currentRow);
        var normalFeedings = widgetPopupPrescricao.filtredFeedingsForWidget;
        var filter = [{
                "name": "unit",
                "operator": "=",
                "value": localStorageEngine.getLocalVar('__CDFILIAL')
            },
            {
                "name": "id",
                "operator": "=",
                "value": localStorageEngine.getLocalVar('__patient')
            },
            {
                "name": "serviceDate",
                "operator": "=",
                "value": moment(currentRow.serviceDate, 'YYYY-MM-DD').format('DD/MM/YYYY')
            },
            {
                "name": "sequential",
                "operator": "=",
                "value": currentRow.sequential
            },
            {
                "name": "unique",
                "operator": "=",
                "value": ""
            }
        ];
        //var rowsToSave = //createRowsToRelateNormalFeedingsToOralPrescription(currentRow, normalFeedings, widgetDetailsPatient);
        if (normalFeedings && normalFeedings.length) {
            var savedOnline = saveOnlineNormalFeedings(widgetPopupPrescricao, normalFeedings);
            var promisesSaveNormalFeedingsEdit = [];
            savedOnline.then(function(successSaveOnline) {
                if (successSaveOnline) {
                    normalFeedings.forEach(function(normalFeeding) {
                        var filterCopied = angular.copy(filter);
                        filterCopied[filterCopied.length - 1].value = normalFeeding.unique;
                        localStorageEngine.getDataSource("/NormalFeedings", filterCopied).then(function(filteredNormalFeeding) {
                            filteredNormalFeeding[0].changed = !successSaveOnline;
                            filteredNormalFeeding[0].selectedByUser = normalFeeding.selectedByUser;
                            filteredNormalFeeding[0].__is_new = normalFeeding.__is_new;
                            filteredNormalFeeding[0].unit = normalFeeding.unit;
                            filteredNormalFeeding[0].id = normalFeeding.id;
                            filteredNormalFeeding[0].serviceDate = normalFeeding.serviceDate;
                            filteredNormalFeeding[0].observation = normalFeeding.observation;
                            promisesSaveNormalFeedingsEdit.push(localStorageEngine.save("/NormalFeedings", filteredNormalFeeding));
                        });
                    });
                    ZHPromise.all(promisesSaveNormalFeedingsEdit).then(function() {
                        updateControlFeedingRestriction(widget, currentRow);
                    });
                }
            });

        }
    }

    function updateControlFeedingRestriction(widget, prescription) {
        var endUpdate = ZHPromise.defer();
        prescription = {};
        // var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');
        // var serviceDate = moment(prescription.serviceDate, "DD/MM/YYYY");
        var currentRowPrescription = templateManager.bkpCurrentRowPrescription;
        // if (serviceDate.isValid()) {
        //     serviceDate = prescription.serviceDate;
        // } else {
        //     serviceDate = moment(prescription.serviceDate, "YYYY-MM-DD").format("DD/MM/YYYY");
        // }
        var filterControlFeedings = {
            "id": currentRowPrescription.id,
            // "serviceDate": moment(currentRowPrescription.serviceDate, 'YYYY-MM-DD').format('DD/MM/YYYY'),
            "serviceDate": currentRowPrescription.serviceDate,
            "DTINCLUSAO": currentRowPrescription.DTINCLUSAO
        };
        // widgetDetailsPatient.wasChanged = true;
        updateProcedurePrescription(widget, filterControlFeedings).then(function() {
            if (Array.isArray(widget.patientsToWarning)) {
                var cdpaciente = widget.curPatient.id;
                var index = widget.patientsToWarning.indexOf(cdpaciente);
                if (index > -1) {
                    widget.patientsToWarning.splice(index, 1);
                }
            }

            ScreenService.showMessage(ScreenService.i18n("Restrições alimentares verificadas com sucesso para esta prescrição!"), 'success');
            endUpdate.resolve();
        }, function() {
            var cdpaciente = widget.curPatient.id;
            if (Array.isArray(widget.patientsToWarning)) {
                var index = widget.patientsToWarning.indexOf(cdpaciente);
                if (index === -1) {
                    widget.patientsToWarning.push(cdpaciente);
                }
            } else {
                widget.patientsToWarning = [];
                widget.patientsToWarning.push(cdpaciente);
            }

            ScreenService.showMessage(ScreenService.i18n("Falha na conexão! Atualize as prescrições no Controle Alimentar para verificar inconsistências com as restrições desse paciente!"));
            endUpdate.resolve();
        });
        return endUpdate;
    }

    function updateProcedurePrescription(widget, filter) {
        var deferred = ZHPromise.defer();
        SynchronizeService.synchronizedIntolerance(widget, filter).then(function(callback) {
            deferred.resolve(callback);
        }, function(callback) {
            deferred.reject(callback);
        });
        return deferred;
    }

    function saveSpecialDiet(widgetPopupPrescricao) {
        // var widgetDetailsPatient = widgetPopupPrescricao.container.getWidget('WidgetDetailsPatient');
        // var currentRow = angular.copy(widgetPopupPrescricao.currentRow);
        // var rows = rowsToInsertUpdateDeleteNormalFeedings(widgetPopupPrescricao);
        var rowsToUpdate = [];
        widgetPopupPrescricao.filtredFeedingsForWidget.forEach(function(feeding) {
            widgetPopupPrescricao.bkpFeedings.forEach(function(bkp) {
                if ((bkp.formulationId === feeding.formulationId && bkp.plateType === feeding.plateType) && !bkp.remove && feeding.key) {
                    rowsToUpdate.push(feeding);
                }
            });
        });
        var normalFeedings = rowsToUpdate;
        var savedOnline;
        var rowsToSave;
        if (Array.isArray(rowsToUpdate) && rowsToUpdate.length) {
            savedOnline = saveOnlineNormalFeedings(widgetPopupPrescricao, rowsToUpdate, rowsToSave);
        }
        savedOnline.then(function(successSaveOnline) {
            normalFeedings.forEach(function(normalFeeding, index) {
                var filter = [{
                    name: 'key',
                    operator: '=',
                    value: ''
                }];
                filter[0].value = normalFeeding.key;
                localStorageEngine.getDataSource("/NormalFeedings", filter).then(function(filteredNormalFeeding) {
                    filteredNormalFeeding = filteredNormalFeeding[0];
                    filteredNormalFeeding.changed = !successSaveOnline;
                    filteredNormalFeeding.selectedByUser = normalFeeding.selectedByUser;
                    filteredNormalFeeding.__is_new = normalFeeding.__is_new;
                    filteredNormalFeeding.unit = normalFeeding.unit;
                    filteredNormalFeeding.id = normalFeeding.id;
                    filteredNormalFeeding.serviceDate = normalFeeding.serviceDate;
                    filteredNormalFeeding.observation = normalFeeding.observation;
                    if (successSaveOnline && successSaveOnline.dataset && successSaveOnline.dataset.error && Array.isArray(successSaveOnline.dataset.error.SEQUENTIAL) && successSaveOnline.dataset.error.SEQUENTIAL.length) {
                        filteredNormalFeeding[0].sequential = successSaveOnline.dataset.error.SEQUENTIAL[index];
                    }
                    localStorageEngine.save("/NormalFeedings", filteredNormalFeeding);
                });
            });

        });
        // saveProfile(widgetDetailsPatient);
    }

    function saveOnlineNormalFeedings(widgetPopupPrescricao, rowsToInsertOrUpdate, rowsToSave) {
        var saveOnline = ZHPromise.defer();
        // var widgetPopupPrescricao = widgetDetailsPatient.container.getWidget('widgetPopupPrescricao');
        if (!rowsToSave) {
            rowsToSave = rowsToInsertOrUpdate;
        }
        SynchronizeService.persistOnline('/NormalFeedings', '/NormalFeedings/save', 'DataSet', rowsToInsertOrUpdate).then(function(callback) {
            var currentRowPrescription = templateManager.bkpCurrentRowPrescription;
            var findParams = {
                'CDFILIAL': currentRowPrescription.unit,
                'type': 'normalFeedings'
            };
            SynchronizeService.persistOnline('/NormalFeedings', '/Structures/find', 'DataSet', [findParams]).then(function(callbackPlates) {
                callbackPlates.dataset.Plates.normalFeedings = Array.isArray(callbackPlates.dataset.Plates.normalFeedings) ? callbackPlates.dataset.Plates.normalFeedings : [];
                localStorageEngine.setDataSource('/NormalFeedings', callbackPlates.dataset.Plates.normalFeedings);
                updateControlFeedingRestriction(widgetPopupPrescricao, widgetPopupPrescricao.currentRow).then(function() {
                    saveOnline.reject();
                });
            }, function() {
                if (callback && callback.dataset && callback.dataset.error && Array.isArray(callback.dataset.error.SEQUENTIAL) && Array.isArray(rowsToSave)) {
                    rowsToSave.forEach(function(row, index) {
                        row.unique = callback.dataset.error.SEQUENTIAL[index];
                    });

                    MapNutritionistService.saveOralPrescriptionNormalFeedings(rowsToSave, false);
                }
                updateControlFeedingRestriction(widgetPopupPrescricao, widgetPopupPrescricao.currentRow).then(function() {
                    saveOnline.resolve(callback);
                });
            });
        }, function() {
            if (rowsToSave) {
                if (Array.isArray(rowsToSave)) {
                    rowsToSave.map(function(row) {
                        row.changed = true;
                    });
                } else {
                    rowsToSave.changed = true;
                }
                localStorageEngine.save('/NormalFeedings', rowsToSave).then(function() {
                    var currentRowPrescription = templateManager.bkpCurrentRowPrescription;
                    if (currentRowPrescription.idformdiet !== '4' && currentRowPrescription.dietService !== '4') {
                        updateControlFeedingRestriction(widgetPopupPrescricao, widgetPopupPrescricao.currentRow).then(function() {
                            saveOnline.resolve(false);
                        }).finally(function() {
                            saveOnline.resolve(false);
                        });
                    } else {
                        saveOnline.resolve(false);
                    }
                });
            }
        });
        return saveOnline;
    }
};

Configuration(function(ContextRegister) {
    ContextRegister.register('MapKitchenMaidController', MapKitchenMaidController);
});


// FILE: js/controllers/MapNutritionistController.js
var MapNutritionistController = function(BioEvaluationRepository, AppointmentService, localStorageEngine, ScreenService, MapNutritionistService, Environment, templateManager, eventAggregator, ProfileService, ProfilesRepository, PrescriptionService, Validation, PatientsRepository, ZHPromise, BedsService, CRUD, SynchronizeService, restEngine) {
    var self = this;
    var intoleranciaCrud = CRUD.init("/FoodIntoleranceGroup");
    var normalFeedingCrud = CRUD.init("/NormalFeedings");
    var controlFeedingCrud = CRUD.init("/ControlFeedings");

    eventAggregator.subscribe('callCheckNutritionist', checkNutritionist);

    function fixDateToSave(fieldEdited) {
        var fieldFixed = angular.copy(fieldEdited);
        fieldFixed.date = moment(fieldFixed.date, 'YYYY-MM-DDTHH:mm:ss').format('DD/MM/YYYY HH:mm:ss');
        return fieldFixed;
    }

    this.containerAfterinit = function() {
        if (!templateManager.addEventResize) {
            templateManager.keyboardOpen = false;
            window.addEventListener('resize', function() {
                templateManager.keyboardOpen = !templateManager.keyboardOpen;
            });
            templateManager.addEventResize = true;
        }
        var online = localStorageEngine.getLocalVar('__ONLINE');
        var syncronized = localStorageEngine.getLocalVar('__SYNCRONIZED');
        ScreenService.showLoader();
        localStorageEngine.getDataSource('/ParamGeral').then(function(paramGeral) {
            if (online === false && syncronized === true) {
                eventAggregator.publish("callCheckNutritionist", null);
            } else if (online === false && syncronized === false && paramGeral.length === 0) {
                var message = ScreenService.i18n('É necessário sincronizar os dados. Deseja realizar agora?');
                ScreenService.confirmMessage(message, 'question', function() {
                    eventAggregator.publish("wscCallUpdate", null);
                }, function() {
                    ScreenService.openWindow('Dashboard');
                });
            }
        }).finally(function() {
            ScreenService.hideLoader();
        });
    };

    function stopEditting(widget, isEditinCanceled) {

        var isEditingScreen = (widget.showAntrExamList === true && widget.showAddEditAntroExam === true) || (widget.showAddEditBioExam === true && widget.showBioqExamList === true);
        var isListingScreen = widget.showAntrExamList === true || widget.showBioqExamList === true || widget.showPopUpRestr;
        var isEdittingSingleField = widget.curEditing;

        if (widget.isSubeditting) {
            widget.isSubeditting = false;
            widget.subItemData = [];
            widget.view();
        } else if (widget.isLactaryFeedingEdit) {
            widget.view();
        } else if (isEditingScreen) {
            //Fechar editor de exame
            widget.showAddEditBioExam = false;
            widget.showAddEditAntroExam = false;
            widget.view();
        } else if (isListingScreen) {
            //Fechar lista de exame
            widget.selectedRows = [];
            widget.showBioqExamList = false;
            widget.showAntrExamList = false;
            widget.showPopUpRestr = false;
            widget.avaliationExam = {};
            widget.list();
        } else {
            //Fechar Editores simples
            widget.container.showMenu = true;
            widget.fieldEdited.isEditting = false;
            if (isEdittingSingleField) {
                var indexRemoved = widget.curPatient[widget.curEditing].indexOf(widget.fieldEdited);
                if (indexRemoved && indexRemoved > -1 && widget.fieldEdited.__is_new) {
                    widget.curPatient[widget.curEditing].splice(indexRemoved, 1);
                }
            }
            widget.fieldEdited = {};
            widget.list();
        }
        widget.subitemtoadd = '';
        if (Boolean(isEditinCanceled)) {
            returnBackUpRow(widget);
        }

    }

    function returnBackUpRow(widget) {

        if (Boolean(widget.isLactaryFeedingEdit)) {
            returnLactaryFeedingBackup(widget);
        }
        if (Boolean(widget.prescriptionBackup)) {
            returnPrescriptionBackup(widget);
        }
        if (Boolean(widget.appointmentBackup)) {
            returnAppointmentBackup(widget);
        }

        templateManager.updateTemplate();
    }

    function returnAppointmentBackup(widget) {
        var appointmentBackup = widget.appointmentBackup;
        var index;
        if (Boolean(appointmentBackup)) {
            widget.curPatient.Appointments.map(function(appointments, i) {
                if (appointments.date === appointmentBackup.date) {
                    index = i;
                }
            });
            widget.curPatient.Appointments[index] = widget.appointmentBackup;
            delete widget.appointmentBackup;
        }
    }

    function returnLactaryFeedingBackup(widget) {
        var lactaryFeedingBackUp = widget.lactaryFeedingBackUp;

        if (Boolean(lactaryFeedingBackUp)) {
            var curLactaryFeedingIndex;

            widget.currentRow.lactaryFeedings.map(function(a, i) {
                if (a.key === lactaryFeedingBackUp.key) {
                    curLactaryFeedingIndex = i;
                }
            });
            widget.currentRow.lactaryFeedings[curLactaryFeedingIndex] = lactaryFeedingBackUp;
        }
        deleteLactaryFeedingBackupParams(widget);
    }

    function deleteLactaryFeedingBackupParams(widget) {

        delete widget.lactaryFeedingBackUp;
        delete widget.isLactaryFeedingEdit;

    }

    function returnPrescriptionBackup(widget) {
        var prescriptionBackup = widget.prescriptionBackup;
        var index;
        if (Boolean(prescriptionBackup)) {
            widget.curPatient.Prescriptions.map(function(prescriptions, i) {
                if (prescriptions.date === prescriptionBackup.date) {
                    index = i;
                }
            });
            widget.curPatient.Prescriptions[index] = widget.prescriptionBackup;
            delete widget.prescriptionBackup;
        }
    }

    this.cancelPatientEditing = function(widget) {

        widget.isPatientEditing = false;
        widget.isHospitalizationEditing = false;
        widget.list();

        templateManager.updateTemplate();

    };

    function saveProfile(anyWidget) {
        var widget = anyWidget.container.getWidget('WidgetDetailsPatient');
        var curPatientInfo = widget.curPatient;
        var currentRow = widget.currentRow;
        var profile;

        if (widget.fieldEdited && Boolean(widget.fieldEdited.otherData)) {

            currentRow.otherdata = widget.curPatient.Profiles;
            profile = widget.fieldEdited.otherData[0];
            profile.fieldProfileEdit = true;
            profile[widget.fieldEdited.currentEditedName] = widget.fieldEdited.currentEditedModel;

        } else {

            profile = {};
            profile.id = curPatientInfo.id;
            profile.unit = curPatientInfo.unit;
            profile.onlyUpdateLastVist = true;
        }
        var main = anyWidget.container.getWidget('WidgetMain');
        profile.lastVisit = moment().format("DD/MM/YYYY HH:mm:ss");
        profile.confidentiality = profile.confidentiality ? profile.confidentiality : null;
        profile.changed = true;
        profile.__is_new = false;
        SynchronizeService.persistOnline('/Profiles', '/Profiles/save', 'DataSet', [profile]).then(function() {
            profile.changed = false;
            widget.curPatient.lastVisit = moment().format("DD/MM/YYYY HH:mm:ss");
            ProfilesRepository.save(profile);
            PatientsRepository.getPatientById(curPatientInfo.id).then(function(patient) {
                patient.lastVisit = moment().format("DD/MM/YYYY HH:mm:ss");
                PatientsRepository.save(patient).then(function() {
                    main.reload();
                });
            });
        }, function() {
            ProfilesRepository.save(profile);
        });

    }

    function NotFoundFromDataBase(widget, row) {
        var rowToLocalAdd = Array.isArray(widget.localDataSourceLoaded.FoodIntoleranceGroup) && widget.localDataSourceLoaded.FoodIntoleranceGroup.length ? widget.localDataSourceLoaded.FoodIntoleranceGroup.filter(function(filterData) {
            return row.selectedRow === true && filterData.cdpaciente === widget.curPatient.id && filterData.cdgrupprod === row.cdgrupprod && filterData.cdsubgrprod === row.cdsubgrprod;
        })[0] : null;
        if (rowToLocalAdd) {
            intoleranciaCrud.saveRow(rowToLocalAdd, "/FoodIntoleranceGroup");
            widget.dataFoodGroup = widget.dataFoodGroup.concat(angular.copy(rowToLocalAdd));
        }
        if (!rowToLocalAdd) {
            widget.wasChanged = true;
        }
        return rowToLocalAdd ? false : true;

    }

    function saveProfileIntoleranceGroup(widget) {
        var curPatientProfile = widget.curPatient;
        var selectedRows = widget.selectedRows;
        var rowsDiff = [];
        var rowsToAdd = [];
        if (widget.dataFoodGroup && widget.dataFoodGroup.length) {
            rowsDiff = diffArrays(widget.dataFoodGroup, widget.selectedRows);
            rowsDiff.forEach(function(row) {
                if (NotFoundFromDataBase(widget, row)) {
                    row.changed = true;
                    row.__is_new = false;
                    row.lastVisit = moment().format("DD/MM/YYYY HH:mm:ss");
                    row.id = curPatientProfile.id;
                    row.unit = curPatientProfile.unit;
                    row.fieldFoodGroup = true;
                    row.sequential = Date.now();
                    row.cdpaciente = curPatientProfile.id;
                    rowsToAdd.push(row);
                }
            });
        } else {
            selectedRows.forEach(function(row) {
                if (NotFoundFromDataBase(widget, row)) {
                    row.changed = true;
                    row.__is_new = false;
                    row.lastVisit = moment().format("DD/MM/YYYY HH:mm:ss");
                    row.id = curPatientProfile.id;
                    row.cdpaciente = curPatientProfile.id;
                    row.unit = curPatientProfile.unit;
                    row.fieldFoodGroup = true;
                    row.sequential = Date.now();
                    rowsToAdd.push(row);
                }
            });
        }
        var profile = angular.copy(rowsToAdd);
        if (profile && profile.length) {
            widget.savedChangeRestriction = true;
            widget.dataFoodGroup = widget.dataFoodGroup.concat(angular.copy(profile));
            localStorageEngine.save("/FoodIntoleranceGroup", profile).then(function() {
                profile.forEach(function(rowToSave) {
                    intoleranciaCrud.saveRow(rowToSave, "/FoodIntoleranceGroup");
                });
            });
        }
        deleteProfileIntoleranceGroup(widget);
        saveProfile(widget);
    }

    function deleteProfileIntoleranceGroup(widget) {
        var curPatientProfile = widget.curPatient;
        var popupFoodGroup = widget.popupFoodGroup;
        var rowsToDelete = [];

        if (widget.dataFoodGroup && widget.dataFoodGroup.length) {
            popupFoodGroup.forEach(function(row) {
                row.changed = true;
                row.__is_new = false;
                row.onlyUpdateLastVist = true;
                row.lastVisit = moment().format("DD/MM/YYYY HH:mm:ss");
                row.id = curPatientProfile.id;
                row.cdpaciente = curPatientProfile.id;
                row.unit = curPatientProfile.unit;
                row.cdfilial = curPatientProfile.unit;
                row.fieldFoodGroup = true;
                row.sequential = Date.now();
                if (row.selectedRow === false) {
                    rowsToDelete.push(row);
                }
            });
            var profileDelete = angular.copy(rowsToDelete);

            if (profileDelete && profileDelete.length) {
                intoleranciaCrud.removeDataset(profileDelete, "/FoodIntoleranceGroup").then(function() {
                    var index;
                    profileDelete.forEach(function(row) {
                        var deleteArray = widget.dataFoodGroup.filter(function(filterData) {
                            return filterData.cdgrupprod === row.cdgrupprod && filterData.cdsubgrprod === row.cdsubgrprod;
                        })[0];
                        index = widget.dataFoodGroup.indexOf(deleteArray);
                        if (index > -1) {
                            widget.dataFoodGroup.splice(index, 1);
                            widget.wasChanged = true;
                        }
                    });
                    controlVisibilityUpdateRestriction(widget);
                });
            } else if (widget.savedChangeRestriction) {
                controlVisibilityUpdateRestriction(widget);
            }
        }
        var rowsSelected = popupFoodGroup.filter(function(row) {
            return row.selectedRow;
        });
        widget.allFieldsAreSelected = rowsSelected.length === popupFoodGroup.length ? true : false;

        widget.showPopUpRestr = false;
        widget.list();
    }

    function controlVisibilityUpdateRestriction(widget) {
        if (widget.wasChanged === true) {
            updateProcedurePrescription(widget).then(function() {
                if (Array.isArray(widget.patientsToWarning)) {
                    var cdpaciente = widget.curPatient.id;
                    var index = widget.patientsToWarning.indexOf(cdpaciente);
                    if (index > -1) {
                        widget.patientsToWarning.splice(index, 1);
                    }
                }
                ScreenService.showMessage(ScreenService.i18n("Intolerâncias nas Prescrições do Controle Alimentar atualizadas com sucesso!"), 'success');
            }, function() {
                var cdpaciente = widget.curPatient.id;
                if (Array.isArray(widget.patientsToWarning)) {
                    var index = widget.patientsToWarning.indexOf(cdpaciente);
                    if (index === -1) {
                        widget.patientsToWarning.push(cdpaciente);
                    }
                } else {
                    widget.patientsToWarning = [];
                    widget.patientsToWarning.push(cdpaciente);
                }
                ScreenService.showMessage(ScreenService.i18n("Falha na conexão! Atualize as prescrições no Controle Alimentar para verificar inconsistências com as restrições desse paciente!"));

            });
        }
    }
    eventAggregator.subscribe('updateProcedureRestriction', function(widget) {
        controlVisibilityUpdateRestriction(widget);
    });

    eventAggregator.subscribe('callDeleteOnline', function(params) {
        var prescription = params.prescription;

        SynchronizeService.persistOnline('/ControlFeedings', '/ControlFeedings/remove', 'DataSet', [prescription]).then(function() {
            prescription.changed = false;
            prescription.successToRemoveOnline = true;
            controlFeedingCrud.removeRow(prescription, '/ControlFeedings');
        }, function() {
            controlFeedingCrud.removeRow(prescription, '/ControlFeedings');
        });
        saveProfile(params.widgetDetailsPatient);
    });

    eventAggregator.subscribe('callDeletePlatesOnline', function(params) {
        var dataSourceName = params.dataSourceName;
        var plates = params.plates;
        if (typeof plates === 'object' && !Array.isArray(plates)) {
            plates = [plates];
        }
        SynchronizeService.persistOnline(dataSourceName, dataSourceName + '/remove', 'DataSet', plates).then(function() {
            var findParams = {
                'CDFILIAL': params.widgetDetailsPatient.curPatient.unit
            };
            SynchronizeService.persistOnline('/NormalFeedings', '/Structures/find', 'DataSet', [findParams]).then(function(callback) {
                callback.dataset.Plates.normalFeedings = Array.isArray(callback.dataset.Plates.normalFeedings) ? callback.dataset.Plates.normalFeedings : [];
                callback.dataset.Plates.lactaryFeedings = Array.isArray(callback.dataset.Plates.lactaryFeedings) ? callback.dataset.Plates.lactaryFeedings : [];
                callback.dataset.Plates.enteralFeedings = Array.isArray(callback.dataset.Plates.enteralFeedings) ? callback.dataset.Plates.enteralFeedings : [];
                localStorageEngine.setDataSource('/NormalFeedings', callback.dataset.Plates.normalFeedings);
                localStorageEngine.setDataSource('/LactaryFeedings', callback.dataset.Plates.lactaryFeedings);
                localStorageEngine.setDataSource('/EnteralFeedings', callback.dataset.Plates.enteralFeedings);
            }, function() {
                plates.map(function(plate) {
                    plate.successToRemoveOnline = true;
                });
                controlFeedingCrud.removeDataset(plates, dataSourceName);
            });
        }, function() {
            controlFeedingCrud.removeDataset(plates, dataSourceName);
        });
        saveProfile(params.widgetDetailsPatient);
    });

    eventAggregator.subscribe('saveAlterPatient', function(params) {
        var rowToSave = params.rowToSave;
        var dataSourceName = params.dataSourceName;
        var route = params.route;
        var type = params.type;
        if (Array.isArray(rowToSave) && rowToSave.length) {
            SynchronizeService.persistOnline(dataSourceName, route, type, rowToSave).then(function() {
                rowToSave[0].changed = false;
            }).finally(function() {
                params.promise.resolve();
            });

        } else if (typeof rowToSave === 'object') {
            var promise = ZHPromise.defer();
            var arrayKeys = Object.keys(rowToSave);
            arrayKeys.forEach(function(key, index) {
                SynchronizeService.persistOnline(key, route, type, [rowToSave[key]]).then(function() {
                    rowToSave[key].changed = false;
                    if (index === arrayKeys.length - 1) {
                        promise.resolve();
                    }
                });
            });
            promise.then(function() {
                params.promise.resolve();

            });
        }
    });

    function updateProcedurePrescription(widget, filter) {
        var deferred = ZHPromise.defer();
        SynchronizeService.synchronizedIntolerance(widget, filter).then(function(callback) {
            deferred.resolve(callback);
        }, function(callback) {
            deferred.reject(callback);
        });
        return deferred.promise;
    }

    function diffArrays(foodGroup, selectedRows) {
        var diffArray = [];
        if ((foodGroup && !foodGroup.length) || (selectedRows && !selectedRows.length)) {
            return [];
        }
        selectedRows.forEach(function(elem) {
            var row = notInArray(foodGroup, elem);
            if (row) {
                diffArray.push(row);
            }
        });
        return diffArray;
    }

    function notInArray(array, elem) {
        var arr = array.filter(function(row) {
            return row.cdgrupprod === elem.cdgrupprod && row.cdsubgrprod === elem.cdsubgrprod;
        });
        return (arr && arr.length) ? null : elem;
    }

    function savePrescription(fieldEdited, widget) {

        var curPatientPrescriptions = widget.curPatient.Prescriptions;
        if (!curPatientPrescriptions) {
            curPatientPrescriptions = [];
        }
        var diets = widget.localDataSourceLoaded.Diets;

        diets.forEach(function(diet) {
            if (diet.dietId === fieldEdited.dietId) {
                fieldEdited.dietName = diet.dietName;
            }
        });
        if (!fieldEdited.prescription) {
            fieldEdited.prescription = '';
        }
        if (!fieldEdited.clinical) {
            fieldEdited.clinical = '';
        }
        if (!fieldEdited.medicalPrescription) {
            fieldEdited.medicalPrescription = '';
        }
        if (!fieldEdited.nutritionalStatusId) {
            fieldEdited.nutritionalStatusId = '';
        }
        setNewRecordOnReferenceArray(fieldEdited, curPatientPrescriptions);

        var newPrescription = fixDateToSave(fieldEdited);
        SynchronizeService.persistOnline('/Prescriptions', '/Prescriptions/save', 'DataSet', [newPrescription]).then(function() {
            newPrescription.changed = false;
            PrescriptionService.save(newPrescription);
        }, function() {
            PrescriptionService.save(newPrescription);
        });

    }
    eventAggregator.subscribe("AppointmentSave", function(parameters) {
        var widget = parameters[0];
        var avaliationName = parameters[1];
        saveAppointment(widget, avaliationName);
    });

    function isValidAppointmentDate(widget) {
        var parent = widget.getParent();
        var has = false;
        if (parent.curPatient.Appointments) {
            parent.curPatient.Appointments.forEach(function(appointment) {
                if (appointment.date === widget.currentRow.date) {
                    has = true;
                }
            });
        }
        if (widget.changed) {
            return true;
        }
        if (has && widget.currentRow.__is_new && widget.needToSave) {
            ScreenService.showMessage(ScreenService.i18n('Já existe uma consulta marcada para este horário!'));
            return false;
        }
        return true;
    }

    function saveAppointment(widget, avaliationName) {
        var existsAval = widget.existsAval;
        var widgetAppointments = widget.container.getWidget('appointmentsPopup');
        var currentRow = widgetAppointments.currentRow;
        var valid = isValidAppointmentDate(widgetAppointments);
        var aux = angular.copy(currentRow);
        if (currentRow.hasOwnProperty('$$hashKey')) {
            delete aux.$$hashKey;
        }
        if (JSON.stringify(widgetAppointments.backupRow) !== JSON.stringify(aux)) {
            widgetAppointments.needToSave = true;
            widgetAppointments.changed = true;
        }
        if (widgetAppointments.isValid()) {
            if (widgetAppointments.needToSave) {
                if (valid) {
                    var indexEdited;
                    if (widget.curPatient.Appointments) {
                        indexEdited = widget.curPatient.Appointments.indexOf(currentRow);
                    } else {
                        widget.curPatient.Appointments = [];
                        indexEdited = -1;
                    }
                    var data = widget.localDataSourceLoaded;
                    currentRow.unit = widget.curPatient.unit;
                    data.Diets.forEach(function(diet) {
                        if (diet.dietId === currentRow.dietId) {
                            currentRow.dietName = diet.dietName;
                        }
                    });

                    data.HealthInsurance.forEach(function(conv) {
                        if (conv.healthInsuranceId === currentRow.healthInsuranceId) {
                            currentRow.healthInsuranceName = conv.healthInsuranceName;
                        }
                    });

                    if (!currentRow.nutritionalStatusId) {
                        currentRow.nutritionalStatusId = '';
                    } else {
                        data.NutritionalStatus.forEach(function(nut) {
                            if (nut.nutritionalStatusId === currentRow.nutritionalStatusId) {
                                currentRow.nutritionalStatusName = nut.nutritionalStatusName;
                            }
                        });
                    }

                    if (!currentRow.medicalTeamId) {
                        currentRow.medicalTeamId = '';
                    } else {
                        data.MedicalTeams.forEach(function(eqp) {
                            if (eqp.medicalTeamId === currentRow.medicalTeamId) {
                                currentRow.medicalTeamName = eqp.medicalTeamName;
                            }
                        });
                    }

                    currentRow.normalizedDate = moment(currentRow.date, 'YYYY-MM-DDTHH:mm:ss').format('DD/MM/YYYY HH:mm:ss');
                    widget.curPatient.Appointments[indexEdited] = currentRow;
                    widget.curPatient.Appointments[indexEdited].changed = true;

                    widget.curPatient.Appointments[indexEdited].organization = getLocalVar('__NRORGTRAB');

                    var newAppointment = fixDateToSave(currentRow);
                    if (currentRow.__is_new && indexEdited === -1) {
                        widget.curPatient.Appointments.unshift(currentRow);
                    }

                    SynchronizeService.persistOnline('/Appointments', '/Appointments/save', 'DataSet', [newAppointment]).then(function() {
                        newAppointment.changed = false;
                        AppointmentService.save(newAppointment).then(function() {
                            Environment.lastPatientVisit();
                        });
                    }, function() {
                        AppointmentService.save(newAppointment).then(function() {
                            Environment.lastPatientVisit();
                        });
                    });

                    widget.fieldEdited = '';
                    saveProfile(widget);
                    widget.list();
                    Environment.setCurrentAppointmentDate(newAppointment.date);
                    widgetAppointments.needToSave = false;
                    widgetAppointments.changed = false;
                    widgetAppointments.actions[0].label = ScreenService.i18n('Voltar');
                    widgetAppointments.backupRow = angular.copy(widgetAppointments.currentRow);
                    localStorageEngine.setLocalVar('__normalizedAppointment', getLocalVar('__appointment'));
                }
            }
            if (valid) {
                if (avaliationName === 'bioq') {
                    widget.subitemtoadd = 'avaliationbioq';
                    if (existsAval) {
                        var bioExamList = widget.container.getWidget('bioqList');
                        bioExamList.edit();
                        ScreenService.openPopup(bioExamList);
                    } else {
                        openBioEvalPopup(widget);
                    }
                } else if (avaliationName === 'antr') {
                    widget.subitemtoadd = 'avaliationantr';
                    setLocalVar('__appointmentdata', currentRow);
                    var antrList = widget.container.getWidget('antrList');
                    eventAggregator.publish('avaliationsSaved', widget);
                    widget.selectedRows = [];
                    antrList.edit();
                    ScreenService.openPopup(antrList);
                } else {
                    ScreenService.closePopup();
                    widget.list();
                    widget.activate();
                }
            }
        }
    }

    function setNewRecordOnReferenceArray(record, referenceArray) {

        var recordIndex = referenceArray.indexOf(record);

        if (recordIndex === -1) {

            var copiedRecord = angular.copy(record);
            referenceArray.unshift(copiedRecord);

        } else {

            var filteredRecordProperties = Object.keys(referenceArray[recordIndex]);

            filteredRecordProperties.forEach(function(propertie) {
                referenceArray[recordIndex][propertie] = record[propertie];
            });

            referenceArray[recordIndex].changed = true;

        }

    }
    this.redirect = function(widget) {
        if (widget.isValid()) {
            eventAggregator.publish('changeSelectedExam', widget);
            saveBioEvaluation(widget);
        }
    };

    function saveBioEvaluation(widget) {
        var avaliation = widget.currentRow;
        var avaliationFixed;
        var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');

        if (widget.isValid()) {
            avaliation.changed = true;
            if (!avaliation.pregnancyWeek) {
                avaliation.pregnancyWeek = '';
            }
            widgetDetailsPatient.AvaliationFields[6].options.forEach(function(opcao) {
                if (avaliation.pregnancyId && avaliation.pregnancyId === opcao.value) {
                    avaliation.pregnancyName = opcao.name;
                }
            });
            if (widget.isSubItemBeingAdded) {
                avaliation.appointmentDate = getLocalVar('__normalizedAppointment');
                avaliation.age = widgetDetailsPatient.curPatient.age;
                avaliation.gender = widgetDetailsPatient.curPatient.gender;
                avaliation.id = widgetDetailsPatient.curPatient.id;
                avaliation.oldExaminationId = avaliation.examinationId;
                avaliation.unit = widgetDetailsPatient.curPatient.unit;
                avaliation.key = Date.now();

                avaliationFixed = angular.copy(avaliation);
                avaliationFixed.examinationDate = moment(avaliationFixed.examinationDate, 'YYYY-MM-DDTHH:mm:ss').format('DD/MM/YYYY HH:mm:ss');

                if (!widgetDetailsPatient.curPatient.BioEvaluations) {
                    widgetDetailsPatient.curPatient.BioEvaluations = [];
                }
                widgetDetailsPatient.curPatient.BioEvaluations.unshift(avaliation);

                SynchronizeService.persistOnline('/BioEvaluations', '/BioEvaluations/save', 'DataSet', [avaliationFixed]).then(function() {
                    avaliationFixed.changed = false;
                    avaliationFixed.__is_new = false;
                    BioEvaluationRepository.save(avaliationFixed);
                }, function() {
                    BioEvaluationRepository.save(avaliationFixed);

                });

                setDefaultDateFormaOnSolicitationDate(avaliation);
                widgetDetailsPatient.list();
                widgetDetailsPatient.activate();
                ScreenService.closePopup();
                if (!widgetDetailsPatient.existsAval) {
                    var bioExamList = widget.container.getWidget('bioqList');
                    bioExamList.edit();
                    ScreenService.openPopup(bioExamList);
                    widgetDetailsPatient.existsAval = true;
                }
            } else {
                var filter = [{
                    name: "key",
                    operator: "=",
                    value: avaliation.key
                }];
                localStorageEngine.getDataSource("/BioEvaluations", filter).then(function(filteredAvaliation) {
                    filteredAvaliation[0].key = avaliation.key;
                    filteredAvaliation[0].changed = true;
                    filteredAvaliation[0].examinationName = avaliation.examinationName;
                    filteredAvaliation[0].examinationUnit = avaliation.examinationUnit;
                    filteredAvaliation[0].oldExaminationId = filteredAvaliation[0].examinationId;
                    filteredAvaliation[0].examinationStatus = avaliation.examinationStatus;
                    filteredAvaliation[0].examinationId = avaliation.examinationId;
                    filteredAvaliation[0].examinationValue = avaliation.examinationValue;
                    filteredAvaliation[0].pregnancyId = avaliation.pregnancyId;
                    filteredAvaliation[0].pregnancyWeek = avaliation.pregnancyWeek;
                    var aux = moment(avaliation.examinationDate, "YYYY-MM-DDTHH:mm:ss");
                    filteredAvaliation[0].examinationDate = aux._i;
                    if (Array.isArray(widgetDetailsPatient.curPatient.BioEvaluations)) {
                        var index = widgetDetailsPatient.curPatient.BioEvaluations.findIndex(function(bioEvaluation) {
                            return bioEvaluation.key === avaliation.key;
                        });
                        if (index > -1) {
                            widgetDetailsPatient.curPatient.BioEvaluations[index] = angular.copy(filteredAvaliation[0]);
                        }
                    }
                    SynchronizeService.persistOnline('/BioEvaluations', '/BioEvaluations/save', 'DataSet', filteredAvaliation).then(function() {
                        filteredAvaliation[0].changed = false;
                        localStorageEngine.save("/BioEvaluations", filteredAvaliation);
                    }, function() {
                        localStorageEngine.save("/BioEvaluations", filteredAvaliation);

                    });
                });
                avaliation.examinationDate = moment(avaliation.examinationDate, "YYYY-MM-DDTHH:mm:ss").format("DD/MM/YYYY HH:mm:ss");
                ScreenService.closePopup();
            }
            saveProfile(widgetDetailsPatient);
        }
    }

    function setDefaultDateFormaOnSolicitationDate(avaliation) {
        avaliation.examinationDate = moment(avaliation.examinationDate).format("DD/MM/YYYY HH:mm:ss");
    }
    this.closeBioqPopup = function(widget) {
        ScreenService.closePopup();
        widget.list();
        widget.activate();
    };
    this.clearEdited = function(widget) {
        stopEditting(widget, true);
    };

    this.closePopupLactaryEnteral = function(widget) {
        ScreenService.closePopup();
        widget.container.getWidget('WidgetDetailsPatient').list();
        widget.container.getWidget('WidgetDetailsPatient').activate();
        widget.container.showMenu = true;
    };

    this.closePopupItemLacEnt = function(widget) {
        ScreenService.closePopup();
        widget.container.getWidget('widgetPopupPrescricao').view();
    };

    this.closePopup = function(widget) {
        closePopupPrescription(widget);
    };

    function closePopupPrescription(widgetPopup) {
        var widgetPatient = widgetPopup.container.getWidget('WidgetDetailsPatient');
        var widgetPopupPlates = widgetPopup.container.getWidget('widgetPopupPlates');
        widgetPopup.currentRow = {};
        widgetPopupPlates.selectedRows = [];
        widgetPopup.filtredFeedingsForWidget = [];
        ScreenService.closePopup();
        widgetPopup.isVisible = false;
        widgetPatient.list();
        widgetPatient.activate();
        widgetPopup.container.showMenu = true;
    }

    this.saveVal = function(widget) {

        var fieldEdited = widget.fieldEdited;

        if (fieldEdited) {
            var originalIsNewStatus = fieldEdited.originalIsNewStatus ? fieldEdited.originalIsNewStatus : null;
            fieldEdited.__is_new = originalIsNewStatus && originalIsNewStatus === true ? true : fieldEdited.__is_new;
        }

        widget.curEditing = '';
        switch (widget.currentScreen) {
            case 'personalData':
                if (widget.showPopUpRestr === true) {
                    saveProfileIntoleranceGroup(widget);
                } else {
                    saveProfile(widget);
                    stopEditting(widget);
                }
                break;

            case 'nutriApointment':
                if (widget.subitemtoadd === 'avaliationantr') {
                    if (widget.container.getWidget('antrAvaliationPopup').isValid()) {
                        AppointmentService.anthCalculate(widget);
                        saveProfile(widget);
                    }
                } else {
                    saveAppointment(widget);
                    widget.infos = '';
                }
                break;
        }
    };

    this.cancelButton = function(widget) {
        returnBackUpRow(widget);
        ScreenService.closePopup();
        widget.list();
        widget.activate();
    };

    this.cancelBioEval = function(widget) {
        var current = widget.currentRow;
        current.examinationDate = moment(current.examinationDate, "YYYY-MM-DDTHH:mm:ss").format("DD/MM/YYYY HH:mm:ss");
        ScreenService.closePopup();
        widget.container.getWidget('WidgetDetailsPatient').list();
    };

    this.saveDietPrescription = function(widget) {
        var currentRow = widget.currentRow;
        var patientWidget = widget.container.getWidget('WidgetDetailsPatient');
        if (widget.isValid()) {
            if (!patientWidget.prescriptionBackup) {
                var duplicatedDate = false;
                if (patientWidget.curPatient.Prescriptions) {
                    patientWidget.curPatient.Prescriptions.filter(function(prescriptions) {
                        if (prescriptions.date === currentRow.date) {
                            duplicatedDate = true;
                        }
                    });
                    if (duplicatedDate) {
                        ScreenService.showMessage(ScreenService.i18n('Já existe uma prescrição marcada para este horário!'));
                        return false;
                    }
                }
            }
            patientWidget.fieldEdited = '';
            savePrescription(currentRow, patientWidget);
            saveProfile(patientWidget);
            patientWidget.list();
            ScreenService.closePopup();
        }
    };

    this.saveNewPrescription = function(widget) {

        var patientWidget = widget.container.getWidget('WidgetDetailsPatient');
        var widgetPrescricao = widget.container.getWidget('widgetPopupPrescricao');
        var currentRow = widgetPrescricao.currentRow;
        var interval = widget.currentRow.interval;
        var period = widget.currentRow.period;
        var intervalMin;
        var periodMin;
        if (interval && period) {
            intervalMin = (parseInt(interval.split(':')[0]) * 60) + parseInt(interval.split(':')[1]);
            periodMin = (parseInt(period.split(':')[0]) * 60) + parseInt(period.split(':')[1]);
        }
        var fieldAutomatic = widget.getField('automatic');
        var isAutomatic;
        if (fieldAutomatic) {
            isAutomatic = widget.getField('automatic').value();
        }

        if (currentRow.isNewRegister) {
            currentRow.DTINCLUSAO = moment().format('DD/MM/YYYY HH:mm:ss');
        }
        var msg;
        if (widget.isValid()) {
            switch (currentRow.idtpservhosp) {
                case "0":
                case "3":
                case "4":
                case null:
                    if (widget.filtredFeedingsForWidget && widget.filtredFeedingsForWidget.length) {
                        var count = widget.filtredFeedingsForWidget.filter(function(feeding) {
                            return feeding.selectedByUser === true;
                        }).length;
                        if (count === 0 && widget.currentRow.dietService !== '4' && widget.currentRow.idformdiet !== '4') {
                            msg = ScreenService.i18n("Selecione algum prato para salvar a prescrição!");
                            ScreenService.showMessage(msg);
                        } else {
                            saveOralPrescription(widget);
                        }
                    } else if ((currentRow.idtpservhosp === '0' || currentRow.idtpservhosp === '3' ||
                            currentRow.idtpservhosp === '4') || !currentRow.idtpservhosp) {
                        if (widget.currentRow.dietService === '4' || widget.currentRow.idformdiet === '4') {
                            msg = ScreenService.i18n("Selecione algum prato para salvar a prescrição!");
                            ScreenService.showMessage(msg);
                        } else {
                            msg = ScreenService.i18n("Não há cardápio para a configuração informada!");
                            ScreenService.showMessage(msg);
                        }
                    }
                    break;

                case "2":
                    widget.container.getWidget('widgetPopupPrescricao').currentRow.curLactaryFeeding = widget.container.getWidget('widgetPopupItemLac').currentRow;
                    var row = angular.copy(currentRow);
                    row.serviceDate = moment(row.serviceDate, "YYYY-MM-DD").format("DD/MM/YYYY");
                    var contIndex = row.indexOfOriginalControlFeedings;
                    var patiIndex = row.indexOfOriginalCurPatFeedings;
                    if (currentRow.isNewRegister) {
                        currentRow.__is_new = true;
                        saveLactaryPrescription(widget);
                    } else {
                        if (isValidHour(widget.getField('hourLactaryPlate').value())) {
                            if (isAutomatic === 'S') {
                                if (isTimeValid(interval, period)) {
                                    if (intervalMin > periodMin) {
                                        msg = ScreenService.i18n("Tempo de intervalo deve ser menor do que período");
                                        ScreenService.showMessage(msg);
                                    } else if (intervalMin === 0 || periodMin === 0) {
                                        msg = ScreenService.i18n("Tempo de intervalo e período deve ser maior do que 0");
                                        ScreenService.showMessage(msg);
                                    } else {
                                        saveLactaryFeeding(widget);
                                    }
                                }
                            } else {
                                saveLactaryFeeding(widget);
                            }
                        } else {
                            msg = ScreenService.i18n('Hora Ingestão inválida!');
                            ScreenService.showMessage(msg);
                        }
                    }
                    break;

                case "1":
                    if (currentRow.__is_new && !widget.subItemData) {
                        currentRow.serviceDateDefault = currentRow.serviceDate;
                        currentRow.serviceDate = moment(currentRow.serviceDate).format('DD/MM/YYYY');
                        currentRow.id = patientWidget.curPatient.id;
                        currentRow.cdservfili = currentRow.serviceId;
                        currentRow.nutritionist = localStorageEngine.getLocalVar('__CDOPERADOR');
                        currentRow.hospitalization = patientWidget.curPatient.hospitalization;
                        var rowToSave = saveParenteralPrescription(widget);
                        patientWidget.localDataSourceLoaded.ControlFeedings.unshift(rowToSave);
                        patientWidget.curPatient.ControlFeedings.unshift(rowToSave);
                    } else {
                        var fieldVolPrescription = widget.getField('volPrescription');
                        var fieldInfudido = widget.getField('volInfudido');
                        if (fieldVolPrescription && fieldInfudido && fieldInfudido.value() > fieldVolPrescription.value()) {
                            ScreenService.showMessage(ScreenService.i18n('A quantidade infundida deve ser menor ou igual a quantidade prescrita.'));
                        } else {
                            if (isValidHour(widget.getField('time').value())) {
                                if (isAutomatic === 'S') {
                                    if (isTimeValid(interval, period)) {
                                        if (intervalMin > periodMin) {
                                            msg = ScreenService.i18n("Tempo de intervalo deve ser menor do que período!");
                                            ScreenService.showMessage(msg);
                                        } else if (intervalMin === 0 || periodMin === 0) {
                                            msg = ScreenService.i18n("Tempo de intervalo e período deve ser maior do que 0!");
                                            ScreenService.showMessage(msg);
                                        } else {
                                            saveEnteralFeeding(widget);
                                            widget.subItemData = null;
                                            patientWidget.localDataSourceLoaded.ControlFeedings[contIndex] = row;
                                            patientWidget.curPatient.ControlFeedings[patiIndex] = row;
                                        }
                                    }
                                } else {
                                    saveEnteralFeeding(widget);
                                    widget.subItemData = null;
                                    patientWidget.localDataSourceLoaded.ControlFeedings[contIndex] = row;
                                    patientWidget.curPatient.ControlFeedings[patiIndex] = row;
                                }
                            } else {
                                msg = ScreenService.i18n('Hora Ingestão inválida!');
                                ScreenService.showMessage(msg);
                            }
                        }
                    }
                    break;
            }
        }
    };

    function isValidHour(hour) {
        hour = hour.replace(':', '');
        var aux = moment(hour, "HHmm").format("HHmm");
        if (hour === aux) {
            return true;
        } else {
            return false;
        }
    }

    function isTimeValid(interval, period) {
        var minuteInterval = parseInt(interval.split(':')[1]);
        var minutePeriod = parseInt(period.split(':')[1]);
        var msg;

        if (minuteInterval < 60) {
            if (minutePeriod < 60) {
                return true;
            } else {
                msg = ScreenService.i18n('Período inválido!');
                ScreenService.showMessage(msg);
                return false;
            }
        } else {
            msg = ScreenService.i18n('Intervalo inválido!');
            ScreenService.showMessage(msg);
            return false;
        }

    }

    function saveOralPrescription(widget) {
        var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');
        var nonFormatedDate = widget.currentRow.serviceDate;
        var formatedData = moment(widget.currentRow.serviceDate).format("DD/MM/YYYY");
        widget.currentRow.serviceDate = widget.currentRow.__is_new && formatedData !== "Invalid date" ? formatedData : widget.currentRow.serviceDate;
        templateManager.bkpCurrentRowPrescription = angular.copy(widget.currentRow);
        var currentRow = widget.currentRow;
        var normalFeedings = widget.filtredFeedingsForWidget;
        if (currentRow && currentRow.isNewRegister) {
            currentRow.hospitalization = widgetDetailsPatient.curPatient.hospitalization;
            currentRow.id = widgetDetailsPatient.curPatient.id;
            currentRow.serviceDateDefault = nonFormatedDate;
            currentRow.cdservfili = currentRow.serviceId;
            var rowToSave = createRowToSavePrescription(widget);
            currentRow.serviceDate = formatedData;
            var promise = ZHPromise.defer();
            SynchronizeService.persistOnline('/ControlFeedings', '/ControlFeedings/save', 'DataSet', [rowToSave]).then(function(callback) {
                rowToSave.changed = false;
                rowToSave.__is_new = false;
                if (callback && callback.dataset && callback.dataset.error && callback.dataset.error.SEQUENTIAL) {
                    rowToSave.sequential = callback.dataset.error.SEQUENTIAL;
                }
                MapNutritionistService.savePrescription(rowToSave);
            }, function() {
                MapNutritionistService.savePrescription(rowToSave);
            }).finally(function() {
                promise.resolve();
            });
            promise.then(function() {
                saveOralPrescriptionNormalFeedings(rowToSave, normalFeedings, widgetDetailsPatient).then(function() {
                    widgetDetailsPatient.wasChanged = true;
                    if (currentRow.idformdiet !== '4' && currentRow.dietService !== '4') {
                        updateProcedurePrescription(widgetDetailsPatient, {
                            id: currentRow.id,
                            serviceDate: currentRow.serviceDate,
                            DTINCLUSAO: rowToSave.DTINCLUSAO
                        }).then(function() {
                            if (Array.isArray(widgetDetailsPatient.patientsToWarning)) {
                                var cdpaciente = widgetDetailsPatient.curPatient.id;
                                var index = widgetDetailsPatient.patientsToWarning.indexOf(cdpaciente);
                                if (index > -1) {
                                    widgetDetailsPatient.patientsToWarning.splice(index, 1);
                                }
                            }
                            ScreenService.showMessage(ScreenService.i18n("Restrições alimentares verificadas com sucesso para esta prescrição!"), 'success');
                        }, function() {
                            var cdpaciente = widgetDetailsPatient.curPatient.id;
                            if (Array.isArray(widgetDetailsPatient.patientsToWarning)) {
                                var index = widgetDetailsPatient.patientsToWarning.indexOf(cdpaciente);
                                if (index === -1) {
                                    widgetDetailsPatient.patientsToWarning.push(cdpaciente);
                                }
                            } else {
                                widgetDetailsPatient.patientsToWarning = [];
                                widgetDetailsPatient.patientsToWarning.push(cdpaciente);
                            }
                            ScreenService.showMessage(ScreenService.i18n("Falha na conexão! Atualize as prescrições no Controle Alimentar para verificar inconsistências com as restrições desse paciente!"));
                        });
                    }
                });
                widgetDetailsPatient.localDataSourceLoaded.ControlFeedings.unshift(rowToSave);
                widgetDetailsPatient.curPatient.ControlFeedings.unshift(rowToSave);
            });
        } else {
            if (widget.currentRow.idformdiet === '4' || widget.currentRow.dietService === '4') {
                saveSpecialDiet(widget);
            } else {
                saveNormalFeedingsEdit(widget);
            }
        }
        closePopupPrescription(widget);
        saveProfile(widgetDetailsPatient);
    }

    function rowsToInsertUpdateDeleteNormalFeedings(widgetPopupPrescricao) {
        var rowsToInsert = [];
        var rowsToUpdate = [];
        var rowsToDelete = [];
        var rows;
        widgetPopupPrescricao.filtredFeedingsForWidget.forEach(function(feeding) {
            rowsToDelete = widgetPopupPrescricao.bkpFeedings.filter(function(bkp) {
                if ((bkp.formulationId !== feeding.formulationId || bkp.plateType !== feeding.plateType) && !feeding.key && !feeding.alreadyInserted) {
                    rowsToInsert.push(feeding);
                    feeding.alreadyInserted = true;
                }
                if ((bkp.formulationId === feeding.formulationId && bkp.plateType === feeding.plateType) && !bkp.remove && feeding.key) {
                    rowsToUpdate.push(feeding);
                }
                return (bkp.formulationId !== feeding.formulationId || bkp.plateType !== feeding.plateType) && bkp.remove;
            });
        });
        rowsToDelete.map(function(row) {
            row.manualDelete = true;
        });
        rows = {
            'rowsToInsert': rowsToInsert,
            'rowsToUpdate': rowsToUpdate,
            'rowsToDelete': rowsToDelete
        };
        return rows;
    }

    function saveSpecialDiet(widgetPopupPrescricao) {
        var widgetDetailsPatient = widgetPopupPrescricao.container.getWidget('WidgetDetailsPatient');
        var currentRow = angular.copy(widgetPopupPrescricao.currentRow);
        var rows = rowsToInsertUpdateDeleteNormalFeedings(widgetPopupPrescricao);
        var normalFeedings = rows.rowsToUpdate;
        var savedOnline;
        var rowsToSave;
        if (Array.isArray(rows.rowsToInsert) && rows.rowsToInsert.length) {
            currentRow.serviceDate = moment(currentRow.serviceDate, 'YYYY-MM-DD').format('DD/MM/YYYY');
            rowsToSave = createRowsToRelateNormalFeedingsToOralPrescription(widgetPopupPrescricao.currentRow, rows.rowsToInsert, widgetDetailsPatient);
        }
        if (Array.isArray(rows.rowsToDelete) && rows.rowsToDelete.length) {
            SynchronizeService.persistOnline('/NormalFeedings', '/NormalFeedings/remove', 'DataSet', rows.rowsToDelete).then(function() {
                var findParams = {
                    'CDFILIAL': widgetDetailsPatient.curPatient.unit,
                    'type': 'normalFeedings'
                };
                SynchronizeService.persistOnline('/NormalFeedings', '/Structures/find', 'DataSet', [findParams]).then(function(callback) {
                    callback.dataset.Plates.normalFeedings = Array.isArray(callback.dataset.Plates.normalFeedings) ? callback.dataset.Plates.normalFeedings : [];
                    // callback.dataset.Plates.lactaryFeedings = Array.isArray(callback.dataset.Plates.lactaryFeedings) ? callback.dataset.Plates.lactaryFeedings : [];
                    localStorageEngine.setDataSource('/NormalFeedings', callback.dataset.Plates.normalFeedings);
                    // localStorageEngine.setDataSource('/LactaryFeedings', callback.dataset.Plates.lactaryFeedings);
                }, function() {
                    rows.rowsToDelete.forEach(function(rowDelete) {
                        rowDelete.successToRemoveOnline = true;
                        normalFeedingCrud.removeRow(rowDelete, '/NormalFeedings');
                    });
                });
            }, function() {
                rows.rowsToDelete.forEach(function(rowDelete) {
                    normalFeedingCrud.removeRow(rowDelete, '/NormalFeedings');
                });
            });

        }
        var rowsToSaveInsert = createRowsToRelateNormalFeedingsToOralPrescription(widgetPopupPrescricao.currentRow, rows.rowsToInsert, widgetDetailsPatient);
        var rowsToInsertOrUpdate = rowsToSaveInsert.concat(rows.rowsToUpdate);
        if (Array.isArray(rowsToInsertOrUpdate) && rowsToInsertOrUpdate.length) {
            savedOnline = saveOnlineNormalFeedings(widgetDetailsPatient, rowsToInsertOrUpdate, rowsToSave);
        }
        savedOnline.then(function(successSaveOnline) {
            normalFeedings.forEach(function(normalFeeding, index) {
                var filter = [{
                    name: 'key',
                    operator: '=',
                    value: ''
                }];
                filter[0].value = normalFeeding.key;
                localStorageEngine.getDataSource("/NormalFeedings", filter).then(function(filteredNormalFeeding) {
                    filteredNormalFeeding = filteredNormalFeeding[0];
                    filteredNormalFeeding.changed = !successSaveOnline;
                    filteredNormalFeeding.selectedByUser = normalFeeding.selectedByUser;
                    filteredNormalFeeding.__is_new = normalFeeding.__is_new;
                    filteredNormalFeeding.unit = normalFeeding.unit;
                    filteredNormalFeeding.id = normalFeeding.id;
                    filteredNormalFeeding.serviceDate = normalFeeding.serviceDate;
                    filteredNormalFeeding.observation = normalFeeding.observation;
                    if (successSaveOnline && successSaveOnline.dataset && successSaveOnline.dataset.error && Array.isArray(successSaveOnline.dataset.error.SEQUENTIAL) && successSaveOnline.dataset.error.SEQUENTIAL.length) {
                        filteredNormalFeeding[0].sequential = successSaveOnline.dataset.error.SEQUENTIAL[index];
                    }
                    localStorageEngine.save("/NormalFeedings", filteredNormalFeeding);
                });
            });

        });
        saveProfile(widgetDetailsPatient);
    }

    function saveOnlineNormalFeedings(widgetDetailsPatient, rowsToInsertOrUpdate, rowsToSave) {
        var saveOnline = ZHPromise.defer();
        var widgetPopupPrescricao = widgetDetailsPatient.container.getWidget('widgetPopupPrescricao');
        if (!rowsToSave) {
            rowsToSave = rowsToInsertOrUpdate;
        }
        SynchronizeService.persistOnline('/NormalFeedings', '/NormalFeedings/save', 'DataSet', rowsToInsertOrUpdate).then(function(callback) {
            var findParams = {
                'CDFILIAL': widgetDetailsPatient.curPatient.unit,
                'type': 'normalFeedings'
            };
            SynchronizeService.persistOnline('/NormalFeedings', '/Structures/find', 'DataSet', [findParams]).then(function(callbackPlates) {
                callbackPlates.dataset.Plates.normalFeedings = Array.isArray(callbackPlates.dataset.Plates.normalFeedings) ? callbackPlates.dataset.Plates.normalFeedings : [];
                // callbackPlates.dataset.Plates.lactaryFeedings = Array.isArray(callbackPlates.dataset.Plates.lactaryFeedings) ? callbackPlates.dataset.Plates.lactaryFeedings : [];
                localStorageEngine.setDataSource('/NormalFeedings', callbackPlates.dataset.Plates.normalFeedings);
                // localStorageEngine.setDataSource('/LactaryFeedings', callbackPlates.dataset.Plates.lactaryFeedings);
                updateControlFeedingRestriction(widgetDetailsPatient, widgetPopupPrescricao.currentRow).then(function() {
                    saveOnline.reject();
                });
            }, function() {
                if (callback && callback.dataset && callback.dataset.error && Array.isArray(callback.dataset.error.SEQUENTIAL) && Array.isArray(rowsToSave)) {
                    rowsToSave.forEach(function(row, index) {
                        row.unique = callback.dataset.error.SEQUENTIAL[index];
                    });

                    MapNutritionistService.saveOralPrescriptionNormalFeedings(rowsToSave, false);
                }
                updateControlFeedingRestriction(widgetDetailsPatient, widgetPopupPrescricao.currentRow).then(function() {
                    saveOnline.resolve(callback);
                });
            });
        }, function() {
            if (rowsToSave) {
                if (Array.isArray(rowsToSave)) {
                    rowsToSave.map(function(row) {
                        row.changed = true;
                    });
                } else {
                    rowsToSave.changed = true;
                }
                localStorageEngine.save('/NormalFeedings', rowsToSave).then(function() {
                    var currentRowPrescription = templateManager.bkpCurrentRowPrescription;
                    if (currentRowPrescription.idformdiet !== '4' && currentRowPrescription.dietService !== '4') {
                        updateControlFeedingRestriction(widgetDetailsPatient, widgetPopupPrescricao.currentRow).then(function() {
                            saveOnline.resolve(false);
                        }).finally(function() {
                            saveOnline.resolve(false);
                        });
                    } else {
                        saveOnline.resolve(false);
                    }
                });
            }
        });
        return saveOnline;
    }

    function saveNormalFeedingsEdit(widget) {

        var widgetPopupPrescricao = widget.container.getWidget('widgetPopupPrescricao');
        var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');
        var currentRow = angular.copy(widgetPopupPrescricao.currentRow);
        var normalFeedings = widgetPopupPrescricao.filtredFeedingsForWidget;
        var filter = [{
                "name": "unit",
                "operator": "=",
                "value": localStorageEngine.getLocalVar('__CDFILIAL')
            },
            {
                "name": "id",
                "operator": "=",
                "value": localStorageEngine.getLocalVar('__patient')
            },
            {
                "name": "serviceDate",
                "operator": "=",
                "value": moment(currentRow.serviceDate, 'YYYY-MM-DD').format('DD/MM/YYYY')
            },
            {
                "name": "sequential",
                "operator": "=",
                "value": currentRow.sequential
            },
            {
                "name": "unique",
                "operator": "=",
                "value": ""
            }
        ];
        //var rowsToSave = //createRowsToRelateNormalFeedingsToOralPrescription(currentRow, normalFeedings, widgetDetailsPatient);
        if (normalFeedings && normalFeedings.length) {
            var savedOnline = saveOnlineNormalFeedings(widgetDetailsPatient, normalFeedings);
            var promisesSaveNormalFeedingsEdit = [];
            savedOnline.then(function(successSaveOnline) {
                if (successSaveOnline) {
                    normalFeedings.forEach(function(normalFeeding) {
                        var filterCopied = angular.copy(filter);
                        filterCopied[filterCopied.length - 1].value = normalFeeding.unique;
                        localStorageEngine.getDataSource("/NormalFeedings", filterCopied).then(function(filteredNormalFeeding) {
                            filteredNormalFeeding[0].changed = !successSaveOnline;
                            filteredNormalFeeding[0].selectedByUser = normalFeeding.selectedByUser;
                            filteredNormalFeeding[0].__is_new = normalFeeding.__is_new;
                            filteredNormalFeeding[0].unit = normalFeeding.unit;
                            filteredNormalFeeding[0].id = normalFeeding.id;
                            filteredNormalFeeding[0].serviceDate = normalFeeding.serviceDate;
                            filteredNormalFeeding[0].observation = normalFeeding.observation;
                            promisesSaveNormalFeedingsEdit.push(localStorageEngine.save("/NormalFeedings", filteredNormalFeeding));
                        });
                    });
                    ZHPromise.all(promisesSaveNormalFeedingsEdit).then(function() {
                        updateControlFeedingRestriction(widget, currentRow);
                    });
                }
            });

        }
    }

    function updateControlFeedingRestriction(widget, prescription) {
        var endUpdate = ZHPromise.defer();
        var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');
        var serviceDate = moment(prescription.serviceDate, "DD/MM/YYYY");
        var currentRowPrescription = templateManager.bkpCurrentRowPrescription;
        if (serviceDate.isValid()) {
            serviceDate = prescription.serviceDate;
        } else {
            serviceDate = moment(prescription.serviceDate, "YYYY-MM-DD").format("DD/MM/YYYY");
        }
        var filterControlFeedings = {
            "id": widgetDetailsPatient.curPatient.id,
            "serviceDate": moment(currentRowPrescription.serviceDate, 'YYYY-MM-DD').format('DD/MM/YYYY'),
            "DTINCLUSAO": currentRowPrescription.DTINCLUSAO
        };
        widgetDetailsPatient.wasChanged = true;
        updateProcedurePrescription(widgetDetailsPatient, filterControlFeedings).then(function() {
            if (Array.isArray(widgetDetailsPatient.patientsToWarning)) {
                var cdpaciente = widgetDetailsPatient.curPatient.id;
                var index = widgetDetailsPatient.patientsToWarning.indexOf(cdpaciente);
                if (index > -1) {
                    widgetDetailsPatient.patientsToWarning.splice(index, 1);
                }
            }
            endUpdate.resolve();
            ScreenService.showMessage(ScreenService.i18n("Restrições alimentares verificadas com sucesso para esta prescrição!"), 'success');
        }, function() {
            var cdpaciente = widgetDetailsPatient.curPatient.id;
            if (Array.isArray(widgetDetailsPatient.patientsToWarning)) {
                var index = widgetDetailsPatient.patientsToWarning.indexOf(cdpaciente);
                if (index === -1) {
                    widgetDetailsPatient.patientsToWarning.push(cdpaciente);
                }
            } else {
                widgetDetailsPatient.patientsToWarning = [];
                widgetDetailsPatient.patientsToWarning.push(cdpaciente);
            }
            endUpdate.resolve();
            ScreenService.showMessage(ScreenService.i18n("Falha na conexão! Atualize as prescrições no Controle Alimentar para verificar inconsistências com as restrições desse paciente!"));
        });
        return endUpdate;
    }

    function createRowToSavePrescription(widget) {

        var copiedFieldEdited = angular.copy(widget.currentRow);

        copiedFieldEdited.changed = true;
        copiedFieldEdited.sequential = Date.now();
        copiedFieldEdited.service = copiedFieldEdited.serviceName;
        copiedFieldEdited.serviceId = copiedFieldEdited.idtpservhosp;
        copiedFieldEdited.serviceCode = copiedFieldEdited.serviceId;
        copiedFieldEdited.unit = localStorageEngine.getLocalVar('__CDFILIAL');
        copiedFieldEdited.nutritionist = localStorageEngine.getLocalVar('__CDOPERADOR');
        copiedFieldEdited.cdservfili = widget.currentRow.serviceId;
        var curDate = moment(copiedFieldEdited.serviceDateDefault).format("DD/MM/YYYY");
        copiedFieldEdited.serviceDate = curDate === "Invalid date" ? copiedFieldEdited.serviceDate : curDate;

        return copiedFieldEdited;
    }

    function saveOralPrescriptionNormalFeedings(currentOralPrescription, normalFeedings, widgetDetailsPatient) {
        var endSave = ZHPromise.defer();
        var rowsToSave = createRowsToRelateNormalFeedingsToOralPrescription(currentOralPrescription, normalFeedings, widgetDetailsPatient);
        SynchronizeService.persistOnline('/NormalFeedings', '/NormalFeedings/save', 'DataSet', rowsToSave).then(function(callback) {
            var findParams = {
                'CDFILIAL': widgetDetailsPatient.curPatient.unit,
                'type': 'normalFeedings'
            };
            SynchronizeService.persistOnline('/NormalFeedings', '/Structures/find', 'DataSet', [findParams]).then(function(callbackPlates) {
                callbackPlates.dataset.Plates.normalFeedings = Array.isArray(callbackPlates.dataset.Plates.normalFeedings) ? callbackPlates.dataset.Plates.normalFeedings : [];
                // callbackPlates.dataset.Plates.lactaryFeedings = Array.isArray(callbackPlates.dataset.Plates.lactaryFeedings) ? callbackPlates.dataset.Plates.lactaryFeedings : [];
                // var promises = [];
                // promises.push();
                // promises.push(localStorageEngine.setDataSource('/LactaryFeedings', callbackPlates.dataset.Plates.lactaryFeedings));
                widgetDetailsPatient.curPatient.NormalFeedings = callbackPlates.dataset.Plates.normalFeedings.filter(function(normalFeeding) {
                    return normalFeeding.id === widgetDetailsPatient.curPatient.id;
                });
                // widgetDetailsPatient.curPatient.LactaryFeedings = callbackPlates.dataset.Plates.lactaryFeedings.filter(function(lactaryFeeding) {
                //     return lactaryFeeding.id === widgetDetailsPatient.curPatient.id;
                // });
                localStorageEngine.setDataSource('/NormalFeedings', callbackPlates.dataset.Plates.normalFeedings).then(function() {
                    endSave.resolve();
                });
            }, function() {
                if (callback && callback.dataset && callback.dataset.error && Array.isArray(callback.dataset.error.SEQUENTIAL) && callback.dataset.error.SEQUENTIAL.length && Array.isArray(rowsToSave) && rowsToSave.length) {
                    rowsToSave.forEach(function(row, index) {
                        row.unique = callback.dataset.error.SEQUENTIAL[index];
                    });
                }
                MapNutritionistService.saveOralPrescriptionNormalFeedings(rowsToSave, false).then(function() {
                    endSave.resolve();
                });
            });
        }, function() {
            localStorageEngine.save('/NormalFeedings', rowsToSave).then(function() {
                endSave.resolve();
            });
        });
        return endSave;
    }

    function createRowsToRelateNormalFeedingsToOralPrescription(oralPrescription, normalFeedings, widgetDetailsPatient) {

        var unique = 0;
        var objects = [];

        normalFeedings.forEach(function(normalFeeding) {
            unique++;
            var serviceDate = moment(oralPrescription.serviceDate, 'DD/MM/YYYY');
            // if (dish.IDOPCCNFCPME === 'S') dish.NMTIPOPRAT += ' (Opção)';
            var plateTypeName = oralPrescription.idformdiet === "4" || oralPrescription.dietService === "4" ? normalFeeding.plateTypeName : normalFeeding.plateType;
            var object = {
                "id": oralPrescription.id,
                "unit": localStorageEngine.getLocalVar('__CDFILIAL'),
                "serviceId": oralPrescription.serviceId ? oralPrescription.serviceId : oralPrescription.cdservfili,
                "key": Date.now(),
                "unique": Validation.strPad(unique, 4),
                "plateName": normalFeeding.plateName,
                "plateType": normalFeeding.idopccnfcard === 'S' ?
                    (normalFeeding.cdtipoprat ? normalFeeding.cdtipoprat + ' (Opção)' : normalFeeding.plateType + ' (Opção)') : (normalFeeding.cdtipoprat ? normalFeeding.cdtipoprat : normalFeeding.plateType),
                "plateTypeName": plateTypeName,
                "serviceDate": serviceDate.isValid() ? oralPrescription.serviceDate : moment(oralPrescription.serviceDate, 'YYYY-MM-DD').format('DD/MM/YYYY'),
                "service": oralPrescription.idtpservhosp,
                "plateNumbers": "1",
                "plateAuxId": 'P',
                "formulationId": normalFeeding.codplate && !normalFeeding.formulationId ? normalFeeding.codplate : normalFeeding.formulationId,
                "observation": normalFeeding.observation,
                "dietId": oralPrescription.dietId,
                "changed": true,
                "__is_new": true,
                "selectedByUser": normalFeeding.selectedByUser ? normalFeeding.selectedByUser : false,
                "isRequiredOnMenu": normalFeeding.isRequiredOnMenu,
                "cdservfili": oralPrescription.cdservfili,
                "codplate": normalFeeding.codplate,
                "sequential": oralPrescription.sequential,
                "DTINCLUSAO": oralPrescription.DTINCLUSAO
            };
            objects.push(object);
            widgetDetailsPatient.localDataSourceLoaded.NormalFeedings.unshift(object);

        });
        return objects;
    }

    function saveLactaryPrescription(widget) {

        var currentRow = widget.currentRow;
        var widgetPatient = widget.container.getWidget('WidgetDetailsPatient');
        if (widget.isValid()) {
            var promise = ZHPromise.defer();
            currentRow.serviceDateDefault = currentRow.serviceDate;
            currentRow.serviceDate = moment(currentRow.serviceDate).format('DD/MM/YYYY');

            setLactaryPrescriptionDefaultValues(currentRow);
            var rowToSave = createRowToSavePrescription(widget);
            SynchronizeService.persistOnline('/ControlFeedings', '/ControlFeedings/save', 'DataSet', [rowToSave]).then(function(callback) {
                rowToSave.changed = false;
                rowToSave.__is_new = false;
                if (callback && callback.dataset && callback.dataset.error && callback.dataset.error.SEQUENTIAL) {
                    rowToSave.sequential = callback.dataset.error.SEQUENTIAL;
                }
                MapNutritionistService.savePrescription(rowToSave);
                promise.resolve(rowToSave.sequential);
            }, function() {
                MapNutritionistService.savePrescription(rowToSave);
                promise.resolve(rowToSave.sequential);
            });

            promise.then(function(sequential) {
                currentRow.sequential = sequential;
                widgetPatient.localDataSourceLoaded.ControlFeedings.unshift(currentRow);
                widgetPatient.curPatient.ControlFeedings.unshift(currentRow);
            });
            ScreenService.closePopup();
            saveProfile(widgetPatient);

        }
    }

    function saveLactaryFeeding(widget) {
        var widgetPopupPrescricao = widget.container.getWidget('widgetPopupPrescricao');
        var curLactaryFeeding = widgetPopupPrescricao.currentRow.curLactaryFeeding;
        curLactaryFeeding.DTINCLUSAO = widgetPopupPrescricao.currentRow.DTINCLUSAO;
        if (widget.isValid()) {
            if (!Boolean(curLactaryFeeding.key)) {
                saveNewLactaryFeeding(widget);
            } else {
                saveLactaryFeedingOnPrescriptionEdit(widget);
                var index = widgetPopupPrescricao.currentRow.lactaryFeedings.findIndex(function(feeding) {
                    return feeding.key === widget.currentRow.key;
                });
                var lactaryDate = moment(widget.currentRow.lactaryDate, "YYYY-MM-DD").format("DD/MM/YYYY");
                var hourLactaryPlate = moment(widget.currentRow.hourLactaryPlate, 'HHmm').format('HH:mm');
                widget.currentRow.lactaryDate = lactaryDate;
                widget.currentRow.hourLactaryPlate = hourLactaryPlate;
                widgetPopupPrescricao.currentRow.lactaryFeedings[index] = widget.currentRow;
            }
        }
    }

    function saveNewLactaryFeeding(widget) {

        var currentRow = widget.container.getWidget('widgetPopupPrescricao').currentRow;
        var curLactaryFeeding = currentRow.curLactaryFeeding;
        var isAutomatic = widget.getField('automatic').value();
        var lactaryFeeding;
        var rowsToSave = [];
        var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');
        lactaryFeeding = createRowToRelateLactaryFeedingToLactaryPrescription(widget);
        rowsToSave.push(lactaryFeeding);
        widgetDetailsPatient.localDataSourceLoaded.LactaryFeedings.unshift(lactaryFeeding);
        widgetDetailsPatient.curPatient.LactaryFeedings.unshift(lactaryFeeding);
        setNewRecordOnReferenceArray(lactaryFeeding, currentRow.lactaryFeedings);
        if (isAutomatic === 'S') {
            var arrayDateLactary = saveAutoFills(curLactaryFeeding, 'lactaryDate', 'hourLactaryPlate');
            arrayDateLactary.forEach(function(data) {
                var obj = moment(data, "YYYY-MM-DD HHmm");
                var dt = obj.format("YYYY-MM-DD");
                var hr = obj.format("HHmm");
                lactaryFeeding = createRowToRelateLactaryFeedingToLactaryPrescription(widget, dt, hr);
                rowsToSave.push(lactaryFeeding);
                widgetDetailsPatient.localDataSourceLoaded.LactaryFeedings.unshift(lactaryFeeding);
                widgetDetailsPatient.curPatient.LactaryFeedings.unshift(lactaryFeeding);
                setNewRecordOnReferenceArray(lactaryFeeding, currentRow.lactaryFeedings);
            });
        }
        SynchronizeService.persistOnline('/LactaryFeedings', '/LactaryFeedings/save', 'DataSet', rowsToSave).then(function(response) {
            var findParams = {
                'CDFILIAL': widgetDetailsPatient.curPatient.unit,
                'type': 'lactaryFeedings'
            };
            SynchronizeService.persistOnline('/LactaryFeedings', '/Structures/find', 'DataSet', [findParams]).then(function(callback) {
                // callback.dataset.Plates.normalFeedings = Array.isArray(callback.dataset.Plates.normalFeedings) ? callback.dataset.Plates.normalFeedings : [];
                callback.dataset.Plates.lactaryFeedings = Array.isArray(callback.dataset.Plates.lactaryFeedings) ? callback.dataset.Plates.lactaryFeedings : [];
                // localStorageEngine.setDataSource('/NormalFeedings', callback.dataset.Plates.normalFeedings);
                localStorageEngine.setDataSource('/LactaryFeedings', callback.dataset.Plates.lactaryFeedings);
                widgetDetailsPatient.localDataSourceLoaded.LactaryFeedings = angular.copy(callback.dataset.Plates.lactaryFeedings);
                widgetDetailsPatient.curPatient.LactaryFeedings = callback.dataset.Plates.lactaryFeedings.filter(function(lactaryFeeding) {
                    return lactaryFeeding.id === widgetDetailsPatient.curPatient.id;
                });
                var datePrescription = currentRow.serviceDate;
                var dateFormat = datePrescription.indexOf('/') > -1 ? datePrescription : moment(datePrescription).format('DD/MM/YYYY');
                currentRow.lactaryFeedings = widgetDetailsPatient.curPatient.LactaryFeedings.filter(function(lactaryFeeding) {
                    return lactaryFeeding.sequential === currentRow.sequential && lactaryFeeding.serviceDate === dateFormat;
                });
            }, function() {
                if (response && response.dataset && response.dataset.error && Array.isArray(response.dataset.error.SEQUENTIAL)) {
                    rowsToSave.forEach(function(row, index) {
                        row.unique = response.dataset.error.SEQUENTIAL[index];
                    });
                }
                MapNutritionistService.saveLactaryFeeding(rowsToSave, false);
            });

        }, function() {
            MapNutritionistService.saveLactaryFeeding(rowsToSave);
        });

        curLactaryFeeding.key = lactaryFeeding.key;
        ScreenService.closePopup();
        widget.container.getWidget('widgetPopupPrescricao').view();
        deleteLactaryFeedingBackupParams(widget);
        saveProfile(widgetDetailsPatient);

    }

    function saveLactaryFeedingOnPrescriptionEdit(widget) {

        var curLactaryFeeding = widget.container.getWidget('widgetPopupPrescricao').currentRow.curLactaryFeeding;
        var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');
        var filter = [{
            "name": "key",
            "operator": "=",
            "value": curLactaryFeeding.key
        }];

        localStorageEngine.getDataSource("/LactaryFeedings", filter).then(function(filteredLactaryFeeding) {
            filteredLactaryFeeding[0].changed = true;
            filteredLactaryFeeding[0].observation = curLactaryFeeding.observation;
            filteredLactaryFeeding[0].plateAuxId = 'P';
            filteredLactaryFeeding[0].formulationId = curLactaryFeeding.formulationId;
            filteredLactaryFeeding[0].plateNumbers = curLactaryFeeding.plateNumbers;
            filteredLactaryFeeding[0].hourLactaryPlate = moment(curLactaryFeeding.hourLactaryPlate, "HHmm").format("HH:mm");
            filteredLactaryFeeding[0].lactaryDate = moment(curLactaryFeeding.lactaryDate, "DD/MM/YYYY").format("DD/MM/YYYY");
            filteredLactaryFeeding[0].DTINCLUSAO = curLactaryFeeding.DTINCLUSAO;
            SynchronizeService.persistOnline('/LactaryFeedings', '/LactaryFeedings/save', 'DataSet', filteredLactaryFeeding).then(function() {
                var findParams = {
                    'CDFILIAL': widgetDetailsPatient.curPatient.unit,
                    'type': 'lactaryFeedings'
                };
                SynchronizeService.persistOnline('/LactaryFeedings', '/Structures/find', 'DataSet', [findParams]).then(function(callback) {
                    // callback.dataset.Plates.normalFeedings = Array.isArray(callback.dataset.Plates.normalFeedings) ? callback.dataset.Plates.normalFeedings : [];
                    callback.dataset.Plates.lactaryFeedings = Array.isArray(callback.dataset.Plates.lactaryFeedings) ? callback.dataset.Plates.lactaryFeedings : [];
                    // localStorageEngine.setDataSource('/NormalFeedings', callback.dataset.Plates.normalFeedings);
                    localStorageEngine.setDataSource('/LactaryFeedings', callback.dataset.Plates.lactaryFeedings);
                }, function() {
                    filteredLactaryFeeding[0].changed = false;
                    filteredLactaryFeeding[0].__is_new = false;
                    localStorageEngine.save("/LactaryFeedings", filteredLactaryFeeding);
                });

            }, function() {
                localStorageEngine.save("/LactaryFeedings", filteredLactaryFeeding);
            });

            ScreenService.closePopup();
            widget.container.getWidget('widgetPopupPrescricao').view();
            deleteLactaryFeedingBackupParams(widget);
            saveProfile(widgetDetailsPatient);
        });
    }

    function setLactaryPrescriptionDefaultValues(currentRow) {

        if (!Boolean(currentRow.observacao)) {
            currentRow.observacao = "";
        }

        currentRow.unit = getLocalVar("__CDFILIAL");
        currentRow.dietName = currentRow.dietName ? currentRow.dietName : "";

    }

    function createRowToRelateLactaryFeedingToLactaryPrescription(widget, dateLactary, hourLactary) {

        var clonedLactaryPrescription = angular.copy(widget.container.getWidget('widgetPopupPrescricao').currentRow);
        var curLactaryFeeding = widget.container.getWidget('widgetPopupItemLac').currentRow;

        var lactaryFeeding = {
            "id": clonedLactaryPrescription.id,
            "serviceDate": moment(clonedLactaryPrescription.serviceDate, "YYYY-MM-DD").format("DD/MM/YYYY"),
            "unit": localStorageEngine.getLocalVar('__CDFILIAL'),
            "sequential": clonedLactaryPrescription.sequential,
            "key": Date.now(),
            "serviceId": clonedLactaryPrescription.serviceId,
            "unique": clonedLactaryPrescription.cdservfili,
            "formulationId": curLactaryFeeding.cdformulacao,
            "formulationName": curLactaryFeeding.formulationName,
            "observation": curLactaryFeeding.observation,
            "idtpservhosp": clonedLactaryPrescription.idtpservhosp,
            "plateNumbers": curLactaryFeeding.plateNumbers,
            "hourLactaryPlate": hourLactary ? moment(hourLactary, "HHmm").format("HH:mm") : moment(curLactaryFeeding.hourLactaryPlate, "HHmm").format("HH:mm"),
            "lactaryDate": dateLactary ? moment(dateLactary, "YYYY-MM-DD").format("DD/MM/YYYY") : moment(curLactaryFeeding.lactaryDate, "YYYY-MM-DD").format("DD/MM/YYYY"),
            "plateAuxId": 'P',
            "__is_new": true,
            "changed": true,
            "__isSelected": true,
            "isSelected": true,
            "DTINCLUSAO": clonedLactaryPrescription.DTINCLUSAO
        };

        return lactaryFeeding;

    }

    function saveEnteralFeeding(widget) {

        var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');
        if (widget.isValid()) {
            var currentRow = widget.currentRow;
            var widgetPopupPrescricao = widget.container.getWidget('widgetPopupPrescricao');
            var isAutomatic = widget.getField('automatic').value();
            if (!Boolean(currentRow.key)) {
                var rowsToSave = createRowToRelateEnteralFeedingToEnteralPrescription(widget);
                var allRowsToSave = [];
                allRowsToSave.push(rowsToSave);
                widgetDetailsPatient.localDataSourceLoaded.EnteralFeedings.unshift(rowsToSave);
                widgetDetailsPatient.curPatient.EnteralFeedings.unshift(rowsToSave);
                var backupCurrentRow = angular.copy(widget.currentRow);
                var serviceDate = moment(widgetPopupPrescricao.currentRow.serviceDate, "YYYY-MM-DD").format("DD/MM/YYYY");
                var bkpDate = widgetPopupPrescricao.currentRow.serviceDate;
                widgetPopupPrescricao.currentRow.serviceDate = serviceDate;
                widgetPopupPrescricao.getField('serviceDate').value(bkpDate);
                var time = moment(widget.currentRow.time, "HHmm").format("HH:mm");
                widget.currentRow.enteralDate = moment(widget.currentRow.enteralDate, "YYYY-MM-DD").format("DD/MM/YYYY");
                widget.currentRow.time = time;
                widgetPopupPrescricao.currentRow.feedings.unshift(rowsToSave);
                if (isAutomatic === 'S') {
                    var arrayDateEnteral = saveAutoFills(backupCurrentRow, 'enteralDate', 'time');
                    arrayDateEnteral.forEach(function(data) {
                        var obj = moment(data, "YYYY-MM-DD HHmm");
                        var dt = obj.format("YYYY-MM-DD");
                        var hr = obj.format("HHmm");
                        var rowsToSave = createRowToRelateEnteralFeedingToEnteralPrescription(widget, dt, hr);
                        allRowsToSave.push(rowsToSave);
                        widgetDetailsPatient.localDataSourceLoaded.EnteralFeedings.unshift(rowsToSave);
                        widgetDetailsPatient.curPatient.EnteralFeedings.unshift(rowsToSave);
                        var serviceDate = moment(widgetPopupPrescricao.currentRow.serviceDate, "YYYY-MM-DD").format("DD/MM/YYYY");
                        var bkpDate = widgetPopupPrescricao.currentRow.serviceDate;
                        widgetPopupPrescricao.currentRow.serviceDate = serviceDate;
                        widgetPopupPrescricao.getField('serviceDate').value(bkpDate);
                        var time = moment(widget.currentRow.time, "HHmm").format("HH:mm");
                        widget.currentRow.enteralDate = moment(widget.currentRow.enteralDate, "YYYY-MM-DD").format("DD/MM/YYYY");
                        widget.currentRow.time = time;
                        widgetPopupPrescricao.currentRow.feedings.unshift(rowsToSave);
                    });
                }
                SynchronizeService.persistOnline('/EnteralFeedings', '/EnteralFeedings/save', 'DataSet', allRowsToSave).then(function(callback) {
                    var findParams = {
                        'CDFILIAL': widgetDetailsPatient.curPatient.unit,
                        'type': 'enteralFeedings'
                    };
                    SynchronizeService.persistOnline('/EnteralFeedings', '/Structures/find', 'DataSet', [findParams]).then(function(response) {
                        response.dataset.Plates.enteralFeedings = Array.isArray(response.dataset.Plates.enteralFeedings) ? response.dataset.Plates.enteralFeedings : [];
                        localStorageEngine.setDataSource('/EnteralFeedings', response.dataset.Plates.enteralFeedings);
                        widgetDetailsPatient.localDataSourceLoaded.EnteralFeedings = angular.copy(response.dataset.Plates.enteralFeedings);
                        widgetDetailsPatient.curPatient.EnteralFeedings = response.dataset.Plates.enteralFeedings.filter(function(enteralFeeding) {
                            return enteralFeeding.id === widgetDetailsPatient.curPatient.id;
                        });
                        var datePrescription = widgetPopupPrescricao.currentRow.serviceDate;
                        var dateFormat = datePrescription.indexOf('/') > -1 ? datePrescription : moment(datePrescription).format('DD/MM/YYYY');
                        widgetPopupPrescricao.currentRow.feedings = widgetDetailsPatient.curPatient.EnteralFeedings.filter(function(enteralFeeding) {
                            return enteralFeeding.sequential === widgetPopupPrescricao.currentRow.sequential && enteralFeeding.serviceDate === dateFormat;
                        });
                    }, function() {
                        if (callback && callback.dataset && callback.dataset.error && Array.isArray(callback.dataset.error.SEQUENTIAL)) {
                            allRowsToSave.forEach(function(row, index) {
                                row.unique = callback.dataset.error.SEQUENTIAL[index];
                            });
                        }
                        MapNutritionistService.saveEnteralFeeding(allRowsToSave, false);
                    });
                }, function() {
                    MapNutritionistService.saveEnteralFeeding(allRowsToSave);
                });

            } else {
                saveEnteralFeedingEdit(widget);
                var index = widgetPopupPrescricao.currentRow.feedings.findIndex(function(feeding) {
                    return feeding.key === widget.currentRow.key;
                });
                if (widget.currentRow.preparationId === "A") {
                    widget.currentRow.preparationName = "Aberto";
                } else {
                    widget.currentRow.preparationName = "Fechado";
                }
                var enteralDate = moment(widget.currentRow.enteralDate, "YYYY-MM-DD").format("DD/MM/YYYY");
                widget.currentRow.enteralDate = enteralDate;
                widgetPopupPrescricao.currentRow.feedings[index] = widget.currentRow;
            }

            ScreenService.closePopup();
            saveProfile(widgetDetailsPatient);

        }

    }

    function saveEnteralFeedingEdit(widget) {
        var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');
        var widgetPopupPrescricao = widget.container.getWidget('widgetPopupPrescricao');
        if (widget.isValid()) {

            var subItemData = widget.container.getWidget('widgetPopupItemEnt').currentRow;
            var filter = [{
                name: "key",
                operator: "=",
                value: widget.currentRow.key
            }];
            var bkpIsNew;
            localStorageEngine.getDataSource("/EnteralFeedings", filter).then(function(filteredEnteralFeeding) {
                bkpIsNew = filteredEnteralFeeding[0].__is_new;
                filteredEnteralFeeding[0].changed = true;
                filteredEnteralFeeding[0].__is_new = false;
                filteredEnteralFeeding[0].plateAuxId = "P";
                filteredEnteralFeeding[0].formulationId = "";
                filteredEnteralFeeding[0].observation = subItemData.observation;
                filteredEnteralFeeding[0].serviceNumbers = subItemData.serviceNumbers;
                filteredEnteralFeeding[0].qtinfuconf = subItemData.volInfudido;
                filteredEnteralFeeding[0].plateNumbers = subItemData.volPrescription;
                filteredEnteralFeeding[0].time = moment(subItemData.time, 'HHmm').format('HH:mm');
                filteredEnteralFeeding[0].enteralDate = moment(subItemData.enteralDate).format("DD/MM/YYYY") === "Invalid date" ? subItemData.enteralDate : moment(subItemData.enteralDate).format("DD/MM/YYYY");
                filteredEnteralFeeding[0].productId = "";
                filteredEnteralFeeding[0].accessRoadId = subItemData.accessRoadId;
                filteredEnteralFeeding[0].preparationId = subItemData.preparationId;
                filteredEnteralFeeding[0].serviceDate = moment(filteredEnteralFeeding[0].serviceDate, "DD/MM/YYYY").format("DD/MM/YYYY");

                setFomulationByEnteralFeedingPreparation(subItemData.preparationId, filteredEnteralFeeding[0], subItemData.formulationId);
                SynchronizeService.persistOnline('/EnteralFeedings', '/EnteralFeedings/save', 'DataSet', filteredEnteralFeeding).then(function() {
                    var findParams = {
                        'CDFILIAL': widgetDetailsPatient.curPatient.unit,
                        'type': 'enteralFeedings'
                    };
                    SynchronizeService.persistOnline('/EnteralFeedings', '/Structures/find', 'DataSet', [findParams]).then(function(callback) {
                        callback.dataset.Plates.enteralFeedings = Array.isArray(callback.dataset.Plates.enteralFeedings) ? callback.dataset.Plates.enteralFeedings : [];
                        localStorageEngine.setDataSource('/EnteralFeedings', callback.dataset.Plates.enteralFeedings);
                        widgetDetailsPatient.localDataSourceLoaded.EnteralFeedings = angular.copy(callback.dataset.Plates.enteralFeedings);
                        widgetDetailsPatient.curPatient.EnteralFeedings = callback.dataset.Plates.enteralFeedings.filter(function(enteralFeeding) {
                            return enteralFeeding.id === widgetDetailsPatient.curPatient.id;
                        });
                        var datePrescription = widgetPopupPrescricao.currentRow.serviceDate;
                        var dateFormat = datePrescription.indexOf('/') > -1 ? datePrescription : moment(datePrescription).format('DD/MM/YYYY');
                        widgetPopupPrescricao.currentRow.feedings = widgetDetailsPatient.curPatient.EnteralFeedings.filter(function(enteralFeeding) {
                            return enteralFeeding.sequential === widgetPopupPrescricao.currentRow.sequential && enteralFeeding.serviceDate === dateFormat;
                        });
                    }, function() {
                        filteredEnteralFeeding[0].changed = false;
                        localStorageEngine.save("/EnteralFeedings", filteredEnteralFeeding);
                    });
                }, function() {
                    filteredEnteralFeeding[0].__is_new = bkpIsNew;
                    localStorageEngine.save("/EnteralFeedings", filteredEnteralFeeding);
                });

            });

        }

    }

    function createRowToRelateEnteralFeedingToEnteralPrescription(widget, dateEnteral, timeEnteral) {

        var enteralPrescription = widget.container.getWidget('widgetPopupPrescricao').currentRow;
        var subitemData = widget.currentRow;
        var serviceDate = moment(enteralPrescription.serviceDate, "YYYY-MM-DD").format("DD/MM/YYYY");
        var enteralFeeding = {
            "__is_new": true,
            "accessRoadId": subitemData.accessRoadId,
            "accessRoadName": subitemData.accessRoadName,
            "changed": true,
            "sequential": enteralPrescription.sequential,
            "key": Date.now(),
            "enteralDate": dateEnteral ? moment(dateEnteral, "YYYY-MM-DD").format("DD/MM/YYYY") : moment(subitemData.enteralDate).format("DD/MM/YYYY"),
            "formulationId": "",
            "formulationName": widget.currentRow.formulationName,
            "id": enteralPrescription.id,
            "idtpservhosp": enteralPrescription.idtpservhosp,
            "observation": subitemData.observation,
            "plateAuxId": 'P',
            "plateNumbers": subitemData.volPrescription,
            "volPrescription": subitemData.volPrescription,
            "preparationId": subitemData.preparationId,
            "productId": "",
            "qtinfuconf": subitemData.volInfudido,
            "volInfudido": subitemData.volInfudido,
            "serviceDate": serviceDate,
            "serviceNumbers": subitemData.serviceNumbers,
            "time": timeEnteral ? moment(timeEnteral, "HHmm").format("HH:mm") : moment(subitemData.time, "HHmm").format("HH:mm"),
            "unit": localStorageEngine.getLocalVar('__CDFILIAL'),
            "DTINCLUSAO": enteralPrescription.DTINCLUSAO
        };

        setFomulationByEnteralFeedingPreparation(subitemData.preparationId, enteralFeeding, subitemData.formulationId);

        return enteralFeeding;

    }

    function setFomulationByEnteralFeedingPreparation(preparationId, enteralFeeding, formulatioId) {

        if (preparationId === "A") {
            enteralFeeding.preparationName = "Aberto";
            enteralFeeding.formulationId = formulatioId;
        } else if (preparationId === "F") {
            enteralFeeding.preparationName = "Fechado";
            enteralFeeding.productId = formulatioId;
        }

    }

    function saveParenteralPrescription(widget) {

        var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');
        if (widget.isValid()) {
            var rowToSave = createRowToSavePrescription(widget);
            SynchronizeService.persistOnline('/ControlFeedings', '/ControlFeedings/save', 'DataSet', [rowToSave]).then(function(callback) {
                rowToSave.changed = false;
                rowToSave.__is_new = false;
                if (callback && callback.dataset && callback.dataset.error && callback.dataset.error.SEQUENTIAL) {
                    rowToSave.sequential = callback.dataset.error.SEQUENTIAL;
                }
                MapNutritionistService.savePrescription(rowToSave);
            }, function() {
                MapNutritionistService.savePrescription(rowToSave);
            });

            ScreenService.closePopup();
            widgetDetailsPatient.list();
            saveProfile(widgetDetailsPatient);
            return rowToSave;
        }
    }

    this.reloadScreenAndClearFilters = function(widget) {
        eventAggregator.publish("reloadScreenAndClearFilters", widget);
    };
    this.closeAntroAval = function(widget) {
        ScreenService.closePopup();
        widget.view();
        widget.resultadoAuth = '';
    };
    this.closeAndList = function(widget) {
        ScreenService.closePopup();
        widget.subitemtoadd = '';
        widget.infos = '';
        widget.list();
        widget.activate();
    };

    this.closePacientDetailsWidget = function(widget) {
        widget.isVisible = false;
        widget.container.showMenu = true;
        var widgetMain = widget.container.getWidget('WidgetMain');
        widgetMain.activate();
    };
    eventAggregator.subscribe('openBioEvalPopup', function(widget) {
        openBioEvalPopup(widget);
    });

    function openBioEvalPopup(widget) {
        var bioEvalPopup = widget.container.getWidget('bioEvalPopup');
        bioEvalPopup.newRow();
        bioEvalPopup.edit();
        bioEvalPopup.showAddEditBioExam = true;
        bioEvalPopup.isSubItemBeingAdded = true;
        bioEvalPopup.currentRow.__is_new = true;
        bioEvalPopup.currentRow.examinationDate = moment().format("YYYY-MM-DDTHH:mm:ss");
        if (widget.curPatient.BioEvaluations) {
            bioEvalPopup.currentRow.priority = widget.curPatient.BioEvaluations.length + 1;
        } else {
            bioEvalPopup.currentRow.priority = 1;
        }
        bioEvalPopup.getField('examinationDate').readOnly = false;
        bioEvalPopup.label = ScreenService.i18n('Nova Avaliação Bioquímica');
        reloadFields(bioEvalPopup);
        ScreenService.openPopup(bioEvalPopup);
    }
    this.closeAntroAval = function(widget) {
        ScreenService.closePopup();
        widget.view();
        widget.resultadoAuth = '';
    };

    this.addNewSubItem = function(widget) { //Sub-item
        if (widget.currentScreen === "nutriApointment") {

            widget.resultadoAuth = [];
            if (widget.subitemtoadd === 'avaliationbioq') {
                openBioEvalPopup(widget);
            }
            if (widget.subitemtoadd === 'avaliationantr') {
                var antrAvaliationPopup = widget.container.getWidget('antrAvaliationPopup');
                delete antrAvaliationPopup.fields;
                antrAvaliationPopup.fields = [];
                var fdSelected = widget.selectedRows;

                if (fdSelected && fdSelected.length > 0) {
                    var neededFields = [];
                    var methodsSelects = ['36', '40', '41', '42', '43', '51'];
                    var possibleFields = widget.localDataSourceLoaded.DataTypeEvaluations;
                    var fieldsUsedOnMethods = widget.localDataSourceLoaded.DataEvaluations;

                    fdSelected.forEach(function(fd) {
                        var mtID = fd.method;
                        fieldsUsedOnMethods.forEach(function(fum) {
                            if (fum.methodEvaluationId === mtID) {
                                neededFields.push(fum.dataEvaluationId);
                            }
                        });
                    });
                    var fieldsNeeded = [];
                    possibleFields.forEach(function(pf) {
                        neededFields.forEach(function(nf) {
                            if (nf === pf.dataTypeEvaluationId) {
                                if (fieldsNeeded.indexOf(pf) === -1) {
                                    fieldsNeeded.push(pf);
                                }
                            }
                        });
                    });

                    antrAvaliationPopup.infos = widget.infos;

                    var field = {
                        "name": "",
                        "label": "",
                        "template": "field/number.html",
                        "validations": {
                            "required": true
                        },
                        "isVisible": true,
                        "class": 6
                    };

                    fieldsNeeded.forEach(function(newField) {
                        if (methodsSelects.indexOf(newField.dataTypeEvaluationId) === -1) {
                            field.name = newField.dataTypeEvaluationId;
                            field.label = newField.dataTypeEvaluationName;
                            field.valueField = field.name;
                            var aux = metaDataFactory.fieldFactory(field, antrAvaliationPopup);
                            setFieldValue(antrAvaliationPopup, newField.dataTypeEvaluationId, aux, methodsSelects);
                            antrAvaliationPopup.fields.push(aux);
                            if (field.name === '01') {
                                antrAvaliationPopup.getField('01').setValue(getLocalVar('__appointmentdata').weight); //weight
                            } else if (field.name === '02') {
                                antrAvaliationPopup.getField('02').setValue(getLocalVar('__appointmentdata').height); //height
                            }
                        } else {
                            makeSelectField(newField, antrAvaliationPopup, methodsSelects);
                        }
                    });
                    widget.fieldsNeeded = fieldsNeeded;

                    AppointmentService.saveCheckedRows(fdSelected);
                    antrAvaliationPopup.edit();
                    antrAvaliationPopup.activate();
                    ScreenService.openPopup(antrAvaliationPopup);
                } else {
                    ScreenService.showMessage(ScreenService.i18n("Selecione algum item"));
                    widget.showAddEditAntroExam = false;
                    widget.view();
                }
            }
            if (widget.subitemtoadd === 'enteralProd') {
                widget.isSubeditting = true;
                widget.edit();
            }

        } else if (widget.currentRow.idtpservhosp === "2") {
            addLactaryFeedings(widget);
        } else if (widget.currentRow.idtpservhosp === "1") {
            addEnteralFeedings(widget);
        }
    };

    function makeSelectField(newField, widget, methodsSelects) {
        var fieldSelect = {
            "name": newField.dataTypeEvaluationId,
            "label": newField.dataTypeEvaluationName,
            "template": "tecdiet#field/select-native.html",
            "isVisible": true,
            "class": 6,
            "valueField": "value",
            "customStyle": "margin-bottom: 15px;",
            "descriptionField": "description",
            "dataSource": {
                "data": [{
                        "value": "4",
                        "description": "Seis Meses"
                    },
                    {
                        "value": "3",
                        "description": "Três Meses"
                    },
                    {
                        "value": "2",
                        "description": "Um Mês"
                    },
                    {
                        "value": "1",
                        "description": "Uma Semana"
                    }
                ]
            },
            "validations": {
                "required": true
            },
            "hideEmptyValue": true,
        };
        if (newField.dataTypeEvaluationId > '39' && newField.dataTypeEvaluationId < '44') {
            fieldSelect.dataSource.data = [{
                    "value": "1",
                    "description": "Sim"
                },
                {
                    "value": "0",
                    "description": "Não"
                }
            ];
        }
        if (newField.dataTypeEvaluationId === '51') {
            fieldSelect.dataSource.data = [{
                    "value": "4",
                    "description": "Sedentarismo (> ou = 1 e < 1,4)"
                },
                {
                    "value": "3",
                    "description": "Leve (> ou = 1,4 e < 1,6)"
                },
                {
                    "value": "2",
                    "description": "Moderado (> ou = 1,6 e < 1,9)"
                },
                {
                    "value": "1",
                    "description": "Intenso (> ou = 1,9 e < 2,5)"
                }
            ];
        }
        var aux = metaDataFactory.fieldFactory(fieldSelect, widget);
        setFieldValue(widget, newField.dataTypeEvaluationId, aux, methodsSelects);
        widget.fields.push(aux);
    }

    function configurationWidget(widget, mode) {
        Object.keys(widget[mode]).forEach(function(currentProperty) {
            var property = widget[mode][currentProperty];
            if (Array.isArray(property.fields)) {
                property.fields.forEach(function(nameField) {
                    widget.getField(nameField)[property.name] = property.value;
                });
            } else {
                Object.keys(property.fields).forEach(function(nameField) {
                    widget.getField(nameField)[property.name] = property.fields[nameField];
                });
            }
            if (typeof property.widget === "object") {
                if (property.widget.condition) {
                    if (property.widget.condition.currentRow) {
                        widget[property.name] = property.widget.condition.values[widget.currentRow[property.widget.condition.propertyWidget]];
                    } else {
                        widget[property.name] = property.widget.condition.values[widget[property.widget.condition.propertyWidget]];
                    }
                }
            }
        });
    }

    function setFieldValue(antrAvaliationPopup, id, field, methodsSelects) {
        var widget = antrAvaliationPopup.container.getWidget('WidgetDetailsPatient');
        if (widget.infos) {
            widget.infos.forEach(function(info) {
                if (info.dataTypeEvaluationId === id) {
                    if (methodsSelects.indexOf(info.dataTypeEvaluationId) !== -1) {
                        field.setValue(info.value);
                    } else {
                        field.setValue(parseFloat(info.value));
                    }
                }
            });
        }
    }

    this.changeAccessRoad = function(field) {
        field.widget.currentRow.accessRoadId = field.value();
        var accessRoad = field.dataSource.data.filter(function(access) {
            return access.accessRoadId === field.value();
        })[0];
        field.value(accessRoad.accessRoadId);
        field.widget.currentRow.accessRoadName = accessRoad.accessRoadName;
    };

    this.changeFormulation = function(field) {
        var selectedFormulation = field.dataSource.data.filter(function(formulation) {
            return formulation.formulationId === field.value();
        })[0];
        field.widget.currentRow.formulationName = selectedFormulation.formulationName;
    };

    function addEnteralFeedings(widget) {
        var widgetPopupItemEnt = widget.container.getWidget('widgetPopupItemEnt');
        widgetPopupItemEnt.newRow();
        widgetPopupItemEnt.edit();
        widgetPopupItemEnt.getField('accessRoadId').reload();
        widgetPopupItemEnt.getField('preparationId').reload();
        widgetPopupItemEnt.getField('formulationId').reload();
        widgetPopupItemEnt.getField('formulationId').disabled = true;
        widgetPopupItemEnt.getField('formulationId').readOnly = true;
        widgetPopupItemEnt.getField('automatic').isVisible = true;
        widgetPopupItemEnt.getField('period').isVisible = true;
        widgetPopupItemEnt.getField('interval').isVisible = true;
        widgetPopupItemEnt.getField('period').readOnly = true;
        widgetPopupItemEnt.getField('interval').readOnly = true;
        delete widgetPopupItemEnt.getField('interval').validations;
        delete widgetPopupItemEnt.getField('period').validations;
        configurationWidget(widgetPopupItemEnt, "modeInsert");
        ScreenService.openPopup(widgetPopupItemEnt);
    }

    eventAggregator.subscribe("disablePregnancyWeeks", function(widget) {
        disablePregnancyWeeks(widget);
    });

    function disablePregnancyWeeks(widget) {
        var status = widget.currentRow.pregnancyId;
        var pregnancyWeek = widget.getField('pregnancyWeek');
        if (status === 'S') {
            pregnancyWeek.readOnly = false;
            pregnancyWeek.validations = {
                "range": {
                    "max": 45,
                    "min": 0
                }
            };
        } else {
            pregnancyWeek.readOnly = true;
            pregnancyWeek.validations = {};
            widget.currentRow.pregnancyWeek = "";
        }
        pregnancyWeek.reload();
    }
    this.disablePregnancyWeeks = function(widget) {
        disablePregnancyWeeks(widget);
    };
    this.changeExam = function(widget) {
        eventAggregator.publish('changeSelectedExam', widget);
    };

    function addLactaryFeedings(widget) {

        widget.edit();
        setLactaryFeedingDefaultState(widget);

    }

    function setLactaryFeedingDefaultState(widget) {

        widget.currentRow.curLactaryFeeding = {};
        var widgetPopupItemLac = widget.container.getWidget('widgetPopupItemLac');
        widgetPopupItemLac.newRow();
        widgetPopupItemLac.getField('formulationId').reload();
        widgetPopupItemLac.getField('automatic').isVisible = true;
        widgetPopupItemLac.getField('period').isVisible = true;
        widgetPopupItemLac.getField('interval').isVisible = true;
        widgetPopupItemLac.getField('period').readOnly = true;
        widgetPopupItemLac.getField('interval').readOnly = true;
        delete widgetPopupItemLac.getField('interval').validations;
        delete widgetPopupItemLac.getField('period').validations;
        configurationWidget(widgetPopupItemLac, "modeInsert");
        ScreenService.openPopup(widgetPopupItemLac);
    }

    function removeDeadPatients(patientList) {
        var deadPatients = [];
        if (patientList && patientList.length > 0) {
            patientList.forEach(function(ptitem) {
                if (ptitem.isDead) {
                    deadPatients.push(patientList.indexOf(ptitem));
                }
            });
        }
        if (deadPatients && deadPatients.length > 0) {
            deadPatients.forEach(function(deadMan) {
                patientList.splice(deadMan, 1);
            });
        }
    }
    this.changePatient = function(widget, direction) {
        var widgetDetailsPatient = widget.container.getWidget('WidgetDetailsPatient');
        var filterPatient = [{
            "name": "unit",
            "operator": "=",
            "value": localStorageEngine.getLocalVar('__CDFILIAL')
        }, {
            "name": [{
                "name": "formattedName",
                "direction": "ASC"
            }],
            "operator": "ORDER_BY",
            "value": ""
        }];

        localStorageEngine.getDataSource('/Patients', filterPatient).then(function(patients) {
            var paciente = widgetDetailsPatient.curPatient;
            var curPatient;
            patients.forEach(function(a) {
                if (a.id === paciente.id) {
                    curPatient = a;
                }
            });
            var curPosition = patients.indexOf(curPatient);
            var patientsLength = patients.length;
            var nextPosition = curPosition + direction;
            removeDeadPatients(patients);

            if (patients[nextPosition] && nextPosition >= 0 && nextPosition <= patientsLength) {
                widgetDetailsPatient.curPatient = patients[nextPosition];
            } else if (nextPosition < 0) {
                nextPosition = patientsLength - 1;
                widgetDetailsPatient.curPatient = patients[nextPosition];
            } else if (nextPosition >= patientsLength) {
                nextPosition = 0;
                widgetDetailsPatient.curPatient = patients[nextPosition];
            }
            eventAggregator.publish("openPatientDetails", patients[nextPosition]);
        });
    };

    this.closePopupNewHospitalization = function(widget) {
        var fieldHall = widget.getField('hall');
        fieldHall.dataSourceFilter = [];
        fieldHall.dataSource.name = '/HallsDistincts';
        fieldHall.reload();
        ScreenService.closePopup();
    };

    eventAggregator.subscribe('sendPatientUpdate', function(patientData) {
        var widget = patientData.widget;
        var editedPatient = patientData.patient[0];
        var promise = ZHPromise.defer();
        editedPatient.situation = patientData.situation;
        editedPatient.dischargeDate = Validation.now();
        editedPatient.changed = true;
        editedPatient.newHospitalization = editedPatient.situation !== "INTERNADO" ? false : true;
        if (editedPatient.situation === 1) {
            editedPatient.isDead = true;
        }
        SynchronizeService.persistOnline('/Patients', '/Patients/save', 'DataSet', [editedPatient]).then(function() {
            editedPatient.changed = false;
        }, function() {
            editedPatient.changed = true;
            editedPatient.newHospitalization = true;
        }).finally(function() {
            promise.resolve();
        });
        promise.then(function() {
            MapNutritionistService.confirmationPatientStatus(patientData).then(function() {
                BedsService.getBed(editedPatient.bed).then(function(bedData) {
                    BedsService.changeOccupiedBedToVacate(bedData).then(function() {
                        var containerMain = widget.container;
                        containerMain.unSelectAll(widget);
                        Environment.windowReload(containerMain);
                    });
                });
            });
        });
    });

    this.directSave = function(widget) {
        var directFunctionSaveByEditing = {
            "isPatientEditing": this.savePatient,
            "isHospitalizationEditing": this.saveHospitalization
        };
        Object.keys(directFunctionSaveByEditing).forEach(function(key) {
            if (widget[key]) {
                directFunctionSaveByEditing[key].apply(null, [widget]);
            }
        });
    };

    this.maskFormat = function(field, precision, completeWith, valueBindCurrentRow) {
        if (field, typeof precision === 'number', typeof completeWith === 'string') {
            if (!isNaN(field.value())) {
                var value = String(field.value()).padStart(precision, completeWith);
                field.value(value);
                var valueBind = valueBindCurrentRow || field.valueField || field.name;
                field.widget.currentRow[valueBind] = value;
            }
        }
    };
    this.verifyAdjusts = function(field, precision, completeWith) {
        var adjusts = getLocalVar("__ISAJUSTA");
        if (adjusts === 'S') {
            this.maskFormat(field, precision, completeWith);
        }
    };

    this.savePatient = function(widget) {
        if (widget.isValid()) {
            var currentRow = widget.currentRow;
            if (isValidPatientData(currentRow)) {
                var rowToSave = createRowToSavePatient(currentRow);
                rowToSave.patientSequentialCode = getLocalVar("__ISSEQUENTIAL");
                rowToSave.patientAjusta = getLocalVar("__ISAJUSTA");
                restEngine.post('/Patients/save', {
                    "requestType": "DataSet",
                    "dataset": [rowToSave]
                }).then(function(callback) {
                    var msg;
                    if (callback.dataset && callback.dataset.error && Array.isArray(callback.dataset.error.ERROR) && callback.dataset.error.ERROR[0] && callback.dataset.error.ERROR[0].indexOf('ORA-00001') !== -1) {
                        msg = ScreenService.i18n('Já existe um paciente cadastrado com este código!');
                        ScreenService.showMessage(msg);
                    } else {
                        localStorageEngine.save('/Profiles', {
                            'confidentiality': null,
                            'eatingHabits': "",
                            'foodIntolerance': "",
                            'id': rowToSave.id,
                            'observation': null,
                            'unit': rowToSave.unit
                        }).then(function() {
                            rowToSave.__is_new = false;
                            rowToSave.changed = false;
                            rowToSave.dischargeDate = null;
                            MapNutritionistService.savePatient(rowToSave).then(function() {
                                widget.container.getWidget('WidgetMain').reload();
                            });
                            widget.container.getWidget('WidgetMain').reload();
                            ScreenService.closePopup();
                            widget.container.getWidget('WidgetMain').reload();
                            msg = ScreenService.i18n('Paciente cadastrado com sucesso!');
                            ScreenService.showMessage(msg, 'success');
                        });
                    }
                }, function() {
                    var msg = ScreenService.i18n('Não foi possível conectar ao servidor, tente novamente mais tarde.');
                    ScreenService.showMessage(msg);
                });
            }
        }
    };

    function isValidPatientData(currentRow) {
        var dataNascimento = moment(currentRow.birthDate, "YYYY-MM-DD");
        var date = moment().diff(dataNascimento, 'days');
        if (date < 0) {
            ScreenService.showMessage(ScreenService.i18n("Data de nascimento inválida!"));
            return false;
        }
        date = moment().diff(dataNascimento, 'years');
        if (date > 130) {
            ScreenService.showMessage(ScreenService.i18n("Data de nascimento inválida!"));
            return false;
        }
        if (dataNascimento > moment()) {
            ScreenService.showMessage(ScreenService.i18n("Data de nascimento inválida!"));
            return false;
        }
        return true;
    }
    var getAge = function(dateOfBirth, format, typeReturn) {
        var dateBirth = moment(dateOfBirth, format);
        if (dateBirth.isValid()) {
            var dateDefault = moment(dateOfBirth, format).format('YYYY-MM-DD');
            var years = moment().diff(dateDefault, 'years');
            var months = moment().diff(dateDefault, 'month');
            var decimalMonths = moment().diff(dateDefault, 'month', true) - Math.trunc(moment().diff(dateDefault, 'month', true));
            if (decimalMonths >= 0.5) {
                months++;
            }
            var monthBirth = months - (years * 12);
            switch (typeReturn) {
                case 'years':
                    return years;
                case 'months':
                    return months;
                case 'full':
                    return {
                        '0': years,
                        '1': monthBirth
                    };
                default:
                    return null;
            }
        }
    };

    function createRowToSavePatient(patientInfos) {
        var yearAndMonth = getAge(patientInfos.birthDate, "YYYY-MM-DD", "full");

        var rowToSave = {
            "organizationNumber": localStorageEngine.getLocalVar("__NRORGTRAB"),
            "unit": localStorageEngine.getLocalVar("__CDFILIAL"),
            "id": patientInfos.id,
            "name": patientInfos.name,
            "age": patientInfos.age,
            "idnamepaci": patientInfos.id + ' - ' + patientInfos.name + ' ' + patientInfos.lastName.toUpperCase(),
            "normalizedName": removerAcentos(patientInfos.name + ' ' + patientInfos.lastName.toUpperCase()),
            "yearAndMonth": yearAndMonth,
            "formattedName": patientInfos.name + ' ' + patientInfos.lastName.toUpperCase(),
            "lastName": patientInfos.lastName,
            "birthDate": moment(patientInfos.birthDate, "YYYY-MM-DD").format("DD/MM/YYYY"),
            "gender": patientInfos.gender,
            "situation": null,
            "__is_new": true,
            "changed": true

        };

        return rowToSave;

    }

    function removerAcentos(newStringComAcento) {
        var string = newStringComAcento;
        var mapaAcentosHex = {
            a: /[\xE0-\xE6]/g,
            A: /[\xC0-\xC6]/g,
            e: /[\xE8-\xEB]/g,
            E: /[\xC8-\xCB]/g,
            i: /[\xEC-\xEF]/g,
            I: /[\xCC-\xCF]/g,
            o: /[\xF2-\xF6]/g,
            O: /[\xD2-\xD6]/g,
            u: /[\xF9-\xFC]/g,
            U: /[\xD9-\xDC]/g,
            c: /\xE7/g,
            C: /\xC7/g,
            n: /\xF1/g,
            N: /\xD1/g
        };

        for (var letra in mapaAcentosHex) {
            var expressaoRegular = mapaAcentosHex[letra];
            string = string.replace(expressaoRegular, letra);
        }

        return string;
    }

    this.setPatientAge = function(patientWidget) {
        var data = moment(patientWidget.currentRow.birthDate, "YYYY-MM-DD");
        if (data._i) {
            var years = moment().diff(data, 'years');
            if (years < 0) {
                years = 0;
            }
            patientWidget.currentRow.age = years;
            var yearAndMonth = getAge(data, "YYYY-MM-DD", "full");
            var idade;
            if (yearAndMonth[0] === 0) {
                idade = yearAndMonth[1] + ' Meses';
            } else if (yearAndMonth[1] === 0) {
                idade = yearAndMonth[0] + ' Anos';
            } else {
                idade = yearAndMonth[0] + ' Anos e ' + yearAndMonth[1] + ' Meses';
            }
            patientWidget.currentRow.yearAndMonth = idade;
        }
    };

    this.saveHospitalization = function(widget) {
        var fieldEdited = widget.currentRow;
        var promise = ZHPromise.defer();
        if (isValidHospitalization(widget)) {
            if (isValidDate(widget)) {
                createRowToHospitalization(fieldEdited).then(function(rowToSave) {
                    SynchronizeService.persistOnline('/Patients', '/Patients/save', 'DataSet', [rowToSave]).then(function() {
                        rowToSave.changed = false;
                        rowToSave.newHospitalization = false;
                    }).finally(function() {
                        promise.resolve();
                    });
                    promise.then(function() {
                        MapNutritionistService.savePatient(rowToSave).then(function() {
                            widget.reload().then(function() {
                                var fieldHall = widget.getField('hall');
                                fieldHall.dataSource.name = '/HallsDistincts';
                                fieldHall.dataSourceFilter = [];
                                fieldHall.reload();
                                ScreenService.closePopup();
                                widget.container.getWidget('WidgetMain').reload();
                                BedsService.getBed(rowToSave.bed).then(function(vacantBedToOccupy) {
                                    BedsService.changeVacantBedToOccupied(vacantBedToOccupy).then(function() {
                                        self.onEnterWidgetMain(widget);
                                    }, function() {
                                        ScreenService.showMessage(ScreenService.i18n("Não foi possível internar o paciente. Tente novamente"), 'error');
                                    });
                                });
                            });
                        });
                        widget.isHospitalizationEditing = false;
                        widget.list();

                    });
                }, function(error) {
                    ScreenService.showMessage(ScreenService.i18n(error), 'error');
                });
            }
        }
    };

    function isValidDate(widget) {
        var row = angular.copy(widget.currentRow);
        //var date = moment(, "YYYY-MM-DDTHH:mm").format("YYYY-MM-DD");

        if (moment(row.hospitalizationDate, 'YYYY-MM-DDTHH:mm') > moment()) {
            ScreenService.showMessage(ScreenService.i18n('A data informada deve ser menor ou igual a data atual.'));
            return false;
        } else {
            return true;
        }
    }

    function isValidHospitalization(widget) {
        return widget.isValid();
    }

    function createRowToHospitalization(field) {
        var deferred = ZHPromise.defer();
        var rowToHospitalization;
        MapNutritionistService.getPatientById(field.id).then(function(patient) {
            var filter = [{
                "name": "bed",
                "operator": "=",
                "value": field.bed
            }];
            rowToHospitalization = angular.copy(patient);
            localStorageEngine.getDataSource("/BedsPrescriptions", filter).then(function(beds) {
                if (beds && beds.length) {
                    rowToHospitalization = Object.assign(rowToHospitalization, beds[0]);
                }
                rowToHospitalization.lastVisit = null;
                rowToHospitalization.medicalTeamId = field.medicalTeamId;
                rowToHospitalization.newHospitalization = true;
                rowToHospitalization.changed = true;
                rowToHospitalization.healthInsuranceId = field.hospitalizationPattern;
                rowToHospitalization.hospitalization = moment(field.hospitalizationDate, "YYYY-MM-DDTHH:mm").format("DD/MM/YYYY HH:mm");
                rowToHospitalization.situation = "INTERNADO";
                rowToHospitalization.id = patient.id;
                rowToHospitalization.DTINCLUSAO = moment().format("DD/MM/YYYY HH:mm:ss");
                rowToHospitalization.DTULTATU = moment().format("DD/MM/YYYY HH:mm:ss");
                rowToHospitalization.CDOPERINCLUSAO = getLocalVar('__CDOPERADOR');
                rowToHospitalization.CDOPERULTATU = getLocalVar('__CDOPERADOR');
                rowToHospitalization.__is_new = patient.__is_new;
                rowToHospitalization.organizationNumber = getLocalVar("__NRORGTRAB");
                deferred.resolve(rowToHospitalization);
            });
        }, function(error) {
            deferred.reject(error);
        });
        return deferred.promise;
    }

    this.onEnterWidgetMain = function(widget) {
        getBedsPrescriptions(widget);
        getNonHospitalizedPatients(widget);
        getMedicalTeam(widget);

        var filter = [{
                name: [{
                    name: "formattedName",
                    direction: "ASC"
                }],
                operator: "ORDER_BY",
                value: ""
            },
            {
                name: "unit",
                operator: "=",
                value: getLocalVar("__CDFILIAL")
            }
        ];
        widget.reload(filter);
    };

    function getBedsPrescriptions(widget) {

        var filter = [{
            name: [{
                name: "bed",
                direction: "ASC"
            }],
            operator: "ORDER_BY",
            value: ""
        }];

        return localStorageEngine.getDataSource("/BedsPrescriptions", filter).then(function(bedsPrescriptions) {
            widget.BedsPrescriptions = bedsPrescriptions;
        });

    }

    function getNonHospitalizedPatients(widget) {

        var filter = [{
            name: "bed",
            operator: "IS_NULL",
            value: ""
        }, {
            name: [{
                    name: "id",
                    direction: "ASC"
                },
                {
                    name: "name",
                    direction: "ASC"
                }
            ],
            operator: "ORDER_BY",
            value: ""
        }];

        return localStorageEngine.getDataSource("/Patients", filter).then(function(nonHospitalizedPatients) {
            widget.NonHospitalizedPatients = nonHospitalizedPatients;
        });
    }

    function reloadFields(widget) {
        widget.fields.forEach(function(field) {
            if (field.dataSource) {
                field.reload([{
                    "name": [{
                        "name": field.descriptionField,
                        "direction": "ASC"
                    }],
                    "operator": "ORDER_BY",
                    "value": ""
                }]);
            }
        });
    }

    function getMedicalTeam(widget) {
        return localStorageEngine.getDataSource("/MedicalTeams", null).then(function(medicalTeam) {
            widget.MedicalTeam = medicalTeam;
        });
    }

    function clearDependentFields(parentField) {
        var dependentFieldsNames = parentField.dependentFields || [];
        dependentFieldsNames = Array.isArray(dependentFieldsNames) ? dependentFieldsNames : [dependentFieldsNames];
        dependentFieldsNames.forEach(function(dependentFieldName) {
            var dependentField = parentField.widget.getField(dependentFieldName);
            dependentField.setValue("");
            dependentField.clearValue();
            clearDependentFields(dependentField);
        });
    }

    this.onChangeFoodControlService = function(widget) {
        widget.redraw();
        var idformdiet = { //Diet
            '0': ['0', '3', '4'], //'NORMAL', 'ROOM SERVICE', 'ESPECIAL'
            '2': '1', //'LACTARIO',
            '1': '2' //'ENTERAL',
        };
        var currentRow = widget.currentRow;
        var fieldService = widget.getField('serviceId');
        var service = fieldService.dataSource.data.filter(function(service) {
            return service.serviceId === widget.currentRow.serviceId;
        })[0];

        clearDependentFields(fieldService);
        var fieldDiet = widget.getField('dietId');
        fieldDiet.readOnly = false;
        currentRow.serviceId = service.serviceId;
        currentRow.serviceName = service.serviceName;
        currentRow.idtpservhosp = service.service;
        currentRow.unit = service.unit;
        currentRow.selectedDiet = {};
        currentRow.dietId = "";
        currentRow.dietTypeName = "";
        currentRow.dietService = "";
        widget.filtredFeedingForWidget = [];
        widget.filtredFeedingsForWidget = [];

        fieldDiet.dataSourceFilter.forEach(function(currentFilter) {
            if (currentFilter.name === "service") {
                currentFilter.value = idformdiet[currentRow.idtpservhosp];
                if (currentRow.idtpservhosp === "0" || !currentRow.idtpservhosp) {
                    currentFilter.operator = "IN";
                    currentFilter.value = currentFilter.value || ['0', '3', '4'];
                }

            } else {
                currentFilter.value = currentRow[currentFilter.name];
            }
        });

        fieldDiet.reload();
    };
    eventAggregator.subscribe("updateDietFromPrescriptionDiet", function(widgetPopupPrescricao) {
        changeFoodControlDiet(widgetPopupPrescricao, true);
    });

    this.onChangeFoodControlDiet = function(widget, changedDatePrescription) {
        var fieldDate = widget.getField('serviceDate');
        var date = changedDatePrescription === true && fieldDate ? moment(fieldDate.value(), 'YYY-MM-DD') : false;
        if ((changedDatePrescription === true && widget.currentRow.typeClicked === 'oral' && date && date.isValid() && widget.currentRow.dietService !== '4' && widget.currentRow.idformdiet !== '4') || !changedDatePrescription) {
            changeFoodControlDiet(widget);
        }
    };

    function changeFoodControlDiet(widget) {

        var tipodeservico = {
            '0': 'NORMAL',
            '1': 'LACTARIO',
            '2': 'ENTERAL',
            '3': 'ROOM SERVICE',
            '4': 'ESPECIAL'
        };

        var currentRow = widget.currentRow;
        var selectDiet = widget.getField('dietId').dataSource.data.filter(function(currentDiet) {
            return currentDiet.dietId === widget.getField('dietId').value();
        })[0];

        if (selectDiet) {
            currentRow.dietService = selectDiet.service;
            currentRow.dietId = selectDiet.dietId;
            currentRow.dietName = selectDiet.dietName;
            currentRow.dietTypeName = tipodeservico[selectDiet.service];
            currentRow.dietGroup = selectDiet.dietGroup;
        }
        var curDate = currentRow.serviceDate;
        var dateEdited = moment(curDate, 'YYYY-MM-DD').format('DD/MM/YYYY');
        curDate = dateEdited === "Invalid date" ? curDate : dateEdited;
        curDate = moment(curDate, "DD/MM/YYYY").format("DD/MM/YYYY");
        var cdfilial = localStorageEngine.getLocalVar('__CDFILIAL');
        var filter = [{
            "name": "dtCard",
            "operator": "=",
            "value": curDate
        }, {
            "name": "unit",
            "operator": "=",
            "value": cdfilial
        }];

        if (currentRow.idtpservhosp !== "2") { //Não for lactario
            filter.push({
                "name": "dietid",
                "operator": "=",
                "value": currentRow.dietId
            }, {
                "name": "cdservfili",
                "operator": "=",
                "value": currentRow.serviceId
            }, {
                "name": [{
                    "name": "NRORDPRATCAR",
                    "direction": "ASC"
                }],
                "operator": "ORDER_BY",
                "value": ""
            });
            widget.filtredFeedingsForWidget = [];
            if (selectDiet) {
                if (selectDiet.idtpservhosp !== "4" && currentRow.dietService !== '4') {
                    localStorageEngine.getDataSource('/Plates', filter).then(function(platesByDiet) {
                        var filteredPlates = platesByDiet;
                        var promisePlates = ZHPromise.defer();
                        if (!platesByDiet.length) {
                            var newFilter = [];
                            filter.forEach(function(cFilter) {
                                if (cFilter.name !== 'dietid') {
                                    newFilter.push(cFilter);
                                }
                            });
                            localStorageEngine.getDataSource('/PlatesPla', newFilter).then(function(platesPla) {
                                filteredPlates = platesPla;
                                promisePlates.resolve();
                            });
                        } else {
                            promisePlates.resolve();
                        }
                        promisePlates.then(function() {
                            filteredPlates.forEach(function(fe) {
                                fe.selectedByUser = fe.isRequiredOnMenu;
                                fe.plateType = fe.isRequiredOnMenu ? fe.plateType : fe.plateType + ' (' + ScreenService.i18n('Opção') + ')';
                                fe.idopcaoconf = fe.isRequiredOnMenu ? fe.idopcaoconf = 'N' : fe.idopcaoconf = 'S';
                            });
                            var arrayFinal = [];
                            filteredPlates.reduce(function(prev, current) {
                                var index = prev.findIndex(function(j) {
                                    return j.plateType === current.plateType;
                                });
                                if (index < 0) {
                                    index = prev.length;
                                    prev.push({
                                        plateType: current.plateType,
                                        lista: []
                                    });
                                }
                                prev[index].lista.push(current);
                                return prev;
                            }, []).map(function(item) {
                                return item.lista;
                            }).forEach(function(it) {
                                if (it.length === 0) {
                                    it.isFirst = true;
                                    arrayFinal.push(it);
                                } else {
                                    for (var i = 0; i < it.length; i++) {
                                        if (i === 0) {
                                            it[i].isFirst = true;
                                        }
                                        arrayFinal.push(it[i]);
                                    }
                                }
                            });
                            widget.filtredFeedingsForWidget = arrayFinal;
                        });
                    });
                } else {
                    widget.openPopup = false;
                    loadDataPopupPlates(widget);
                }
            }
        }
    }
    eventAggregator.subscribe('loadDataPlatesByUnit', function(widget) {
        loadDataPopupPlates(widget);
    });

    function loadDataPopupPlates(widget) {
        var widgetPopupPlates = widget.container.getWidget('widgetPopupPlates');
        widgetPopupPlates.allFieldsAreSelected = false;
        widgetPopupPlates.selectedRows = [];
        var widgetPrescricao = widget.container.getWidget('widgetPopupPrescricao');
        var filter = [{
            "name": "cdservfili",
            "operator": "=",
            "value": widgetPrescricao.currentRow.serviceId
        }];
        localStorageEngine.getDataSource('/PlatesByUnit', filter).then(function(dataSource) {
            var dataSourceName = widgetPopupPlates.descriptionPopupSelect.dataSourceName;
            widgetPopupPlates[dataSourceName] = angular.copy(dataSource);
            widgetPopupPlates.elementsToSearch = angular.copy(dataSource);
            widgetPopupPlates.elementsToBind = widgetPopupPlates.elementsToSearch.reduce(function(prev, current) {
                var index = prev.findIndex(function(j) {
                    return j.plateType === current.plateType;
                });
                if (index < 0) {
                    index = prev.length;
                    prev.push({
                        plateType: current.plateType,
                        lista: []
                    });
                }
                prev[index].lista.push(current);
                return prev;
            }, []).map(function(item) {
                item.lista.opened = false;
                return item.lista;
            });
            widgetPopupPlates.platesToSearch = angular.copy(dataSource);

            if (widgetPrescricao.filtredFeedingsForWidget && widgetPrescricao.filtredFeedingsForWidget.length) {
                widgetPrescricao.filtredFeedingsForWidget.forEach(function(feeding) {
                    var foundedPlates = widgetPopupPlates.elementsToSearch.filter(function(plate) {
                        return plate.formulationId === feeding.formulationId &&
                            plate.plateType === feeding.plateType;
                    });
                    var selectedPlate = foundedPlates[0];
                    if (selectedPlate) {
                        if (Array.isArray(widgetPopupPlates.selectedRows) && widgetPopupPlates.selectedRows.length) {
                            var row = widgetPopupPlates.selectedRows.filter(function(rowToSelect) {
                                return rowToSelect.formulationId === selectedPlate.formulationId && rowToSelect.plateType === selectedPlate.plateType;
                            })[0];
                            if (!row) {
                                widgetPopupPlates.selectedRows.push(selectedPlate);
                            }
                            selectedPlate.selectedRow = row ? row.selectedRow : true;
                        } else {
                            selectedPlate.selectedRow = true;
                            widgetPopupPlates.selectedRows.push(selectedPlate);
                        }
                    }
                });
                if (widgetPrescricao.filtredFeedingsForWidget.length === widgetPopupPlates.elementsToSearch.length) {
                    widgetPopupPlates.allFieldsAreSelected = true;
                } else {
                    widgetPopupPlates.allFieldsAreSelected = false;
                }
                widgetPopupPlates.backupSelectedRows = angular.copy(widgetPopupPlates[dataSourceName].filter(function(plate) {
                    return plate.selectedRow;
                }));
                widget.elementsToBind.forEach(function(parent) {
                    if (widget.allFieldsAreSelected) {
                        parent.selectedParent = true;
                        parent.opened = true;
                    } else {
                        var isSelected = true;
                        parent.forEach(function(filho) {
                            if (!filho.selectedRow) {
                                isSelected = false;
                            } else {
                                parent.opened = true;
                            }
                        });
                        parent.selectedParent = isSelected;
                    }
                });
            } else {
                widgetPopupPlates.selectedRows = [];
            }
            if (widget.openPopup) {
                ScreenService.openPopup(widget);
            }
        });
    }

    this.selectPlatesOnPopup = function(widget) {
        var popupPrescricaoOral = widget.container.getWidget('widgetPopupPrescricao');
        widget.container.getWidget('widgetPopupPlates').isVisible = false;
        ScreenService.closePopup();
        widget.descriptionPopupSelect.search = "";
        var selectedRows = [];
        widget.selectedRows.forEach(function(rowSelected) {
            var index = popupPrescricaoOral.filtredFeedingsForWidget.findIndex(function(feeding) {
                return feeding.formulationId === rowSelected.formulationId && feeding.plateType === rowSelected.plateType;
            });
            if (index > -1) {
                selectedRows.push(popupPrescricaoOral.filtredFeedingsForWidget[index]);
            } else {
                selectedRows.push(rowSelected);
            }

        });
        var arrayFinal = [];
        selectedRows.reduce(function(prev, current) {
            var index = prev.findIndex(function(j) {
                return j.plateType === current.plateType;
            });
            if (index < 0) {
                index = prev.length;
                prev.push({
                    plateType: current.plateType,
                    lista: []
                });
            }
            prev[index].lista.push(current);
            return prev;
        }, []).map(function(item) {
            return item.lista;
        }).forEach(function(it) {
            if (it.length === 0) {
                it.isFirst = true;
                arrayFinal.push(it);
            } else {
                for (var i = 0; i < it.length; i++) {
                    if (i === 0) {
                        it[i].isFirst = true;
                    }
                    arrayFinal.push(it[i]);
                }
            }
        });
        popupPrescricaoOral.filtredFeedingsForWidget = angular.copy(arrayFinal);

    };

    this.returnBackUpRowPopup = function(widget, namePopup) {
        var widgetPopup;
        if (typeof namePopup === 'string') {
            widgetPopup = widget.container.getWidget(namePopup);
        } else {
            widgetPopup = widget.container.getWidget('widgetPopupPlates');
        }
        widgetPopup.isVisible = false;
        widgetPopup.selectedRows = widgetPopup.backupSelectedRows ? angular.copy(widgetPopup.backupSelectedRows) : [];
        widget.descriptionPopupSelect.search = "";
        widget.search = "";
        ScreenService.closePopup();
    };

    this.returnBackUpRowPopupFormulationLact = function(widget) {
        widget.isVisible = false;
        widget.descriptionPopupSelect.search = "";
        if (widget.dataSourceFilter) {
            widget.dataSourceFilter.forEach(function(filter) {
                if (filter.operator === 'LIKE' || filter.operator === 'LIKE_I') {
                    filter.value = "%%";
                } else {
                    filter.value = "";
                }
            });
        }
        ScreenService.closePopup();
    };

    this.onChangeFormulation = function(fieldFormulation) {
        var widgetPopupItemLac = fieldFormulation.widget;
        var formulation = fieldFormulation.dataSource.getCheckedRows()[0];
        if (formulation) {
            widgetPopupItemLac.getField('un').value(formulation.un);
            widgetPopupItemLac.currentRow.formulationName = formulation.nmformulacao;
            widgetPopupItemLac.currentRow.formulationId = formulation.cdformulacao;
            fieldFormulation.value(formulation.nmformulacao);
        } else {
            widgetPopupItemLac.getField('un').value("");
            widgetPopupItemLac.currentRow.formulationName = "";
            widgetPopupItemLac.currentRow.formulationId = "";
            fieldFormulation.value("");
        }

    };

    this.storeyOnChange = function(widget) {
        widget.redraw();
        var bedField = widget.getField('bed');
        var hallField = widget.getField('hall');
        setTimeout(function() {
            widget.currentRow.hall = '';
            widget.currentRow.bed = '';

            if (widget.currentRow.storey) {
                hallField.dataSource.name = "/Halls";
                bedField.dataSourceFilter = [{
                        "name": "storey",
                        "operator": "=",
                        "value": widget.currentRow.storey
                    },
                    {
                        "name": "unit",
                        "operator": "=",
                        "value": getLocalVar('__CDFILIAL')
                    }
                ];
                hallField.dataSourceFilter = [{
                        "name": "storey",
                        "operator": "=",
                        "value": widget.currentRow.storey
                    },
                    {
                        "name": "unit",
                        "operator": "=",
                        "value": getLocalVar('__CDFILIAL')
                    }
                ];
            } else {
                hallField.dataSourceFilter = [];
                hallField.dataSource.name = "/HallsDistincts";
                bedField.dataSourceFilter = [{
                    "name": "unit",
                    "operator": "=",
                    "value": getLocalVar('__CDFILIAL')
                }];
            }
            bedField.reload();
            hallField.reload();
        }, 200);
    };

    this.hallOnChange = function(widget) {
        widget.redraw();
        var bedField = widget.getField('bed');
        widget.currentRow.bed = '';

        if (widget.currentRow.hall) {

            bedField.dataSourceFilter = [{
                    "name": "hall",
                    "operator": "=",
                    "value": widget.currentRow.hall
                },
                {
                    "name": "unit",
                    "operator": "=",
                    "value": getLocalVar('__CDFILIAL')
                }
            ];
            if (widget.currentRow.storey) {
                bedField.dataSourceFilter.push({
                    "name": "storey",
                    "operator": "=",
                    "value": widget.currentRow.storey
                });
            }
        } else {
            var has = false;

            bedField.dataSourceFilter.forEach(function(filtro) {
                if (filtro.name === "hall") {
                    has = true;
                }
            });
            if (has) {
                bedField.dataSourceFilter.splice(
                    bedField.dataSourceFilter.indexOf(
                        bedField.dataSourceFilter.filter(function(index) {
                            return index.name === "hall";
                        })[0]),
                    1);
            }
        }
        bedField.reload();
    };

    this.bedOnChange = function(widget) {
        if (widget.currentRow.bed) {
            var field = widget.getField('bed');
            var bed = field.dataSource.data.filter(function(filtro) {
                return filtro.bed === field.widget.currentRow.bed;
            })[0];
            widget.currentRow.hall = bed.hall;
            widget.currentRow.storey = bed.storey;
        }
    };

    this.controlAutoFills = function(widget) {
        var isAutomatic = widget.getField('automatic').value();
        var interval = widget.getField('interval');
        var period = widget.getField('period');
        var validationsIntervalExists = interval.validations;
        var validationsPeriodExists = interval.validations;
        if (widget) {
            if (isAutomatic === 'S') {
                interval.readOnly = false;
                period.readOnly = false;
                interval.validations = {
                    "required": true
                };
                period.validations = {
                    "required": true
                };
            } else {
                interval.clearValue();
                period.clearValue();
                interval.readOnly = true;
                period.readOnly = true;
                if (validationsIntervalExists && validationsPeriodExists) {
                    delete interval.validations.required;
                    delete period.validations.required;
                }
            }
        }
    };

    function saveAutoFills(currentRow, fieldDate, fieldHour) {
        var dateTimeInitial = moment(currentRow[fieldDate] + " " + currentRow[fieldHour], "YYYY-MM-DD HHmm");
        var dateTimeFeeding = moment(currentRow[fieldDate] + " " + currentRow[fieldHour], "YYYY-MM-DD HHmm");
        var period = currentRow.period;
        var interval = moment(currentRow.interval, "HHmm");
        var arrayDate = [];
        var periodOfMinutes = (parseInt(period.split(':')[0]) * 60) + parseInt(period.split(':')[1]);
        while (dateTimeFeeding.diff(dateTimeInitial, 'minutes') < periodOfMinutes) {
            dateTimeFeeding.add('m', interval.format('mm')).add('h', interval.format('HH'));
            if (dateTimeFeeding.diff(dateTimeInitial, 'minutes') <= periodOfMinutes) {
                arrayDate.push(dateTimeFeeding.format('YYYY-MM-DD HHmm'));
            }
        }
        return arrayDate;
    }

    function checkNutritionist() {
        localStorageEngine.getDataSource('/nutritionist').then(function(nutritionist) {
            if (nutritionist.length === 0) {
                ScreenService.showMessage(ScreenService.i18n('O operador não possui acesso à essa tela, pois não está cadastrado como nutricionista.'), 'error').then(function() {
                    ScreenService.openWindow('MapKitchenMaid');
                });
            }
        });
    }
};

Configuration(function(ContextRegister) {
    ContextRegister.register('MapNutritionistController', MapNutritionistController);
});


// FILE: js/controllers/PatientController.js
var PatientController = function(templateManager, localStorageEngine, Environment, ScreenService) {

    var containerMain;
    var widgetMain;
    var widgetProfile;
    var widgetPrescription;
    var widgetAppointment;
    var widgetControlFeeding;
    var widgetNavigationStripe;

    this.cancel = function(widget) {
        ScreenService.closeSwipe();
        widget.container.getWidget('WidgetControlFeeding').activate();
    };

    // WIDGET NAVIGATION STRIPE

    this.widgetAfterMoveRow = function(widget, name) {
        if (name === "next") {
            widget.moveToNext();
        } else {
            widget.moveToPrev();
        }
        var patient = widget.currentRow;
        Environment.setCurrentPatient(patient);
        Environment.setCurrentPatientId(patient.id);

        widgetProfile.reload().then(function() {
            widgetProfile.moveToFirst();
        });

        widgetPrescription.reload().then(function() {
            if (widgetPrescription.dataSource.data.length === 0) {
                widgetPrescription.fields.forEach(function(field) {
                    field.value("");
                });
            }
            widgetPrescription.moveToFirst();
        });

        widgetAppointment.reload();
        widgetControlFeeding.reload();

    };

    this.widgetOnEnter = function(widget) {

        widget.reload().then(function() {
            widget.moveToFirst();
        });

    };

    var _initNavigationStripe = function() {

        widgetNavigationStripe.currentRow = Environment.getCurrentPatient();
        widgetNavigationStripe.dataSource.data = Environment.getDataSourceMain();

    };

    var _initContainer = function(container) {

        containerMain = container;
        widgetNavigationStripe = container.getWidget('WidgetNavigationStripe');
        widgetMain = container.getWidget('WidgetMain');
        widgetProfile = container.getWidget('WidgetProfile');
        widgetPrescription = container.getWidget('WidgetPrescription');
        widgetAppointment = container.getWidget('WidgetAppointment');
        widgetControlFeeding = container.getWidget('WidgetControlFeeding');

    };

    this.containerAfterinit = function(container) {

        _initContainer(container);
        _initNavigationStripe();
        container.restoreDefaultMode();

    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('PatientController', PatientController);
});


// FILE: js/controllers/PrescriptionController.js
var PrescriptionController = function(Validation, ScreenService, Environment, PrescriptionService) {

    var containerMain;
    var widgetStripe;
    var widgetMain;

    var _verify = function(widget) {

        widget.reload().then(function() {

            widget.dataSource.data.forEach(function(prescription) {

                if (Date.parse(prescription.date) < Date.parse(Validation.now())) {
                    widget.dataSource.data.push(prescription);
                }

            });

            widget.moveToFirst();

        });

    };

    this.openHistory = function(widget) {

        widget.editing = true;
        ScreenService.openSwipe(widget);

    };

    this.onEnter = function(widget) {
        _verify(widget);
    };

    var _initStripe = function() {
        widgetStripe.currentRow = Environment.getCurrentPatient();
    };

    var _initContainer = function(container) {

        containerMain = container;
        widgetStripe = container.getWidget('WidgetStripe');
        widgetMain = container.getWidget('WidgetPrescriptionsHistoryMain');

    };

    this.containerAfterinit = function(container) {

        _initContainer(container);
        _initStripe();

    };

    var _validateFields = function(fields) {

        if (!fields.bed) {
            throw "Informe o leito.";
        }

        if (!fields.date) {
            throw "Informe a data da prescrição.";
        }

        if (!fields.dietId) {
            throw "Informe a dieta.";
        }

        if (!fields.nutritionalStatusId) {
            throw "Informe o nível nutricional.";
        }

    };

    this.edit = function(widget) {

        var prev = widget.container.getWidget('WidgetPrescription').currentRow;

        if (prev !== undefined) {

            widget.widgets[0].currentRow = widget.container.getWidget('WidgetPrescription').currentRow;
            widget.widgets[0].template = "widget/form.html";
            widget.widgets[0].edit();
            widget.widgets[0].getField('bed').readOnly = true;
            widget.widgets[0].getField('date').readOnly = true;
            widget.widgets[0].getField('dietName').readOnly = true;
            widget.widgets[0].label = ScreenService.i18n('Editar Prescrição');

            ScreenService.openSwipe(widget);

        } else {

            var msg = ScreenService.i18n('Não há itens a serem editados.');
            ScreenService.showMessage(msg);

        }

    };

    this.add = function(widget) {
        widget.widgets[0].getField('date').readOnly = false;
        widget.widgets[0].getField('dietName').readOnly = false;
        widget.widgets[0].newRow();
        widget.widgets[0].label = ScreenService.i18n('Adicionar Prescrição');
        widget.widgets[0].getField('bed').value(Environment.getCurrentPatient().bed);

        ScreenService.openSwipe(widget);

    };

    this.confirm = function(widget) {
        var WidgetPrescription = widget.container.getWidget('WidgetPrescription');
        try {

            _validateFields(widget.currentRow);

            widget.currentRow.changed = true;
            widget.currentRow.id = Environment.getLocalVar('__patient');
            widget.currentRow.unit = Environment.getLocalVar('__CDFILIAL');

            PrescriptionService.save(widget.currentRow).then(function() {

                Environment.lastPatientVisit();

                _verify(widget.container.getWidget('WidgetPrescription'));

                ScreenService.closeSwipe();
                // ScreenService.showMessage('Salvo com sucesso.');
                WidgetPrescription.activate();
            });

        } catch (exceptionMessage) {
            ScreenService.showMessage(exceptionMessage);
        }

    };

    // HISTORY
    this.openSwipe = function(widget) {

        widget.widgets[0].widgets[0].currentRow = widget.currentRow;
        widget.widgets[0].widgets[0].view();
        widget.widgets[0].widgets[0].activate();
        ScreenService.openSwipe(widget.widgets[0]);

    };

    this.cancel = function(widget) {
        var WidgetPrescription = widget.container.getWidget('WidgetPrescription');
        ScreenService.closeSwipe().then(function() {
            WidgetPrescription.activate();
        });
    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('PrescriptionController', PrescriptionController);
});


// FILE: js/controllers/ProfileController.js
var ProfileController = function(ScreenService, Environment, ProfileService) {

    var containerMain;
    var widgetStripe;
    var widgetMain;

    this.onEnter = function(widget) {

        widget.reload().then(function() {
            widget.moveToFirst();
        });

    };

    var _initStripe = function() {
        widgetStripe.currentRow = Environment.getCurrentPatient();
    };

    var _initContainer = function(container) {

        containerMain = container;
        widgetStripe = container.getWidget('WidgetStripe');
        widgetMain = container.getWidget('WidgetMain');

    };

    this.containerAfterinit = function(container) {

        _initContainer(container);
        _initStripe();

    };

    // EDIT

    this.save = function() {

        ProfileService.save(widgetMain.currentRow).then(function() {
            ScreenService.goBack();
            Environment.lastPatientVisit();
        });

    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('ProfileController', ProfileController);
});


// FILE: js/controllers/ServerController.js
function ServerController(ScreenService, templateManager, Environment, ServerService) {

    this.selectPopUp = function(cadastrarSenha, conferirSenha) {

        var currentPopup = conferirSenha;

        if (!Environment.getLocalVar('password')) {

            currentPopup = cadastrarSenha;

        }

        currentPopup.newRow();
        ScreenService.openPopup(currentPopup);
    };

    this.configureIp = function(configurarIp) {
        popUpConfigurarIp(configurarIp);
    };

    this.validatePass = function(widgetSenha, configurarIp) {

        if (widgetSenha.isValid()) {

            var password = Environment.getLocalVar('password');

            if (password === widgetSenha.currentRow.senha) {

                popUpConfigurarIp(configurarIp);

            } else {

                ScreenService.showMessage(ScreenService.i18n('Senha não confere'), 'error');

            }

        }

    };

    this.setPassword = function(widgetCadastrarSenha, configurarIp) {

        if (widgetCadastrarSenha.isValid()) {

            var row = widgetCadastrarSenha.currentRow;

            if (row.senha.indexOf(" ") >= 0 || row.confirmarsenha.indexOf(" ") >= 0) {

                ScreenService.showMessage(ScreenService.i18n('Senha inválida. Não é permitido "espaço" na senha'), 'error');

            } else if (row.senha !== row.confirmarsenha) {

                ScreenService.showMessage(ScreenService.i18n('Senha não confere'), 'error');

            } else {

                popUpConfigurarIp(configurarIp);
                Environment.setLocalVar('password', row.senha);
                ScreenService.successNotification('Senha cadastrada com sucesso', 3000);

            }

        }

    };

    function popUpConfigurarIp(configurarIp) {
        configurarIp.newRow();
        ScreenService.closePopup();
        ScreenService.openPopup(configurarIp);
        configurarIp.currentRow.ip = getLocalVar('ipSemPorta');
    }

    this.setServerIp = function(widgetConfigurarIp, row, fieldDemonstracao) {

        if (widgetConfigurarIp.isValid()) {

            var ip = row.ip;
            var diretorio = row.diretorio ? '/' + row.diretorio : '';
            var porta = !row.porta ? '' : ':' + row.porta;
            var url = 'https://' + ip + porta + diretorio + window.serviceUrl.replace('..', '');

            templateManager.updateURL(url);

            // do test request
            ServerService.testConnection().then(function(serverStatus) {
                // request success
                Environment.setLocalVar('ipSemPorta', ip);
                Environment.setLocalVar('porta', row.porta);
                Environment.setLocalVar('diretorio', row.diretorio);
                Environment.setLocalVar('ip', url);
                fieldDemonstracao.isVisible = false;
                // close the ip popup
                ScreenService.closePopup();
                templateManager.updateTemplate();

                var conexaoValida = ServerService.compareVersion(serverStatus[0].version);
                var objMessage = ServerService.getObjectMessage(conexaoValida);

                if (conexaoValida === "OK") {
                    ScreenService.successNotification(ScreenService.i18n(objMessage.message), 3000);
                } else {
                    Environment.remove('/connection');
                    ScreenService.showMessage(ScreenService.i18n(objMessage.message), objMessage.type);
                }

            }, function() {
                // request fail
                ScreenService.confirmMessage("Não foi possível conectar ao servidor, deseja manter o IP informado?", 'question', function() {
                    // yes
                    Environment.setLocalVar('ipSemPorta', ip);
                    Environment.setLocalVar('porta', row.porta);
                    Environment.setLocalVar('diretorio', row.diretorio);
                    Environment.setLocalVar('ip', url);
                    fieldDemonstracao.isVisible = false;
                    // close the ip popup
                    ScreenService.closePopup();
                    templateManager.updateTemplate();

                }, function() {

                    // no, set last ip if exists
                    if (Environment.getLocalVar('ip')) {
                        templateManager.updateURL(Environment.getLocalVar('ip'));
                    }

                    widgetConfigurarIp.newRow();
                    templateManager.updateTemplate();

                });
            });
        }
    };
}
Configuration(function(ContextRegister) {
    ContextRegister.register('ServerController', ServerController);
});


// FILE: js/controllers/ServiceRequestController.js
function serviceRequestController(ScreenService, localStorageEngine) {

    this.loadCartFromStorage = function(widget) {
        localStorageEngine.getDataSourceItems("/cartItems").then(function(data) {
            var cartItems = JSON.parse(data);
            if (cartItems && cartItems.length) {
                widget.dataSource.addChangedRows(cartItems);
                widget.reload().then(function() {
                    this.updateCart(widget);
                }.bind(this));
            }
        }.bind(this));
    };

    this.updateCart = function(widget) {
        var cartItems = widget.dataSource.getChangedRows().filter(function(item) {
            return item.QUANTIDADE > 0;
        });
        localStorageEngine.setDataSource("/cartItems", cartItems);
        widget.getHeaderActions("cart").hint = cartItems.length;
    };
}
Configuration(function(ContextRegister) {
    ContextRegister.register("ServiceRequestController", serviceRequestController);
});


// FILE: js/controllers/SynchronizeController.js
var SynchronizeController = function(ScreenService, SynchronizeService, eventAggregator, templateManager, localStorageEngine) {

    this.sync = function() {
        syncService();
    };

    eventAggregator.subscribe('wscCallUpdate', syncService);

    function syncService() {
        ScreenService.showLoader();
        localStorageEngine.getDataSource('/User').then(function(user) {
            if (user.length > 0 && user[0].CDFILIALSELECIONADA !== null && user[0].CDFILIALSELECIONADA !== undefined) {
                localStorageEngine.setLocalVar('__CDFILIAL', user[0].CDFILIALSELECIONADA);
                SynchronizeService.sync().then(function(ocurredError) {
                        if (!ocurredError) {
                            ScreenService.showMessage(ScreenService.i18n('Sincronizado com Sucesso.'), 'success');
                        }
                        var widget = templateManager.container.getWidget('WidgetMain');

                        setTimeout(function() {
                            eventAggregator.publish("reloadScreenAndClearFilters", widget);
                            eventAggregator.publish("reloadMapKitchenMaid", widget);
                            menu.closeMenu(); // jshint ignore:line
                        }, 600);
                        localStorageEngine.setLocalVar('__SYNCRONIZED', true);
                        eventAggregator.publish("callCheckNutritionist", null);
                    },
                    function() {
                        ScreenService.showMessage(ScreenService.i18n('Não foi possível conectar ao servidor, tente novamente mais tarde.'));
                    });
            } else {
                // ScreenService.confirmMessage(ScreenService.i18n('Selecione uma unidade para prosseguir a sincronização.'), 'error', function() {
                //     eventAggregator.publish("openUnitPopup", templateManager.container.getWidget('WidgetUnit'));
                // }, function() {
                //     ScreenService.openWindow('Dashboard');
                // });
                ScreenService.showMessage(ScreenService.i18n('Selecione uma unidade para prosseguir a sincronização.'));
                eventAggregator.publish("openUnitPopup", templateManager.container.getWidget('WidgetUnitSync'));
            }
        }).finally(function() {
            ScreenService.hideLoader();
        });
    }
};
Configuration(function(ContextRegister) {
    ContextRegister.register('SynchronizeController', SynchronizeController);
});


// FILE: js/controllers/UnitController.js
var UnitController = function(ScreenService, UserService, SynchronizeService, localStorageEngine, templateManager, Environment, eventAggregator, restEngine, ZHPromise) {

    this.onEnter = function(widget) {

        widget.reload().then(function() {
            widget.moveToFirst();
        });

    };

    this.openUnitPopup = function() {
        var widgetUnit = templateManager.container.getWidget('WidgetUnit');
        widgetUnit.isVisible = true;
        widgetUnit.edit();
        widgetUnit.search = '';
        widgetUnit.dataSourceFilter = [{
            "name": "CDFILIAL",
            "operator": "<>",
            "value": "@getLocalVar('__CDFILIAL')@"
        }];
        widgetUnit.reload();
        ScreenService.openPopup(widgetUnit);
    };

    this.closeUnitPopup = function(widget) {
        widget.isVisible = false;
        ScreenService.closePopup();
    };

    eventAggregator.subscribe('afterMoveRowUnit', function(container) {
        afterMoveRow(container);
    });

    eventAggregator.subscribe('afterMoveRowUnitSync', function(container) {
        afterMoveRowSync(container);
    });

    eventAggregator.subscribe('openUnitPopup', function(widgetUnit) {
        if (widgetUnit !== undefined) {
            // var widgetUnit = container.getWidget('WidgetUnit');
            widgetUnit.isVisible = true;
            widgetUnit.edit();
            widgetUnit.search = '';
            widgetUnit.dataSourceFilter = [{
                "name": "CDFILIAL",
                "operator": "<>",
                "value": "@getLocalVar('__CDFILIAL')@"
            }];
            widgetUnit.reload();
            ScreenService.openPopup(widgetUnit);
        }
    });

    function afterMoveRow(container) {

        var message = ScreenService.i18n('Deseja confirmar a troca de unidade?');
        ScreenService.confirmMessage(message, 'question', function() {

            var widgetUnit = container.getWidget('WidgetUnit');
            widgetUnit.currentRow.changed = true;
            UserService.save(widgetUnit.currentRow).then(function() {
                SynchronizeService.sync(true).then(function() {

                    ScreenService.showMessage(ScreenService.i18n('Unidade alterada com sucesso!'), 'success');
                    widgetUnit.isVisible = false;
                    ScreenService.closePopup();
                    restEngine.post('/Patients/patientSequentialAjusta', {
                        "requestType": "Row",
                        "row": {
                            'unit': widgetUnit.currentRow.CDFILIAL
                        }
                    }).then(function(callback) {
                        if (callback && callback.dataset && callback.dataset.patientSequentialAjusta) {
                            if (callback.dataset.patientSequentialAjusta.IDCTRLCDPACI) {
                                localStorageEngine.setLocalVar('__ISSEQUENTIAL', callback.dataset.patientSequentialAjusta.IDCTRLCDPACI);
                            }
                            if (callback.dataset.patientSequentialAjusta.IDAJUSTAPACI) {
                                localStorageEngine.setLocalVar('__ISAJUSTA', callback.dataset.patientSequentialAjusta.IDAJUSTAPACI);
                            } else {
                                localStorageEngine.setLocalVar('__ISSEQUENTIAL', null);
                                localStorageEngine.setLocalVar('__ISAJUSTA', null);
                            }
                        }
                    });
                    setTimeout(function() {
                        templateManager.updateTemplate();
                        eventAggregator.publish("reloadScreenAndClearFilters", container.getWidget('WidgetMain'));
                        Environment.setLocalVar('__CDFILIAL', widgetUnit.currentRow.CDFILIAL);
                        Environment.setLocalVar('__NRORGTRAB', widgetUnit.currentRow.NRORG);
                        Environment.setLocalVar('__NMFILIAL', widgetUnit.currentRow.NMFILIAL);

                        var descriptions = widgetUnit.currentRow.CDFILIAL + ' - ' + widgetUnit.currentRow.NMFILIAL;
                        templateManager.userData.descriptions[0] = descriptions;
                        menu.closeMenu(); // jshint ignore:line
                    }, 600);
                });
            }, function() {
                ScreenService.showMessage(ScreenService.i18n("Erro ao alterar filial!"));
            });
        });
    }

    function afterMoveRowSync(container) {

        var message = ScreenService.i18n('Deseja confirmar a unidade selecionada?');
        ScreenService.confirmMessage(message, 'question', function() {
            var widgetUnit = container.getWidget('WidgetUnitSync');
            // widgetUnit.currentRow.changed = true;
            ScreenService.showLoader();
            localStorageEngine.getDataSource('/User').then(function(user) {
                user[0].CDFILIALSELECIONADA = widgetUnit.currentRow.CDFILIAL;
                user[0].CDFILIAL = widgetUnit.currentRow.CDFILIAL;
                restEngine.post('/User/save', {
                    "requestType": "DataSet",
                    "dataset": user
                }, function(success) {
                    var deferred = ZHPromise.defer();
                    if (success.length !== 0) {
                        ScreenService.showMessage(ScreenService.i18n("Erro ao alterar filial!"));
                        ScreenService.openWindow('Dashboard');
                        deferred.reject();
                        return deferred;
                    } else {
                        UserService.save(user).then(function() {
                            widgetUnit.isVisible = false;
                            ScreenService.closePopup();
                            eventAggregator.publish("wscCallUpdate", null);
                        }, function() {
                            ScreenService.showMessage(ScreenService.i18n("Erro ao alterar filial!"));
                            ScreenService.openWindow('Dashboard');
                        });
                    }
                }, undefined, function(error) {
                    console.log(error);
                    ScreenService.showMessage(ScreenService.i18n("Erro ao alterar filial!"));
                    ScreenService.openWindow('Dashboard');
                });
            }).finally(function() {
                ScreenService.hideLoader();
            });
        });
    }

};
Configuration(function(ContextRegister) {
    ContextRegister.register('UnitController', UnitController);
});

